/*
 *  Copyright (c) 2001, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client;

/**
 * an abstract class for DAVMethods that represent write operations
 * (PUT, MKCOL, POST, DELETE, PROPPATCH) that require If: headers
 * to operate on locked resources
 */
public abstract class AbstractWriteDAVMethod extends AbstractDAVMethod
{
    private IfHeader ifHeader=null;
    private String lockToken=null;

    public AbstractWriteDAVMethod(String path)
    {
	super(path);
    }
    
    public final IfHeader getIfHeader()
    {
	return this.ifHeader;
    }

    public final void setIfHeader(IfHeader ifHeader)
    {
	this.ifHeader=ifHeader;
    }

    public void processRequestHeaders()
    {
	if (ifHeader!=null && ! ifHeader.isEmpty())
	{
	    getRequestHeaders().put(DAVConstants.IF_HEADER, 
				    ifHeader.toString());
	}
    }
}

