/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client;

/**
 * various constants used throughout the dav client library.
 *
 * @author Jacob Smullyan
 */ 
public interface DAVConstants
{
    String CRLF="\r\n";
    
    //HTTP methods
    String GET_METHOD = "GET";
    String POST_METHOD = "POST";
    String HEAD_METHOD = "HEAD";
    String PUT_METHOD = "PUT";
    String DELETE_METHOD = "DELETE";
    String OPTIONS_METHOD = "OPTIONS";
    String TRACE_METHOD = "TRACE";
    //DAV extension methods
    String MOVE_METHOD = "MOVE";
    String MKCOL_METHOD = "MKCOL";
    String PROPFIND_METHOD = "PROPFIND";
    String PROPPATCH_METHOD = "PROPPATCH";
    String COPY_METHOD = "COPY";
    String LOCK_METHOD = "LOCK";
    String UNLOCK_METHOD = "UNLOCK";
    
    //headers for requests
    String OVERWRITE_HEADER = "Overwrite";
    String DESTINATION_HEADER = "Destination";
    String IF_HEADER = "If";
    String LOCK_TOKEN_HEADER = "Lock-Token";
    String DEPTH_HEADER = "Depth";
    String TIMEOUT_HEADER = "Timeout";
    String CONTENT_TYPE = "Content-Type";
    
    //headers for responses
    String ALLOW_HEADER = "Allow";
    
    //xml elements
    String ACTIVELOCK_ELEM = "activelock";
    String DEPTH_ELEM = "depth";
    String LOCKTOKEN_ELEM="locktoken";
    String TIMEOUT_ELEM = "timeout";
    String COLLECTION_ELEM = "collection";
    String HREF_ELEM = "href";
    String LINK_ELEM = "link";
    String DST_ELEM = "dst";
    String SRC_ELEM = "src";
    String LOCKENTRY_ELEM = "lockentry";
    String LOCKINFO_ELEM = "lockinfo";
    String LOCKSCOPE_ELEM = "lockscope";
    String EXCLUSIVE_ELEM = "exclusive";
    String SHARED_ELEM = "shared";
    String LOCKTYPE_ELEM = "locktype";
    String WRITE_ELEM = "write";
    String MULTISTATUS_ELEM = "multistatus";
    String RESPONSE_ELEM = "response";
    String PROPSTAT_ELEM = "propstat";
    String STATUS_ELEM = "status";
    String RESPONSEDESCRIPTION_ELEM = "responsedescription";
    String OWNER_ELEM = "owner";
    String PROP_ELEM = "prop";
    String PROPERTYBEHAVIOR_ELEM = "propertybehavior";
    String KEEPALIVE_ELEM = "keepalive";
    String OMIT_ELEM = "omit";
    String PROPERTYUPDATE_ELEM = "propertyupdate";
    String REMOVE_ELEM = "remove";
    String SET_ELEM = "set";
    String PROPFIND_ELEM = "propfind";
    String ALLPROP_ELEM = "allprop";
    String PROPNAME_ELEM = "propname";
    //DAV properties (also XML elements)
    String CREATIONDATE_PROP = "creationdate";
    String DISPLAYNAME_PROP = "displayname";
    String GETCONTENTLANGUAGE_PROP = "getcontentlanguage";
    String GETCONTENTLENGTH_PROP = "getcontentlength";
    String GETCONTENTTYPE_PROP = "getcontenttype";
    String GETETAG_PROP = "getetag";
    String GETLASTMODIFIED_PROP = "getlastmodified";
    String LOCKDISCOVERY_PROP = "lockdiscovery";
    String RESOURCETYPE_PROP = "resourcetype";
    String SOURCE_PROP = "source";
    String SUPPORTEDLOCK_PROP = "supportedlock";
    //Apache's custom executable property
    String EXECUTABLE_PROP="executable";
    //begin an xml document
    String XML_BOILERPLATE = "<?xml version=\"1.0\" ?>";
    String XML_NAMESPACE_ATTR = "xmlns";
    String DAV_NAMESPACE = "DAV:";
    String APACHE_DAV_NAMESPACE="http://apache.org/dav/props/";
    //The file separator used for a DAV virtual filesystem
    String DAV_FILE_SEPARATOR="/";
    //protocols
    String HTTP="http";
    String HTTPS="https";
}




/* $Log: DAVConstants.java,v $
/* Revision 1.13  2001/01/05 08:01:12  smulloni
/* changes to the connection gui for the Explorer; added depth configurability to
/* propfind in the jpython test script; added an experimental "allprop" system
/* property which affects the propfind query type
/*
/* Revision 1.12  2000/12/19 22:06:15  smulloni
/* adding documentation.
/*
/* Revision 1.11  2000/12/18 21:04:56  smulloni
/* changes to support SSL.
/*
/* Revision 1.10  2000/12/03 23:53:25  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.9  2000/11/09 23:34:50  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
