/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeSet;
import org.skunk.trace.Debug;

/**
 *  a wrapper around the results returned by a DAV propfind or other DAV method.
 *  It resembles a beanified ResultSet more than a File object; two DAVFile objects
 *  with the same host, port, and path may have quite different contents, depending
 *  on the sorts of queries that were used to generate it.
 *
 *  @author Jacob Smullyan
 */
public class DAVFile implements Comparable
{
    private ArrayList supportedLocks=null;
    private HashMap customProperties=null;
    private ArrayList availableProperties=null;
    private String name;
    private String etag;
    private String creationDate;
    private String lastModified;
    private String href;
    private String resourceType;
    private String contentType;
    private Long contentLength;
    private String contentLanguage;
    private ArrayList locks=null;
    private TreeSet children=null;
    private Integer status; 
    private String displayName;
    private ArrayList links=null;
    private String responseDescription;
    private String host;
    private int port;
    private String protocol=DAVConstants.HTTP;

    public DAVFile(String path)
    {
	// sachiho added; substitute &amp; to %26 in order to not conflict to url-re-encode
	String data=path;
	StringBuffer buf = new StringBuffer();
    for (int i = 0; i < data.length(); i++) {
        char c = data.charAt(i);
        switch (c) {
        case '&':
		if (data.substring(i+1,i+5).equals("amp;")){
            i += 4;
			buf.append('%');
			buf.append('2');
			buf.append('6');
		}else{buf.append(c);}
		break;
        default:
        buf.append(c);
        break;
        }
    }
	String buf2=new String(buf);
	this.name=buf2;
	//this.name=path;
    }

    public DAVFile(String host, int port, String filename)
    {
	this(filename); // sachiho
	this.host=host;
	this.port=port;
	//this.name=filename;
    }

    public DAVFile(String protocol, String host, int port, String path)
    {
	this(path);  // sachiho
	this.protocol=protocol;
	this.host=host;
	this.port=port;
	//this.name=path;
    }

    public String getName()
    {
	return name;
    }

    public String getFullName()
    {
	StringBuffer sb=new StringBuffer(host);
	sb.append(":");
	sb.append(port);
	sb.append(getName());
	return sb.toString();
    }

    public String getFullNameWithProtocol()
    {
	return new StringBuffer(getProtocol())
	    .append("://")
	    .append(getFullName())
	    .toString();
    }

    public String getProtocol()
    {
	return this.protocol;
    }

    public void setProtocol(String protocol)
    {
	this.protocol=protocol;
    }

    public String getFileName()
    {
	String shornName = (name.charAt(name.length()-1)=='/') 
	    ? name.substring(0, name.length()-1) 
	    : name;
	return shornName.substring(shornName.lastIndexOf("/")+1);
    }

	// sachiho added new method; URLdecode by hand and convert UTF-8 to java unicode
    public String getUnicodeFileName()
	{
	String data = getFileName();
	StringBuffer buf = new StringBuffer();
    for (int i = 0; i < data.length(); i++) {
        char c = data.charAt(i);
        switch (c) {

//        case '&':
//		if (data.substring(i+1,i+5).equals("amp;")){
//            i += 4;
//			buf.append('&');
//		}else{buf.append(c);}
//		break;
//        case '+':
//        buf.append(' ');
//        break;
        case '%':
        // XXX XXX 
        try {
            buf.append((char) Integer.parseInt(data.substring(i+1,i+3), 16));
            i += 2;
        } catch (NumberFormatException e) {
                    String msg = "Decode error ";
            // XXX no need to add sm just for that
            // sm.getString("serverRequest.urlDecode.nfe", data);

            throw new IllegalArgumentException(msg);
        } catch (StringIndexOutOfBoundsException e) {
            String rest  = data.substring(i);
            buf.append(rest);
            if (rest.length()==2)
            i++;
        }
        
        break;
        default:
        buf.append(c);
        break;
        }
    }

	String unibuf=null;
	try {
		String buf2 = new String(buf);
		unibuf = new String(buf2.getBytes("iso-8859-1"), "UTF-8");  //from UTF-8 to unicode
		// System.out.println(unibuf);
	} 
	catch (Exception e) {}

	return unibuf;
	}
	// sachiho added ܂



    public String getHost() 
    { 
	return host; 
    }

    public void setHost(String host) 
    { 
	this.host=host; 
    }

    public int getPort() 
    { 
	return port; 
    }

    public void setPort(int port) 
    { 
	this.port=port; 
    }

    public DAVConnection getDAVConnection()
    {
	return DAVConnectionPool.getDAVConnection(host, port);
    }

    public boolean isCollection()
    {
	return (resourceType!=null && resourceType.equals(DAVConstants.COLLECTION_ELEM));
    }

    public void addChild(DAVFile child)
    {
	if (children==null) children=new TreeSet();
	children.add(child);
    }

    public Iterator children()
    {
	if (children==null) children=new TreeSet();
	return children.iterator();
    }

    public int getChildCount()
    {
	return (children==null) ? 0 : children.size();
    }

    public int getChildCollectionCount()
    {
	if (children==null) return 0;
	Iterator kids=children.iterator();
	int cnt=0;
	while (kids.hasNext())
	{
	    DAVFile kid=(DAVFile)kids.next();
	    if (kid.isCollection()) cnt++;
	}
	return cnt;
    }

    public DAVFile getChildAt(int index)
    {
	if (children==null) return null;
	Iterator kids=children();
	for (int i=0;kids.hasNext();i++)
	{
	    Object o=kids.next();
	    if (i==index) return (DAVFile) o;
	}
	return null;
    }

    public DAVFile getChildNamed(String path)
    {
	if (path==null || children==null) return null;
	for (Iterator kids=children();kids.hasNext();)
	{
	    DAVFile kidFile=(DAVFile) kids.next();
	    if (path.equals(kidFile.getName()))
		return kidFile;
	}
	return null;
    }
	
    public String getResourceType()
    {
	return resourceType;
    }

    public void setResourceType(String resourceType)
    {
	this.resourceType=resourceType;
	//adjust name to reflect whether or not it is a collection
	if (isCollection() && !getName().endsWith("/"))
	    name+="/";	    
    }

    public boolean isLocked()
    {
	return (locks!=null && (!locks.isEmpty()));
    }

    public boolean isExclusiveLocked()
    {
	// I enforce that if a lock is exclusive, there can be only one of
	// them.
	return (isLocked() && locks.size()==1);   
    }

    public String getExclusiveLockToken()
    {
	if (isExclusiveLocked())
	    return getExclusiveLock().getLockToken();
	return null;
    }

    public Lock[] getLocks()
    {
	return getLockArray(locks);
    }

    public Lock getExclusiveLock()
    {
	if (isLocked() && locks.size()==1) 
	{
	    Lock soleLock=(Lock) locks.get(0);
	    if (soleLock.getLockScope().equals(LockScope.EXCLUSIVE))
		return soleLock;
	}
	return null;
    }

    public Lock getLock(String owner)
    {
	if (owner==null || !isLocked()) 
	    return null;
	if (isExclusiveLocked()) 
	    return getExclusiveLock();
	//must be a shared lock

	// N.B. there is a bug here; for some reason, 
	// the code below FAILS for shared locks, while
	// the loop below works.  Seem impossible to you?
	// Does to me, too.  
// 	for (Iterator it=locks.iterator();it.hasNext();)
// 	{
// 	    Lock nextLock=(Lock)it.next();
// 	    if (owner.equals(nextLock.getOwner()))
// 		return nextLock;
// 	}

	int numLocks=locks.size();
	for (int i=0;i<numLocks;i++)
	{
	    Lock lk=(Lock) locks.get(i);
	    if (owner.equals(lk.getOwner()))
		return lk;
	}
	       
	      
	return null;
    }
	

    public void setExclusiveLock(Lock lock)
    {
	//verify that the lock argument is exclusive
	if (lock==null || (!lock.getLockScope().equals(LockScope.EXCLUSIVE)))
	    throw new IllegalArgumentException("lock argument to setExclusiveLock() must be an exclusive lock");
	addLock(lock);
    }	

    public void addLock(Lock lock)
    {
	if (locks==null) locks=new ArrayList();
	if (lock==null) 
	{
	    Debug.trace(this, Debug.DP2, " ** ignoring null argument to addLock()");
	    return;
	}
	if (lock.getLockScope().equals(LockScope.EXCLUSIVE) && locks.size()>0) 
	{
	    Debug.trace(this, Debug.DP3, 
			"in setExclusiveLock() -- other locks {0} on resource exist, are being destroyed!!",
			locks);
	    locks.clear();
	}
	locks.add(lock);
    }

    public void removeLock(Lock lock)
    {
	if (locks!=null && locks.contains(lock))
	    locks.remove(locks.indexOf(lock));
    }

    public String getLastModified()
    {
	return lastModified;
    }

    public void setLastModified(String lastModified)
    {
	this.lastModified=lastModified;
    }

    public String getCreationDate()
    {
	return creationDate;
    }

    public void setCreationDate(String creationDate)
    {
	this.creationDate=creationDate;
    }

    public String getExclusiveLockOwner()
    {
	return (isExclusiveLocked()) ? getExclusiveLock().getOwner() : null;
    }

    public String getEtag()
    {
	return etag;
    }

    public void setEtag(String etag)
    {
	this.etag=etag;
    }

    public Long getContentLength()
    {
	return contentLength;
    }

    public void setContentLength(Long contentLength)
    {
	this.contentLength=contentLength;
    }

    public String getCustomProperty(DAVProperty  property)
    {
	Debug.trace(this, Debug.DP3, "getting custom property {0} from map {1}",
		    new Object[] { property, customProperties } );
	if (customProperties==null) return null;
	Object o=customProperties.get(property);
	return (o==null) ? null : o.toString();
    }

    public void setCustomProperty(DAVProperty property, String propertyValue)
    {
	if (customProperties==null) customProperties=new HashMap();
	customProperties.put(property, propertyValue);
    }

    public Map getCustomProperties()
    {
	return customProperties;
    }

    private Lock[] getLockArray(ArrayList lockList)
    {
	if (lockList==null) return null;
	Lock[] lockArray=new Lock[lockList.size()];
	for (int i=0;i<lockArray.length;i++)
	{
	    lockArray[i]=(Lock) lockList.get(i);
	}
	return lockArray;
    }
	    

    public Lock[] getSupportedLocks()
    {
	return getLockArray(supportedLocks);
    }

    public void addSupportedLock(Lock shlock)
    {
	if (supportedLocks==null) supportedLocks=new ArrayList();
	supportedLocks.add(shlock);
    }
    /**
     *  N.B. this only has meaning after a propname query.  An allprop or prop
     *  will not populate this structure and this method will then return null
     */
    public DAVProperty[] getAvailableProperties()
    {
	if (availableProperties==null)
	    return null;
	//I'm not synchronizing this, because I assume that once the available properties are
	//set, the results won't be touched. 
	DAVProperty[]  props=new DAVProperty[availableProperties.size()];
	for (int i=0;i<props.length;i++)
	{
	    props[i]=(DAVProperty)availableProperties.get(i);
	}
	return props;
    }

    public void addAvailableProperty(DAVProperty property)
    {
	if (availableProperties==null)
	    availableProperties=new ArrayList();
	availableProperties.add(property);
    }

    public Integer getStatus()
    {
	return status;
    }

    public void setStatus(Integer status)
    {
	this.status=status;
    }

    public String getDisplayName()
    {
	return displayName;
    }

    public void setDisplayName(String displayName)
    {
	this.displayName=displayName;
    }

    public String getContentLanguage()
    {
	return contentLanguage;
    }

    public void setContentLanguage(String contentLanguage)
    {
	this.contentLanguage=contentLanguage;
    }

    public String getContentType()
    {
	return contentType;
    }

    public void setContentType(String contentType)
    {
	this.contentType=contentType;
    }

    public Link[] getLinks()
    {
	if (links!=null)
	    return (Link[]) links.toArray();
	else return null;
    }

    public void addLink(Link l)
    {
	if (links==null) links=new ArrayList();
	links.add(l);
    }

    public String getResponseDescription()
    {
	return this.responseDescription;
    }

    public void setResponseDescription(String responseDescription)
    {
	this.responseDescription=responseDescription;
    }

    private static String flush(String s, int len)
    {
	StringBuffer sb=new StringBuffer(len);
	sb.append(s);
	for (int i=0;i<len-s.length();i++)
	    sb.append(' ');
	return sb.toString();
    }

    private static StringBuffer flush(String s, int len, StringBuffer sb)
    {
	int slen=s.length();
	for (int i=0;i<len-slen;i++)
	    sb.append(' ');
	return sb;
    }

    private static void display(String prompt, Object value, StringBuffer buffer)
    {
	if (value!=null) 
	{
	    buffer.append(prompt);
	    flush(":", 25-prompt.length(), buffer)
		.append(value.toString())
		.append("\n");
	}
    }

    public String toVerboseString()
    {
	StringBuffer sb=new StringBuffer();
	display("host", getHost(), sb);
	display("port", new Integer(getPort()), sb);
	display("resource name", getName(), sb);
	display("creation date", creationDate, sb);
	display("last modified", lastModified, sb);
	display("content type", contentType, sb);
	display("content length", contentLength, sb);
	display("content language", contentLanguage, sb);
	display("display name", displayName, sb);
	display("status", getStatus(), sb);
	display("resource type", resourceType, sb);
	display("etag", etag, sb);
	display("supported locks", supportedLocks, sb);
	display("children", children, sb);
	display("active locks", locks, sb);
	display("links", links, sb);
	display("custom properties", customProperties, sb);
	display("available properties", availableProperties, sb);
	display("response description", responseDescription, sb);
	return sb.toString();
    }

    public String toString()
    {
	StringBuffer sb=new StringBuffer("DAVFile [");
	sb.append(getName());
	sb.append(']');
	return sb.toString();
    }

    public int compareToDAVFile(DAVFile otherFile)
    {
	//first host, then port, then isCollection, then name
	String otherHost=otherFile.getHost();
	String thisHost=getHost();
	if (otherHost==null) otherHost="";
	if (thisHost==null) thisHost="";
	if (!otherHost.equals(thisHost))
	{
	    return thisHost.compareTo(otherHost);
	}
	int otherPort=otherFile.getPort();
	int thisPort=getPort();
	if (otherPort!=thisPort)
	{
	    return (new Integer(thisPort)).compareTo(new Integer(otherPort));
	}
	boolean coll1=isCollection();
	boolean coll2=otherFile.isCollection();
	if (coll1 && !coll2)
	    return -1;
	if (coll2 && !coll1)
	    return 1;
	return getName().compareTo(otherFile.getName());
    }

    public int compareTo(Object otherObject)
    {
	if (otherObject instanceof DAVFile)
	    return compareToDAVFile((DAVFile) otherObject);
	return getName().compareTo(otherObject.toString());
    }
}

/* $Log: DAVFile.java,v $
/* Revision 1.17  2001/07/18 20:20:53  smulloni
/* added IfHeader to LockMethod; fixed peculiar DAVFile bug (in getLock());
/* various fixes, new cases to auto.py
/*
/* Revision 1.16  2001/01/05 08:01:12  smulloni
/* changes to the connection gui for the Explorer; added depth configurability to
/* propfind in the jpython test script; added an experimental "allprop" system
/* property which affects the propfind query type
/*
/* Revision 1.15  2001/01/03 20:11:31  smulloni
/* the DAVFileChooser now replaces JFileChooser for remote file access.
/* DAVMethod now has a protocol property.
/*
/* Revision 1.14  2000/12/19 22:06:15  smulloni
/* adding documentation.
/*
/* Revision 1.13  2000/12/03 23:53:25  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.12  2000/12/01 16:25:50  smullyan
/* improvements to look and feel; fixed NPE in DAVFile; new actions for text
/* editor
/*
/* Revision 1.11  2000/11/20 23:30:18  smullyan
/* more editor integration work.
/*
/* Revision 1.10  2000/11/09 23:34:50  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
