/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client;

import java.util.Comparator;

public class DAVFileComparator implements Comparator
{
    public static final int NAME=0;
    public static final int LAST_MODIFIED=1;
    public static final int CREATION_DATE=2;
    private boolean ignoreCase=false;
    private boolean directoriesFirst=true;
    private int sortAttribute;

    public DAVFileComparator()
    {
	this(false, true, NAME);
    }

    public DAVFileComparator(boolean ignoreCase, boolean directoriesFirst, int sortAttribute)
    {
	this.ignoreCase=ignoreCase;
	this.directoriesFirst=directoriesFirst;
	this.sortAttribute=sortAttribute;
    }

    public int compare(Object object1, Object object2)
    {
	if ((object1 instanceof DAVFile) && (object2 instanceof DAVFile))
	{
	    if (object1.equals(object2))
		return 0;
	    
	    DAVFile file1=(DAVFile)object1;
	    DAVFile file2=(DAVFile)object2;

	    if (directoriesFirst)
	    {
		boolean coll1=file1.isCollection();
		boolean coll2=file2.isCollection();
		if (coll1 && !coll2)
		    return -1;
		if (coll2 && !coll1)
		    return 1;
	    }

	    if (ignoreCase)
	    {
		return file1.getName().compareToIgnoreCase(file2.getName());
	    }
	    else
	    {
		return file1.getName().compareTo(file2.getName());
	    }
	}
	throw new IllegalArgumentException("arguments must be instances of DAVFile");	    
    }
}

/* $Log: DAVFileComparator.java,v $
/* Revision 1.4  2000/12/19 22:06:15  smulloni
/* adding documentation.
/*
/* Revision 1.3  2000/12/03 23:53:25  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.2  2000/11/09 23:34:51  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
