/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/


package org.skunk.dav.client;

import java.util.Map;
import org.skunk.minixml.MalformedXMLException;

public interface DAVMethod
{
    /**
     * a hook for processing the request headers
     */
    void processRequestHeaders();
    
    /**
     * a hook for processing the request body.
     */
    void processRequestBody();
    
    /**
     * a hook for processing the response headers.
     */
    void processResponseHeaders();
    
    /**
     * a hook into processing the body of the response. 
     * @throws MalformedXMLException if an XML response is expected
     * and it is not well-formed.
     */
    void processResponseBody() throws MalformedXMLException;
    
    /**
     * @return the DAVFile corresponding to the resourceURL.
     */
    DAVFile getDAVFile();

    /**
     * @return the headers of the response
     */
    Map getResponseHeaders();
    
    /**
     * used to populate the headers of the response
     * @param headers a mapping of headers names to values
     */
    void setResponseHeaders(Map m);
    
    /**
     *  returns the body of the response
     * @return the body of the response
     */
    byte[] getResponseBody();
    
    /**
     * sets the body of the request.
     * @param b the new request body
     */
    void setResponseBody(byte[] body);
    
    /**
     * @return the status of the response
     */    
    int getStatus();
    
    /**
     * used to the populate the status of the response
     * @param status the status 
     */    
    void setStatus(int status);
    
    /**
     * gets the headers of the request
     * @return the request headers
     */   
    Map getRequestHeaders();
    
    /**
     * sets the headers of the request 
     * @param m a map of header names and values
     */   
    void setRequestHeaders(Map headers);
    
    /**
     * returns the method name
     * @return the DAVMethodName of the method
     */
    DAVMethodName getRequestMethodName();
    
    /**
     * returns the path requested
     * @return the path requested.
     */    
    String getRequestURL();
    
    /**
     * returns the body of the request
     * @return the body of the request, if any, or null
     */   
    byte[] getRequestBody();

    /**
     * sets the body of the request.
     * @param body the new request body
     */    
    void setRequestBody(byte[] body);

    /**
     * sets the hostname
     * @param host the hostname
     */
    void setHost(String host);

    /**
     * sets the port
     * @param port the port
     */
    void setPort(int port);

    /**
     * sets the protocol
     * @param protocol the protocol
     */
    void setProtocol(String protocol);
}

/* $Log: DAVMethod.java,v $
/* Revision 1.11  2001/01/03 22:51:53  smulloni
/* added documentation
/*
/* Revision 1.10  2001/01/03 20:11:31  smulloni
/* the DAVFileChooser now replaces JFileChooser for remote file access.
/* DAVMethod now has a protocol property.
/*
/* Revision 1.9  2000/12/19 22:06:15  smulloni
/* adding documentation.
/*
/* Revision 1.8  2000/12/03 23:53:25  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.7  2000/11/09 23:34:51  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
