/*
 *  $Id: DAVProperty.java,v 1.12 2001/06/05 16:30:20 smulloni Exp $
 *  Time-stamp: <01/06/05 12:25:39 smulloni>
 *
 *  Copyright (c) 2000-2001, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client;

import org.skunk.minixml.XMLElement;
import org.skunk.trace.Debug;

/**
 * DAVProperty.java
 * convenience for adding named properties to a propfind
 *
 * Created: Mon Sep 18 13:51:18 2000
 *
 * @author <a href="mailto:jacob.smullyan@smullyan.org"</a>
 * @version $Revision: 1.12 $
 */

public class DAVProperty extends XMLElement
{
    public static final DAVProperty LOCKDISCOVERY=getCanonicalProperty(DAVConstants.LOCKDISCOVERY_PROP);  
    public static final DAVProperty CREATIONDATE=getCanonicalProperty(DAVConstants.CREATIONDATE_PROP);
    public static final DAVProperty DISPLAYNAME=getCanonicalProperty(DAVConstants.DISPLAYNAME_PROP);
    public static final DAVProperty GETCONTENTLANGUAGE=getCanonicalProperty(DAVConstants.GETCONTENTLANGUAGE_PROP);
    public static final DAVProperty GETCONTENTLENGTH=getCanonicalProperty(DAVConstants.GETCONTENTLENGTH_PROP);
    public static final DAVProperty GETCONTENTTYPE=getCanonicalProperty(DAVConstants.GETCONTENTTYPE_PROP);
    public static final DAVProperty GETETAG=getCanonicalProperty(DAVConstants.GETETAG_PROP);
    public static final DAVProperty GETLASTMODIFIED=getCanonicalProperty(DAVConstants.GETLASTMODIFIED_PROP);
    public static final DAVProperty RESOURCETYPE=getCanonicalProperty(DAVConstants.RESOURCETYPE_PROP);
    public static final DAVProperty SOURCE=getCanonicalProperty(DAVConstants.SOURCE_PROP);
    public static final DAVProperty SUPPORTEDLOCK=getCanonicalProperty(DAVConstants.SUPPORTEDLOCK_PROP);
    public static final DAVProperty EXECUTABLE=new DAVProperty(DAVConstants.EXECUTABLE_PROP, 
								 "apache", 
								 DAVConstants.APACHE_DAV_NAMESPACE);

    public static final DAVProperty[] BASIC_PROPERTIES=
    {
	LOCKDISCOVERY,
	CREATIONDATE,
	DISPLAYNAME,
	//GETCONTENTLANGUAGE,
	GETCONTENTLENGTH,
	GETCONTENTTYPE,
	//GETETAG,
	GETLASTMODIFIED,
	RESOURCETYPE,
	//SOURCE,
	//SUPPORTEDLOCK
    };
    

    public DAVProperty (String propertyName)
    {
	super(propertyName);
    }

    public DAVProperty(String propertyName, String namespaceCode, String namespace)
    {
	super(propertyName, namespaceCode, namespace);
    }
    /**
     * 
     */
    public DAVProperty(XMLElement elem)
    {
	super(elem.getElementName(), 
	      elem.getElementNamespaceCode(), 
	      elem.getNamespace(elem.getElementNamespaceCode()));
    }

    public static DAVProperty getCanonicalProperty(String canonicalName)
    {
	return new DAVProperty(canonicalName, "skunk", DAVConstants.DAV_NAMESPACE);
    }  

    public boolean equals(Object otherObject)
    {
	Debug.trace(this, Debug.DP6, "comparing {0} and {1}", 
		    new Object[] {this, otherObject});
		
	if (otherObject instanceof DAVProperty)
	{
	    DAVProperty otherProperty=(DAVProperty) otherObject;
	    String namespace=getNamespace();
	    String otherNamespace=otherProperty.getNamespace();
	    String elementName=getElementName();
	    String otherElementName=otherProperty.getElementName();

	    // fix for possible NullPointerException if namespace is null, pointed out
            // by Mustafa Cakar (Mustafa.Cakar@germany.sun.com), from whose patch
	    // this code is derived.  This differs from his patch in that I don't currently 
	    // prevent a NullPointerException if the element name is null.

	    if (elementName.equals(otherElementName))
	    {
		if (namespace==null && otherNamespace == null)
		{
		    return true;
		}
		else if (namespace != null )
		{
		    return namespace.equals(otherNamespace);
		}										
	    }
	}
	return false;
}

    public int hashCode()
    {
	return (getElementName()+getNamespace()).hashCode();
    }
	
}

/* $Log: DAVProperty.java,v $
/* Revision 1.12  2001/06/05 16:30:20  smulloni
/* incorporated fix from Mustafa Cakar for NullPointerException in
/* DAVProperty.equals().
/*
/* Revision 1.11  2001/01/05 08:01:12  smulloni
/* changes to the connection gui for the Explorer; added depth configurability to
/* propfind in the jpython test script; added an experimental "allprop" system
/* property which affects the propfind query type
/*
/* Revision 1.10  2000/12/19 22:06:15  smulloni
/* adding documentation.
/*
/* Revision 1.9  2000/12/03 23:53:25  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.8  2000/11/14 23:18:31  smullyan
/* fix for responsedescription property in multistatus
/*
/* Revision 1.7  2000/11/13 23:28:29  smullyan
/* first pass at a gui for proppatch, added to the propertiese dialog.  Highly
/* problematic still, needs substantial work.
/*
/* Revision 1.6  2000/11/09 23:34:51  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
