/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/


package org.skunk.dav.client;

import java.util.*;

/**
 * a HashMap subclass in which keys, if they are strings,
 * are coerced into uppercase.
 */
public class HeaderMap extends HashMap
{

  public HeaderMap()
  {
    super();
  }

  public Object put(Object key, Object value)
  {
    //disallow nulls as keys or values, to save bother
    if (key==null || value==null) return null;
    return super.put(getUpperCaseObject(key), value);
  }

  public Object get(Object key)
  {
    return super.get(getUpperCaseObject(key));
  }

  private Object getUpperCaseObject(Object o)
  {
    if (o instanceof String)
    {
      return ((String)o).toUpperCase().trim();
    }
    return o;
  }

  public boolean containsKey(Object key)
  {
    return super.containsKey(getUpperCaseObject(key));
  }


  public Object remove(Object key)
  {
    return super.remove(getUpperCaseObject(key));
  }

  public String toString()
  {
    StringBuffer sb=new StringBuffer();
    Set daSet=keySet();
    for (Iterator it=daSet.iterator();it.hasNext();)
    {
      Object key=it.next();
      sb.append(key);
      sb.append(": ");
      sb.append(get(key));
      sb.append(DAVConstants.CRLF);
    }
    return sb.toString();
  }

  public String getHeader(String headerKey)
  {
    if (containsKey(headerKey))
       return get(headerKey).toString();
    return null;
  }

  //convenience function
  public static String getHeader(String headerKey, Map headerMap)
  {
    if (headerMap instanceof HeaderMap)
       return ((HeaderMap) headerMap).getHeader(headerKey);
    return null;
  }
}

/* $Log: HeaderMap.java,v $
/* Revision 1.7  2000/12/19 22:06:15  smulloni
/* adding documentation.
/*
/* Revision 1.6  2000/12/03 23:53:25  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.5  2000/11/09 23:34:52  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
