/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client;

/**
 *  a struct that stores information pertaining to a lock.
 */
public class Lock
{
    private LockScope lockScope;
    private LockType lockType;
    private Depth depth;
    private Timeout timeout;
    private String lockToken;
    private String owner;

    public Lock(LockScope lockScope, 
		LockType lockType, 
		Depth depth, 
		Timeout timeout, 
		String lockToken, 
		String owner)
    {
	this.lockScope=lockScope;
	this.lockType=lockType;
	this.depth=depth;
	this.timeout=timeout;
	this.lockToken=lockToken;
	this.owner=owner;
    }

    public Lock(LockScope lockScope, LockType lockType)
    {
	this(lockScope, lockType, null, null, null, null);
    }

    public LockScope getLockScope()
    {
	return lockScope;
    }

    public LockType getLockType()
    {
	return lockType;
    }

    public Depth getDepth()
    {
	return depth;
    }

    public Timeout getTimeout()
    {
	return timeout;
    }

    public String getLockToken()
    {
	return lockToken;
    }

    public String getOwner()
    {
	return owner;
    }

    public String toString()
    {
	StringBuffer sb=new StringBuffer("Lock [scope=");
	sb.append(lockScope);
	sb.append(", type=");
	sb.append(lockType);
	if (depth!=null)
	{
	    sb.append(", depth=");
	    sb.append(depth);
	}
	if (timeout!=null)
	{
	    sb.append(", timeout=");
	    sb.append(timeout);
	}
	if (lockToken!=null)
	{
	    sb.append(", lockToken=");
	    sb.append(lockToken);
	}
	if (owner!=null)
	{
	    sb.append(", owner=");
	    sb.append(owner);
	}
	sb.append(']');
	return sb.toString();
    }
	
}


/* $Log: Lock.java,v $
/* Revision 1.5  2000/12/19 22:06:15  smulloni
/* adding documentation.
/*
/* Revision 1.4  2000/12/03 23:53:25  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.3  2000/11/09 23:34:53  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
