/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/


package org.skunk.dav.client;

import java.util.ArrayList;
import java.util.ListIterator;
import java.util.StringTokenizer;
import org.skunk.trace.Debug;

/**
 *  not fully compliant with RFC2518:
 *  currently only supports Infinite and one Second value.
 *  may be extended to support TimeTypes of "Other" as per
 *  RFC2518, section 9.8.
 */
public class Timeout
{
    public static final long MAX_SECONDS=Math.round(Math.pow(2.0, 32.0)) - 1;
    private final static String _INFINITE_VAL="Infinite";
    private final static String _SECOND_VAL="Second-";
    
    private ArrayList timeouts=new ArrayList();
    
    public Timeout(Long timeout, boolean isInfinite)
    {
	if ((timeout!=null)
	    && (timeout.longValue()>MAX_SECONDS))
	    throw new IllegalArgumentException("maximum timeout exceeded");
	if ((timeout==null) && ! isInfinite)
	    throw new IllegalArgumentException("must specify a timeout");
	if (isInfinite)
	    timeouts.add(this._INFINITE_VAL);
	if (timeout!=null) timeouts.add(timeout);
    }
    
    public static Timeout getTimeout(String timeoutStr)
    {
	boolean isInfinite=false;
	Long timeout=null;
	StringTokenizer st=new StringTokenizer(timeoutStr, ":, ");
	while (st.hasMoreTokens())
	{
	    String tok=st.nextToken().trim();
	    if (tok.equals(DAVConstants.TIMEOUT_HEADER))
		continue; 
	    else if ((!isInfinite) && tok.equals(_INFINITE_VAL))
		isInfinite=true;
	    else if ((timeout==null) && tok.equals(_SECOND_VAL))
	    {
		if (st.hasMoreTokens())
		{
		    String longStr=st.nextToken();
		    try
		    {
			timeout=Long.decode(longStr);
		    }
		    catch (NumberFormatException nafta)
		    {
			Debug.trace(Timeout.class, Debug.DP3, nafta);
		    }
		}
	    }
	}
	Timeout t=null;
	try
	{
	    t= new Timeout(timeout, isInfinite);
	}
	finally
	{
	    return t;
	}
    }
    
    public String toString()
    {
	StringBuffer sb=new StringBuffer(DAVConstants.TIMEOUT_HEADER);
	sb.append(": ");
	for (ListIterator li=timeouts.listIterator();li.hasNext();)
	{
	    if (li.nextIndex()>0)
		sb.append(", ");
	    Object nextTimeout=li.next();
	    if (nextTimeout instanceof Long)
		sb.append(_SECOND_VAL);
	    sb.append(nextTimeout);
	}
	return sb.toString();
    }
    
}

/* $Log: Timeout.java,v $
/* Revision 1.5  2000/12/19 22:06:15  smulloni
/* adding documentation.
/*
/* Revision 1.4  2000/12/03 23:53:25  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.3  2000/11/09 23:34:54  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
