/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import java.util.Iterator;
import org.skunk.config.Configurator;

/**
 * an abstraction layer for application-level objects
 *
 * @author Jacob Smullyan
 */

public interface AppContext
{
    /**
     * adds a view
     */
    void addView(View view);

    /**
     * removes a view
     * Precondition: appContext contains this view
     */
    void removeView(View view);

    /**
     * @return an iterator of all Views in the AppContext
     */
    Iterator views();

    /**
     * @return the number of current views
     */
    int getViewCount();

    /**
     * @return the view at the given index
     */
    View getView(int index);

    /**
     * @return the currently active (focussed) view
     */
    View getCurrentView();

    /**
     * @return the dock mode (interface style)
     */
    DockMode getDockMode();

    /**
     *  @return the configurator object for the application
     */
    Configurator getConfigurator();

    /**
     *  @return whether or not the application is trusted to access the filesystem,
     *          open sockets to multiple hosts, etc.
     */
    boolean isTrusted();      //a more fine-grained indicator would be ... fancier.
}

/* $Log: AppContext.java,v $
/* Revision 1.6  2000/12/19 22:06:15  smulloni
/* adding documentation.
/* */
