/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import org.skunk.trace.Debug;

import java.util.Collections;
import java.util.ArrayList;
import java.util.List;

public class BusinessTracker
{
    private static List busyKeys=Collections.synchronizedList(new ArrayList());

    public static void addBusy(Object key)
    {
	Debug.trace(BusinessTracker.class, Debug.DP4, "adding busy object: {0}", key);
	if (!busyKeys.contains(key))
	    busyKeys.add(key);
	if (busyKeys.size()==1)
	    publishBusyState(true);
    }

    public static void removeBusy(Object key)
    {
	Debug.trace(BusinessTracker.class, Debug.DP4, "removing busy object: {0}", key);
	if (busyKeys.contains(key)) 
	    busyKeys.remove(key);
	if (busyKeys.isEmpty())
	    publishBusyState(false);
	    
    }

    public static void removeAllBusy()
    {
	Debug.trace(BusinessTracker.class, Debug.DP4, "clearing all busy objects");
	busyKeys.clear();
	publishBusyState(false);
	
    }

    private static void publishBusyState(boolean busy)
    {
	Debug.trace(BusinessTracker.class, Debug.DP3, "publishing busy state: "+busy);
	StateMonitor.setProperty(StateProperties.BUSY, 
				 new Boolean(busy), 
				 null);
    }
}

/* $Log: BusinessTracker.java,v $
/* Revision 1.3  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.2  2000/12/14 06:36:25  smulloni
/* changes to search and replace in text editor.
/* */
	
    
