/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import java.awt.Event;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import org.skunk.dav.client.gui.action.AddConnectionAction;
import org.skunk.dav.client.gui.action.ConnectionAction;
import org.skunk.dav.client.gui.action.RemoveConnectionAction;


public class ConnectMenu extends JMenu
{
    private Explorer explorer;
    private int displayLimit=10;

    public ConnectMenu(Explorer explorer)
    {
	super();
	this.explorer=explorer;
	String label=ResourceManager.getMessage(ResourceManager.CONNECT_PREVIOUS_KEY);
	setText(label);
	ServerData.init();
	initComponents();
    }

    public ConnectMenu(String text)
    {
	super(text);
	initComponents();
    }

    public ConnectMenu(String text, boolean tearOff)
    {
	super(text, tearOff);
	initComponents();
    }

    public void refresh()
    {
	removeAll();
	initComponents();
	revalidate();
	repaint();
    }

    public int getDisplayLimit()
    {
	return this.displayLimit;
    }

    public void setDisplayLimit(int displayLimit)
    {
	this.displayLimit=displayLimit;
    }

    private void initComponents()
    {
	int incr=0;
        for (Enumeration enum=((Vector)ServerData.getServers().clone()).elements(); 
	     incr<displayLimit && enum.hasMoreElements();
	     incr++)
        {
	    ServerData sd=(ServerData) enum.nextElement();
	    final String host=sd.getHost();
	    final int port=sd.getPort();
	    final String initialPath=sd.getInitialPath();
	    JMenuItem item=new JMenuItem(sd.toString());
	    item.addActionListener(new ConnectionAction(explorer, sd));
	    item.setActionCommand(ConnectionAction.CONNECT_COMMAND);
// 	    if (functionKeyIncrement<=9)
// 	    {
// 		KeyStroke ks=KeyStroke.getKeyStroke(KeyEvent.VK_F1 
// 						    + (functionKeyIncrement++), 
// 						    Event.ALT_MASK);
// 		item.setAccelerator(ks);
// 	    }
	    this.add(item);
        }
	
        JMenuItem addItem=new JMenuItem(AddConnectionAction.ADD_CONNECTION_LABEL);
	addItem.addActionListener(new AddConnectionAction());
//         addItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A,
// 						      Event.ALT_MASK));
        this.add(addItem);
	
        JMenuItem removeItem=new JMenuItem(RemoveConnectionAction.REMOVE_CONNECTION_LABEL);
	removeItem.addActionListener(new RemoveConnectionAction(explorer));
//         removeItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R,
// 							 Event.ALT_MASK));
        this.add(removeItem);
    }
}

/* $Log: ConnectMenu.java,v $
/* Revision 1.9  2001/01/05 08:01:12  smulloni
/* changes to the connection gui for the Explorer; added depth configurability to
/* propfind in the jpython test script; added an experimental "allprop" system
/* property which affects the propfind query type
/*
/* Revision 1.8  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.7  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.6  2000/11/09 23:34:54  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
