/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.IOException;
import java.net.PasswordAuthentication;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane; 
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import org.skunk.dav.client.DAVAuthenticator;
import org.skunk.trace.Debug;

public class DAVAuthenticatorImpl extends DAVAuthenticator
{
    public static final String LOGIN_TITLE=ResourceManager.getMessage(ResourceManager.LOGIN_TITLE);
    private String loginMessage, username, password;
    private boolean rememberPassword;
    private boolean testedOnce=false;
    private ServerData sd;
    //private boolean showRemember=true;

    public DAVAuthenticatorImpl()
    {
	loginMessage=LOGIN_TITLE;
    }

    public DAVAuthenticatorImpl(String host, int port, String initialPath)
    {
	//mea culpa -- this needs to be internationalized
	loginMessage=ResourceManager.getMessage(ResourceManager.LOGIN_MESSAGE, new Object[] {host+":"+port+initialPath}); 
	//"Login to "+host+":"+port+initialPath;
	username=null;
	password=null;
	rememberPassword=false;
    }

    public DAVAuthenticatorImpl(ServerData sd)
    {
	this(sd.getHost(), sd.getPort(), sd.getInitialPath());
	username=sd.getUsername();
	rememberPassword=sd.getRememberPassword();
	password=rememberPassword ? sd.getPassword() : null;
	this.sd=sd;
    }

//     public DAVAuthenticatorImpl(String host, int port, String initialPath, boolean showRemember)
//     {
// 	this(host, port, initialPath);
// 	this.showRemember=false;
//     }
	

    public void reset() {testedOnce=false;}

    protected PasswordAuthentication getPasswordAuthentication()
    {	
	if (username==null || password==null 
	    || username.equals("") || password.equals("")
	    || !testedOnce)
	{
	    AuthenticatorPanel messagePanel=new AuthenticatorPanel(username, 
								   password, 
								   rememberPassword);
// 	    if (!showRemember)
// 		messagePanel.setShowRemember(false);
	    int result=JOptionPane.showConfirmDialog(Explorer.guessActiveExplorer(), 
						     messagePanel,		
						     getLoginMessage(),        
						     JOptionPane.OK_CANCEL_OPTION,
						     JOptionPane.PLAIN_MESSAGE);
	    if (result==JOptionPane.OK_OPTION)
	    {
		String oldUsername=username;
		username=messagePanel.getUsername();
		String oldPassword=password;
		password=messagePanel.getPassword();
		boolean oldRememberPassword=rememberPassword;
		rememberPassword=messagePanel.getRememberPassword();
		boolean changed=false;
		if (sd!=null)
		{
		    if (oldUsername!=null 
			&& !oldUsername.equals(username))
		    {
			sd.setUsername(username);
			changed=true;
		    }
		    if (rememberPassword) 
		    {
			if (! oldRememberPassword)
			{
			    sd.setRememberPassword(true);
			    changed=true;
			}
			if (oldPassword!=null
			    && !oldPassword.equals(password))
			{
			    sd.setPassword(password);
			    changed=true;
			}
		    }
		    else
		    {
			if (oldRememberPassword)
			{
			    sd.setRememberPassword(false);
			    changed=true;
			}
			if (oldPassword!=null)
			{
			    sd.setPassword(null);
			    changed=true;
			}
		    }
		}
		if (changed)
		{
		    try
		    {
			ServerData.saveServers();
		    }
		    catch (IOException oyVeh)
		    {
			Debug.trace(this, Debug.DP2, oyVeh);
		    }
		}
	    }
	    else return null;
	}
	testedOnce=true;
	return new PasswordAuthentication(username, password.toCharArray());
    }

    protected String getLoginMessage()
    {
	return loginMessage;
    }
    
    
    class AuthenticatorPanel extends JPanel
    {
	JTextField usernameField=null;
	JTextField passwordField=null;
	JCheckBox rememberBox=null;
	String username, password;
	boolean rememberPassword;
	final String USERNAME_PROMPT=ResourceManager.getMessage(ResourceManager.USERNAME_PROMPT);
	final String PASSWORD_PROMPT=ResourceManager.getMessage(ResourceManager.PASSWORD_PROMPT);
	final String REMEMBER_PASSWORD_PROMPT=ResourceManager.getMessage(ResourceManager.REMEMBER_PASSWORD_PROMPT);
	//making it possible to turn off the rememberbox
	//boolean showRemember=true;

	AuthenticatorPanel(String username, String password, boolean rememberPassword)
	{
	    super();
	    this.username=username;
	    this.password=password;
	    this.rememberPassword=rememberPassword;
	    initComponents();
	}

	AuthenticatorPanel(String username, String password)
	{
	    this(username, password, true);
	}
	
// 	void setShowRemember(boolean showRemember)
// 	{
// 	    this.showRemember=showRemember;
// 	}

	private void initComponents()
	{
	    JLabel usernameLbl=new JLabel(USERNAME_PROMPT);
	    JLabel passwordLbl=new JLabel(PASSWORD_PROMPT);
	    
	    usernameField=new JTextField(18);
	    if (username!=null) usernameField.setText(username);
	    passwordField=new JPasswordField(18);
	    if (password!=null) passwordField.setText(password);
	    rememberBox=new JCheckBox(REMEMBER_PASSWORD_PROMPT, this.rememberPassword);
	    
	    setLayout(new GridBagLayout());
	    GridBagConstraints gbc=new GridBagConstraints();
	    gbc.gridx=0;
	    gbc.gridy=0;
	    gbc.gridwidth=1;
	    gbc.gridheight=1;
	    gbc.weightx=0.1;
	    gbc.insets=new Insets(2, 2, 2, 2);
	    gbc.fill=gbc.NONE;
	    gbc.ipadx=1;
	    gbc.ipady=1;
	    add(usernameLbl, gbc);
	    gbc.gridx++;
	    gbc.weightx=0.5;
	    add(usernameField, gbc);
	    gbc.weightx=0;
	    gbc.gridx=0;
	    gbc.gridy++;
	    add(passwordLbl, gbc);
	    gbc.gridx++;
	    add(passwordField, gbc);
	    gbc.gridx=0;
	    gbc.gridy++;
	    gbc.gridwidth=2;
	    gbc.anchor=GridBagConstraints.CENTER;
	    //if (showRemember)
	    add(rememberBox, gbc);
	    
	}

	String getUsername()
	{
	    if (usernameField!=null)
		return usernameField.getText().trim();
	    return new String();
	}

	String getPassword()
	{
	    if (passwordField!=null)
		return passwordField.getText().trim();
	    return new String();
	}

	boolean getRememberPassword()
	{
	    if (rememberBox!=null)
		return rememberBox.isSelected();
	    return false;
	}
    }

	
}
