/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import java.awt.Component;
import java.awt.Font;
import java.awt.FontMetrics;
import java.util.StringTokenizer;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;
import org.skunk.dav.client.DAVConstants;
import org.skunk.swing.TreeNodeChooser;
import org.skunk.trace.Debug;

//for test
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.JOptionPane;
import org.skunk.config.Configurator;
import org.skunk.config.InMemoryConfigStore;

public class DAVFileChooser extends TreeNodeChooser
{
    //this shouldn't be hard-coded -- TO BE DONE
    private int maximumLabelLength=320;

    public DAVFileChooser(DAVTreeModel model)
    {
	super(model);
	setRootVisible(false);
	setNodeSeparator(DAVConstants.DAV_FILE_SEPARATOR);
	setComboBoxCellRenderer(new DAVFileComboRenderer());
    }

    protected int getMaximumLabelLength()
    {
	return maximumLabelLength;
    }

    protected void setMaximumLabelLength(int maximumLabelLength)
    {
	this.maximumLabelLength=maximumLabelLength;
    }

    private class DAVFileComboRenderer extends DefaultListCellRenderer
    {
	public Component getListCellRendererComponent(JList list,
						      Object value,
						      int index,
						      boolean isSelected,
						      boolean cellHasFocus)
	{
	    String formattedValue=format(value).toString();
	    Component c=super.getListCellRendererComponent(list,
						      formattedValue,
						      index,
						      isSelected,
						      cellHasFocus);
	    //Debug.trace(this, Debug.DP5, "renderer preferred size: "+c.getPreferredSize());
	    Font daFont=c.getFont();
	    FontMetrics daMetrics= c.getFontMetrics(daFont);
	    int diff=daMetrics.stringWidth(formattedValue) - maximumLabelLength;
	    if (diff>=0)
	    {
		int garbage=1; //trash this many tokens
		while (diff>=0)
		{
		    StringBuffer formatBuffer=new StringBuffer(".../");
		    StringTokenizer st=new StringTokenizer(formattedValue, DAVConstants.DAV_FILE_SEPARATOR);
		    for (int i=0;st.hasMoreTokens();i++)
		    {
			String thing=st.nextToken();
			if (i<=garbage)
			    continue;
			formatBuffer.append(thing)
			    .append(DAVConstants.DAV_FILE_SEPARATOR);
		    }
		    garbage++;
		    formattedValue=formatBuffer.toString();
		    diff=daMetrics.stringWidth(formattedValue) - maximumLabelLength;
		}
		return super.getListCellRendererComponent(list,
							  formattedValue,
							  index,
							  isSelected,
							  cellHasFocus);
	    }
	    return c;
	}

	private Object format(Object value)
	{
	    if (value instanceof DAVTreeNode)
		return ((DAVTreeNode)value).getDAVFile().getFullName();
	    if (value ==null) return "";
	    return value;
	}
    }

    public ChoiceStruct getChoiceStruct()
    {
	Object lastPathComponent=getSelectedPath().getLastPathComponent();
	if (lastPathComponent==null || !(lastPathComponent instanceof DAVTreeNode))
	{
	    return null;
	}
	DAVTreeNode parentNode=(DAVTreeNode) lastPathComponent;
	Object leafNode=getSelectedLeaf();
	
	return new ChoiceStruct(parentNode, 
				getEntryFieldText(), 
				((leafNode !=null && leafNode instanceof DAVTreeNode)
				 ? (DAVTreeNode) leafNode
				 : null));
    }
					     

    public static class ChoiceStruct
    {
	private DAVTreeNode parentNode;
	private String entryText;
	private DAVTreeNode selectedLeaf;
	private String newPath;

	private ChoiceStruct(DAVTreeNode parentNode, String entryText, DAVTreeNode selectedLeaf)
	{
	    this.parentNode=parentNode;
	    this.entryText=entryText;
	    this.selectedLeaf=selectedLeaf;
	    setNewPath();
	}

	public DAVTreeNode getParentNode()
	{
	    return this.parentNode;
	}

	public void setParentNode(DAVTreeNode parentNode)
	{
	    this.parentNode=parentNode;
	}

	public String getFilename()
	{
	    return this.entryText;
	}

	public String getFilePath()
	{
	    return this.newPath;
	}

	public DAVTreeNode getSelectedLeaf()
	{
	    return this.selectedLeaf;
	}

	private void setNewPath()
	{	    
	    if (parentNode!=null)
	    {
		StringBuffer sb=new StringBuffer(parentNode.getDAVFile().getFullName());
		if (entryText!=null)
		    sb.append(entryText);
		newPath=sb.toString();
	    }
	    else newPath=null;
	}
	
    }

    public static class Test
    {
	public static void main(String[] args)
	{
	    DefaultMutableTreeNode rootNode=new DefaultMutableTreeNode("oink");
	    DAVTreeModel model=new DAVTreeModel(rootNode, true);
	    //throwaway configurator, just to get the ServerData class not to barf
	    Configurator.setConfigurator(new Configurator(new InMemoryConfigStore()));
	    ServerData sd=new ServerData("support.skunk.org", 8888, "/", "jsmullyan", "");
	    model.addConnectionNode(sd, null);
	    //let it load
	    while (model.getChildCount(rootNode)<1)
	    {
		try
		{
		    Thread.sleep(333);
		}
		catch (InterruptedException ie)
		{
		}
	    }
	    DAVFileChooser chooser=new DAVFileChooser(model);
	    if (args.length>0 && args[0].equals("visible"))
		chooser.setRootVisible(true);
	    chooser.setCurrentPath(new TreePath(model.getPathToRoot((TreeNode)model.getChild(rootNode, 0))));
	    JOptionPane.showMessageDialog(null, chooser);
	    System.exit(0);
	}
    }
}

/* $Log: DAVFileChooser.java,v $
/* Revision 1.3  2001/06/05 01:09:53  smulloni
/* fixed bug wtih failed connection attempts, which used to set the
/* connection flag as if the connection had succeeded.
/*
/* Revision 1.2  2001/01/03 20:11:31  smulloni
/* the DAVFileChooser now replaces JFileChooser for remote file access.
/* DAVMethod now has a protocol property.
/*
/* Revision 1.1  2001/01/03 00:30:35  smulloni
/* a number of modifications along the way to replacing JFileChooser with
/* something more suitable for remote (virtual) files.
/* */
