/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import java.io.Serializable;
import org.skunk.dav.client.DAVConstants;
import org.skunk.dav.client.DAVProperty;
import org.skunk.dav.client.Lock;

public class DAVTableHeader implements Serializable
{
    static final long serialVersionUID = -817320207861899526L;
    private String davElement;
    private String title;
    private Class daClass;
    private transient DAVProperty davProp;
    public static final String TABLE_HEADER_SUFFIX="_hdr";
    
    public static DAVTableHeader FILE_NAME=new DAVTableHeader(DAVConstants.HREF_ELEM);
    public static DAVTableHeader CREATION_DATE=new DAVTableHeader(DAVConstants.CREATIONDATE_PROP);
    public static DAVTableHeader DISPLAY_NAME=new DAVTableHeader(DAVConstants.DISPLAYNAME_PROP);
    public static DAVTableHeader CONTENT_LANGUAGE=new DAVTableHeader(DAVConstants.GETCONTENTLANGUAGE_PROP);
    public static DAVTableHeader CONTENT_LENGTH=new DAVTableHeader(DAVConstants.GETCONTENTLENGTH_PROP);
    public static DAVTableHeader CONTENT_TYPE=new DAVTableHeader(DAVConstants.GETCONTENTTYPE_PROP);
    public static DAVTableHeader ETAG=new DAVTableHeader(DAVConstants.GETETAG_PROP);
    public static DAVTableHeader LAST_MODIFIED=new DAVTableHeader(DAVConstants.GETLASTMODIFIED_PROP);
    public static DAVTableHeader EXECUTABLE=new DAVTableHeader(DAVConstants.EXECUTABLE_PROP);
    public static DAVTableHeader LOCKTOKEN=new DAVTableHeader(DAVConstants.LOCKTOKEN_ELEM);
    public static DAVTableHeader LOCKTYPE=new DAVTableHeader(DAVConstants.LOCKTYPE_ELEM);
    public static DAVTableHeader LOCKSCOPE=new DAVTableHeader(DAVConstants.LOCKSCOPE_ELEM);
    public static DAVTableHeader OWNER=new DAVTableHeader(DAVConstants.OWNER_ELEM);
    public static DAVTableHeader LOCK=new DAVTableHeader(DAVConstants.LOCKDISCOVERY_PROP);
    public static DAVTableHeader RESOURCETYPE=new DAVTableHeader(DAVConstants.RESOURCETYPE_PROP);

    public String toString()
    {
	return title;
    }

    private DAVTableHeader(String davElement)
    {
	this.davElement=davElement;
	this.title=ResourceManager.getMessage(davElement+TABLE_HEADER_SUFFIX, davElement);
	if (davElement.equals(DAVConstants.GETCONTENTLENGTH_PROP)) daClass=Number.class;
	else if (davElement.equals(DAVConstants.LOCKDISCOVERY_PROP)) daClass=Lock.class;
	else daClass=String.class;
    }

    public DAVProperty getDAVProperty()
    {
        if (davProp==null)
	{
	    davProp=DAVProperty.getCanonicalProperty(davElement);
	}
	return davProp;
    }

    public Class getRenderingClass()
    {	
	return daClass;
    }

    public boolean equals(Object otherObject)
    {
	if (otherObject instanceof DAVTableHeader)
	    return this.toString().equals(otherObject.toString());
	return false;
    }

}

/* $Log: DAVTableHeader.java,v $
/* Revision 1.12  2001/01/05 08:01:12  smulloni
/* changes to the connection gui for the Explorer; added depth configurability to
/* propfind in the jpython test script; added an experimental "allprop" system
/* property which affects the propfind query type
/*
/* Revision 1.11  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.10  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.9  2000/11/10 22:40:05  smullyan
/* added icon to table for resource type; fixes to copy and move; disabling of
/* menu items that are inappropriate.
/*
/* Revision 1.8  2000/11/09 23:34:55  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
