/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import java.net.URLDecoder;
import javax.swing.table.AbstractTableModel;
import org.skunk.dav.client.DAVFile;
import org.skunk.dav.client.DAVProperty;
import org.skunk.dav.client.Lock;
import org.skunk.trace.Debug;

public class DAVTableModel extends AbstractTableModel
{
    private DAVFile file;
    private DAVTableHeader[] headers;

    public DAVTableModel(DAVFile file, DAVTableHeader[] headers)
    {
	super();
	this.file=file;
	this.headers=headers;
    }

    public DAVTableModel(DAVTableHeader[] headers)
    {
	this(null, headers);
    }

    public void setDAVFile(DAVFile file)
    {
	this.file=file;
	fireTableDataChanged();
    }

    public int getRowCount()
    {
	if (file!=null) return file.getChildCount();
	else return 0;
    }

    /**
     * This is currently just a wrapper around DAVFile.getChildAt(int),
     * but should later include a hook for sorting routines.
     */
    private DAVFile getChildAt(DAVFile daFile, int row)
    {
	return daFile.getChildAt(row);
    }

    public DAVFile getDAVFile()
    {
	return file;
    }

    public DAVFile getDAVFile(int row)
    {
	if (row >=0 & row<getRowCount())
	{
	    return getChildAt(file, row);
	}
	return null;
    }

    public int getColumnCount()
    {
	return headers.length;
    }

    public Object getValueAt(int r, int c)
    {
	if (file!=null)
	{
	    DAVTableHeader header=headers[c];
	    DAVFile child=getChildAt(file, r);
	    return getValueForHeader(child, header);
	}
	else return "";
    }

    public String getColumnName(int c)
    {
	return ((c<headers.length)
		&& c>=0 )
	    //&& (!headers[c].equals(DAVTableHeader.LOCK))) 
	    ? headers[c].toString() 
	    : null;
    }

    public Class getColumnClass(int c)
    {
	return ((c<headers.length) && c>=0) ? headers[c].getRenderingClass() : null;
    }

    protected void setTableHeaders(DAVTableHeader[] headers)
    {
	this.headers=headers;
	fireTableStructureChanged();
    }

    private static Object getValueForHeader(DAVFile file, DAVTableHeader header)
    {
	if (header.equals(DAVTableHeader.LOCK))
	    return file.getExclusiveLock();
	else if (header.equals(DAVTableHeader.FILE_NAME))
	    return file.getUnicodeFileName(); // sachiho added ***
//	    return URLDecoder.decode(file.getFileName()); 
	else if (header.equals(DAVTableHeader.CREATION_DATE))
	    return getDate(file.getCreationDate());
	else if (header.equals(DAVTableHeader.DISPLAY_NAME))
	    return file.getDisplayName();
	else if (header.equals( DAVTableHeader.CONTENT_LANGUAGE))
	    return file.getContentLanguage();
	else if (header.equals( DAVTableHeader.CONTENT_LENGTH))	
	    return getLong(file.getContentLength());	
	else if (header.equals( DAVTableHeader.CONTENT_TYPE))
	    return file.getContentType();
	else if (header.equals( DAVTableHeader.ETAG))
	    return file.getEtag();
	else if (header.equals( DAVTableHeader.LAST_MODIFIED))
	    return getDate(file.getLastModified());
	else if (header.equals( DAVTableHeader.EXECUTABLE))
	    return file.getCustomProperty(DAVProperty.EXECUTABLE);
	else if (header.equals( DAVTableHeader.LOCKTOKEN))
	    return file.getExclusiveLockToken();
	else if (header.equals( DAVTableHeader.LOCKTYPE))
	{
	    Lock lock=file.getExclusiveLock();	    
	    return (lock==null) ? null : lock.getLockType();
	}
	else if (header.equals( DAVTableHeader.LOCKSCOPE))
	{
	    Lock lock=file.getExclusiveLock();	    
	    return (lock==null) ? null : lock.getLockScope();
	}
	else if (header.equals( DAVTableHeader.OWNER))
	{
	    Lock lock=file.getExclusiveLock();	    
	    return (lock==null) ? null : lock.getOwner();
	}
	else if (header.equals(DAVTableHeader.RESOURCETYPE))
	{
	    return new Boolean(file.isCollection());
	}
	else 
	{
	    Debug.trace(DAVTableModel.class, Debug.DP3, "tableHeader {0} not recognized!", header);
	}
	return null;
    }

    private static Object getLong(Object input)
    {
	if (input==null) return null;
	try
	{	    
	    return new Long(input.toString());
	}
	catch (NumberFormatException nafta) {}
	return input;
    }

    private static Object getDate(Object input)
    {
	//to be done
	return input;
    }
}



    

/* $Log: DAVTableModel.java,v $
/* Revision 1.16  2001/07/18 03:23:06  smulloni
/* added URL decoding to filename display in the table model and copy/move/rename dialogs;
/* not in the PropertiesDialog table, however.
/*
/* Revision 1.15  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.14  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.13  2000/11/10 22:40:06  smullyan
/* added icon to table for resource type; fixes to copy and move; disabling of
/* menu items that are inappropriate.
/*
/* Revision 1.12  2000/11/09 23:34:55  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
