/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreePath;
import org.skunk.dav.client.gui.DAVTreeNode;
import org.skunk.trace.Debug;


public class DAVTreeModelListener implements TreeModelListener
{
    private Explorer ex;

    public DAVTreeModelListener(Explorer explorer)
    {
	this.ex=explorer;
    }

    /*
     * implementation of TreeModelListener
     */

    public void treeNodesChanged(TreeModelEvent e)
    {
	Debug.trace(this, Debug.DP3, "changed: "+e);
	TreePath teepee=e.getTreePath();
	if (teepee.getPathCount()==1) //event on root node
	    ex.selectLastNode();
    }

    public void treeNodesInserted(TreeModelEvent e)
    {
	Debug.trace(this, Debug.DP3, "inserted: "+e);
	TreePath teepee=e.getTreePath();
	if (teepee.getPathCount()==1) //event on root node
	    ex.selectLastNode();

    }

    public void treeNodesRemoved(TreeModelEvent e)
    {
	Debug.trace(this, Debug.DP3, "removed: "+e);
	TreePath teepee=e.getTreePath();
	if (teepee.getPathCount()==1) //event on root node
	    ex.selectLastNode();
    }

    public void treeStructureChanged(TreeModelEvent e)
    {
	Debug.trace(this, Debug.DP3, "structure changed: " + e);
	TreePath teepee=e.getTreePath();
	if (teepee.getPathCount()==1) //event on root node
	    ex.selectLastNode();
	DAVTreeNode dtn=ex.getSelectedNode();
	if (dtn!=null && dtn.equals(teepee.getLastPathComponent()))
	{
	    ex.setTableFile(dtn.getDAVFile());
	}
    }
}

/* $Log: DAVTreeModelListener.java,v $
/* Revision 1.5  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.4  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.3  2000/11/09 23:34:56  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
