/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import java.net.URLDecoder;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;
import org.skunk.assertF.Assertion;
import org.skunk.dav.client.DAVFile;
import org.skunk.trace.Debug;

public class DAVTreeNode extends DefaultMutableTreeNode
{
    private DAVFile file;
    private boolean showNonCollections;
    private boolean refreshed;
    private boolean showHost;

    public DAVTreeNode(DAVFile file, boolean showNonCollections)
    {
	super();
	this.file=file;
	this.showNonCollections=showNonCollections;
	this.showHost=false;
	Debug.trace(this, 
		    Debug.DP5, 
		    "in constructor: DAVTreeNode({0}, {1})", 
		    new Object[] {file, new Boolean(showNonCollections)});
    }

    public DAVTreeNode(DAVFile file)
    {
	this(file, false);
    }

    public boolean isShowingHost()
    {
	return showHost;
    }

    public void setShowingHost(boolean showingHost)
    {
	this.showHost=showingHost;
    }
    
    public boolean isLeaf()
    {
	if (showNonCollections)
	    return (! file.isCollection());
	else return false;
    }
	
    private void setChildNodes()
    {
	removeAllChildren();
	Assertion.assertF(file!=null, "DAVFile is not null");
	Iterator it=file.children();
	while (it.hasNext())
	{
	    Object o=it.next();
	    Assertion.assertF((o instanceof DAVFile), 
			     "child of DAVFile is a DAVFile");
	    DAVFile kiddie=(DAVFile) o;
	    if (kiddie.isCollection() || showNonCollections)    
		add(new DAVTreeNode(kiddie, showNonCollections));
	}
    }

    public Object getUserObject()
    {
	return toString();
    }

    public String toString()
    {
	// fixes bug found by Elias Sinderson at the 2001 WebDAV Interop
	return (showHost) 
	    ? URLDecoder.decode(file.getFullName()) 
	    : URLDecoder.decode(file.getFileName());
    }

    public void setRefreshed(boolean refreshed)
    {
	this.refreshed=refreshed;
    }

    public boolean isRefreshed()
    {
	return this.refreshed;
    }

    public DAVFile getDAVFile()
    {
	return file;
    }

    public void setDAVFile(DAVFile file)
    {
	this.file=file;
	setChildNodes();
    }
}

/* $Log: DAVTreeNode.java,v $
/* Revision 1.11  2001/07/25 20:05:34  smulloni
/* bug fix: explorer tree wasn't url-decoding filenames.
/*
/* Revision 1.10  2001/01/03 00:30:35  smulloni
/* a number of modifications along the way to replacing JFileChooser with
/* something more suitable for remote (virtual) files.
/*
/* Revision 1.9  2001/01/02 17:32:24  smulloni
/* bugfix: was hard-coding showNonCollections to false for child nodes, which made no
/* sense.
/*
/* Revision 1.8  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.7  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.6  2000/11/09 23:34:56  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
