/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import java.awt.Event;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.Enumeration;
import java.util.Hashtable;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.skunk.dav.client.gui.action.AboutAction;
import org.skunk.dav.client.gui.action.CloseAction;
import org.skunk.dav.client.gui.action.CloseBufferAction;
import org.skunk.dav.client.gui.action.CloseViewAction;
import org.skunk.dav.client.gui.action.ConnectionAction;
import org.skunk.dav.client.gui.action.CopyAction;
import org.skunk.dav.client.gui.action.DeleteAction;
import org.skunk.dav.client.gui.action.DisconnectionAction;
import org.skunk.dav.client.gui.action.DownloadAction;
import org.skunk.dav.client.gui.action.ExitAction;
import org.skunk.dav.client.gui.action.MoveAction;
import org.skunk.dav.client.gui.action.NewCollectionAction;
import org.skunk.dav.client.gui.action.NewFileAction;
import org.skunk.dav.client.gui.action.OpenAction;
import org.skunk.dav.client.gui.action.PreferencesAction;
import org.skunk.dav.client.gui.action.PropertiesAction;
import org.skunk.dav.client.gui.action.RefreshAction;
import org.skunk.dav.client.gui.action.SaveAction;
import org.skunk.dav.client.gui.action.SaveAsAction;
import org.skunk.dav.client.gui.action.UploadAction;
import org.skunk.dav.client.gui.editor.DAVEditor;
import org.skunk.dav.client.gui.editor.DAVEditorUndoManager;
import org.skunk.dav.client.gui.editor.SimpleTextEditor;
import org.skunk.dav.client.gui.editor.action.CopyTextAction;
import org.skunk.dav.client.gui.editor.action.CutAction;
import org.skunk.dav.client.gui.editor.action.GotoLineAction;
import org.skunk.dav.client.gui.editor.action.PasteAction;
import org.skunk.dav.client.gui.editor.action.RedoAction;
import org.skunk.dav.client.gui.editor.action.SearchAction;
import org.skunk.dav.client.gui.editor.action.SearchAndReplaceAction;
import org.skunk.dav.client.gui.editor.action.SelectAllAction;
import org.skunk.dav.client.gui.editor.action.SyntaxHighlightAction;
import org.skunk.dav.client.gui.editor.action.UndoAction;
import org.skunk.dav.client.gui.editor.action.WordWrapAction;
import org.skunk.swing.text.syntax.FileMode;
import org.skunk.trace.Debug;

public class ExplorerMenuBar extends JMenuBar
{
    private static Hashtable menubars=new Hashtable();
    private static AboutAction aboutAction;
    private ConnectMenu connectMenu;
    private Explorer explorer;
    private JMenuItem openItem, deleteItem, copyItem, moveItem, 
	disconnectionItem, newFileItem, newCollectionItem, 
	saveItem, saveAsItem, closeBufferItem, propertiesItem, 
	refreshItem, preferencesItem, uploadItem, downloadItem,
	undoItem, redoItem, cutItem, copyTextItem, pasteItem, 
	selectAllItem, searchItem,  replaceItem,
	gotoLineItem, wordWrapItem, highlightItem;

    private boolean showingExits=true;
    private boolean trusted=true;
    private boolean connected=false;
    private boolean editorInFocus=false;
    private UndoListener undoListener=new UndoListener();

    /*
     * word wrap was broken by the syntax highlighting package; 
     * this flag enables me to keep the word wrap code around until I reimplement it
     */
    private boolean WORD_WRAP=false;

    private ExplorerMenuBar(Explorer explorer)
    {
	super();
	this.explorer=explorer;
	connectMenu=new ConnectMenu(explorer);
        add(createFileMenu());
        add(createEditMenu());
        add(createViewMenu());
        add(createHelpMenu());
	initState();
    }

    private void initState()
    {
	//some housekeeping for maintaining state
	setConnected(false);
	StateMonitor.DomainMatcher dm=new StateMonitor.DomainMatcher()
	    {
		public boolean domainMatches(Object domainKey)
		{
		    return ExplorerMenuBar.this.explorer.equals(domainKey);
		}
	    };
	StateMonitor.registerProperty(this, 
				      StateProperties.CONNECTED, 
				      StateProperties.CONNECTED,
				      null,
				      dm);
	StateMonitor.registerProperty(this,
				      StateProperties.EDITOR_IN_FOCUS,
				      StateProperties.EDITOR_IN_FOCUS,
				      null,
				      null); 
    }

    public static ExplorerMenuBar getMenuBar(Explorer explorer)
    {
	if (!menubars.containsKey(explorer))
	{
	    ExplorerMenuBar menubar=new ExplorerMenuBar(explorer);
	    menubars.put(explorer, menubar);
	    return menubar;
	}
	else return (ExplorerMenuBar) menubars.get(explorer);
    }

    public static void refreshMenus()
    {
	for (Enumeration enum=menubars.elements();enum.hasMoreElements();)
	{
	    ExplorerMenuBar emb=(ExplorerMenuBar) enum.nextElement();
	    emb.refresh();
	}
    }

    public boolean isShowingExits()
    {
	return showingExits;
    }

    public void setShowingExits(boolean showingExits)
    {
	this.showingExits=showingExits;
    }

    public boolean isTrusted()
    {
	return trusted;
    }

    public void setTrusted(boolean trusted)
    {
	this.trusted=trusted;
    }

    public boolean isConnected()
    {
	return this.connected;
    }

    /**
     *  this sends a signal to the menu bar that affects the enabled state of some of its menu items.
     */
    public void setConnected(boolean connected)
    {
	this.connected=connected;
	refreshMenuState();
    }

    public void setEditorInFocus(boolean editorInFocus)
    {
	this.editorInFocus=editorInFocus;
	refreshMenuState();
	resetUndoItems();	
    }

    public void setSearchInProgress(boolean searchInProgress)
    {
	if (Debug.DEBUG) Debug.trace(this, Debug.DP3, "in setSearchInProgress with argument : "+searchInProgress);
	boolean b=!searchInProgress;
	searchItem.setEnabled(b);
	replaceItem.setEnabled(b);
	gotoLineItem.setEnabled(b);
    }

    private void refreshMenuState()
    {
	boolean connectedAndShowingExplorer=connected && !editorInFocus;
	boolean connectedAndShowingEditor=connected && editorInFocus;
	boolean editingText=false;
	boolean highlighting=false;
	boolean canHighlight=false;
	if (connectedAndShowingEditor)
	{
	    Buffer b=ExplorerApp.getAppContext().getCurrentView().getFocussedBuffer();
	    if (b!=null && (b instanceof SimpleTextEditor))
	    {
		editingText=true;
		SimpleTextEditor ed=(SimpleTextEditor)b;
		highlighting=ed.isTokenizing();
		FileMode fm=ed.getSyntaxDocument().getFileMode();
		if (Debug.DEBUG) 
		    Debug.trace(this, Debug.DP4, "file mode of current buffer: {0}", fm);
		canHighlight=highlighting || (fm!=null && fm.getCanHighlight());
	    }
	}

	openItem.setEnabled(connectedAndShowingExplorer);
	copyItem.setEnabled(connectedAndShowingExplorer);
	moveItem.setEnabled(connectedAndShowingExplorer);
	disconnectionItem.setEnabled(connectedAndShowingExplorer);
	newFileItem.setEnabled(connectedAndShowingExplorer);
	newCollectionItem.setEnabled(connectedAndShowingExplorer);
	refreshItem.setEnabled(connectedAndShowingExplorer);
	uploadItem.setEnabled(connectedAndShowingExplorer);
	downloadItem.setEnabled(connectedAndShowingExplorer);
	propertiesItem.setEnabled(connectedAndShowingExplorer);
	saveAsItem.setEnabled(connectedAndShowingEditor);
	closeBufferItem.setEnabled(connectedAndShowingEditor);
	
	searchItem.setEnabled(editingText);
	gotoLineItem.setEnabled(editingText);
	copyTextItem.setEnabled(editingText);
	cutItem.setEnabled(editingText);
	pasteItem.setEnabled(editingText);
	replaceItem.setEnabled(editingText);
	selectAllItem.setEnabled(editingText);
	if (WORD_WRAP) wordWrapItem.setEnabled(editingText);
	highlightItem.setArmed(false);
	highlightItem.setEnabled(editingText && canHighlight);
	highlightItem.setSelected(highlighting);
	highlightItem.setArmed(true);
    }

    private void resetUndoItems()
    {	
	Buffer buffer=ExplorerApp.getAppContext().getCurrentView().getFocussedBuffer();
	DAVEditorUndoManager undoer;
	if ((buffer instanceof DAVEditor) 
	    && (undoer=((DAVEditor)buffer).getUndoManager())!=null)
	{
	    undoItem.setEnabled(undoer.canUndo());
	    redoItem.setEnabled(undoer.canRedo());
	    undoListener.setEditor((DAVEditor)buffer);
	    
	}
	else
	{
	    undoItem.setEnabled(false);
	    redoItem.setEnabled(false);
	    undoListener.setEditor(null);
	}
	
    }
	
    protected void finalize() throws Throwable
    {
	if (menubars.containsKey(explorer))
	    menubars.remove(explorer);
	super.finalize();
    }

    public void refresh()
    {
	connectMenu.refresh();
    }
    
    private JMenu createFileMenu() 
    {
	String label=ResourceManager.getMessage(ResourceManager.FILE_KEY);
        JMenu menu = new JMenu(label);

	label=ResourceManager.getMessage(ResourceManager.CONNECT_KEY);
	JMenuItem connectItem=menu.add(new ConnectionAction(explorer));
	connectItem.setActionCommand(ConnectionAction.CHOOSE_COMMAND);
	connectItem.setText(label);
	connectItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));

	menu.add(getConnectMenu());

	label=ResourceManager.getMessage(ResourceManager.DISCONNECTION_KEY);
	disconnectionItem=menu.add(new DisconnectionAction(explorer));
	disconnectionItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.ALT_MASK | InputEvent.SHIFT_MASK));
	disconnectionItem.setText(label);

	label=ResourceManager.getMessage(ResourceManager.NEW_COLLECTION_KEY);	
	newCollectionItem=menu.add(new NewCollectionAction(explorer));
	newCollectionItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK));
	newCollectionItem.setText(label);

	label=ResourceManager.getMessage(ResourceManager.NEW_FILE_KEY);
	newFileItem=menu.add(new NewFileAction(explorer));
	newFileItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.ALT_MASK | InputEvent.SHIFT_MASK));
	newFileItem.setText(label);

	label=ResourceManager.getMessage(ResourceManager.OPEN_KEY);
	openItem=menu.add(new OpenAction(explorer));
	openItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
	openItem.setText(label);

	label=ResourceManager.getMessage(ResourceManager.SAVE_KEY);
	saveItem=menu.add(new SaveAction());
	saveItem.setText(label);
	saveItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
	saveItem.setEnabled(false);
	StateMonitor.registerProperty(saveItem, 
				      "enabled",
				      StateProperties.EDITOR_IN_FOCUS_AND_DIRTY,
				      null,
				      null); 

	label=ResourceManager.getMessage(ResourceManager.SAVE_AS_KEY);
	saveAsItem=menu.add(new SaveAsAction());
	saveAsItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK));
	saveAsItem.setText(label);

	menu.addSeparator();

	label=ResourceManager.getMessage(ResourceManager.COPY_KEY);
	copyItem=menu.add(new CopyAction(explorer));
	copyItem.setText(label);

	label=ResourceManager.getMessage(ResourceManager.MOVE_KEY);
	moveItem=menu.add(new MoveAction(explorer));
	moveItem.setText(label);

	if (isTrusted())
	{
	    menu.addSeparator();

	    label=ResourceManager.getMessage(ResourceManager.UPLOAD_KEY);
	    uploadItem=menu.add(new UploadAction(explorer));
	    uploadItem.setText(label);

	    label=ResourceManager.getMessage(ResourceManager.DOWNLOAD_KEY);
	    downloadItem=menu.add(new DownloadAction(explorer));
	    downloadItem.setText(label);
	}

	if (isShowingExits())
	{
	    menu.addSeparator();

	    label=ResourceManager.getMessage(ResourceManager.CLOSE_BUFFER_KEY);
	    closeBufferItem=menu.add(new CloseBufferAction());
	    closeBufferItem.setText(label);
	    closeBufferItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_MASK));
	    closeBufferItem.setEnabled(false);
	    
	    JMenuItem tmp;

	    /* given that the app does not at present give the user the ability to create multiple views, 
	       I am commenting this out.  -JS */
// 	    label=ResourceManager.getMessage(ResourceManager.CLOSE_KEY);
// 	    tmp=menu.add(new CloseViewAction());
// 	    tmp.setText(label);
// 	    tmp.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK));

	    label=ResourceManager.getMessage(ResourceManager.EXIT_KEY);
	    tmp=menu.add(new ExitAction());
	    tmp.setText(label);
	    tmp.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_MASK));
	}
        return menu;
    }

    private JMenu getConnectMenu()
    {
	return connectMenu;
    }

    private JMenu createEditMenu()
    {
	String label=ResourceManager.getMessage(ResourceManager.EDIT_KEY);
	JMenu menu = new JMenu(label);

	label=ResourceManager.getMessage(ResourceManager.UNDO_KEY);
	undoItem=menu.add(new UndoAction());
	undoItem.setText(label);
	undoItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, InputEvent.CTRL_MASK));
	undoItem.setEnabled(false);

	label=ResourceManager.getMessage(ResourceManager.REDO_KEY);
	redoItem=menu.add(new RedoAction());
	redoItem.setText(label);
	redoItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, InputEvent.ALT_MASK));
	redoItem.setEnabled(false);

	label=ResourceManager.getMessage(ResourceManager.CUT_KEY);
	cutItem=menu.add(new CutAction()); 
	cutItem.setEnabled(false);
	cutItem.setText(label);
	cutItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK));

	label=ResourceManager.getMessage(ResourceManager.COPY_TEXT_KEY);
	copyTextItem=menu.add(new CopyTextAction());
	copyTextItem.setEnabled(false);
	copyTextItem.setText(label);
	copyTextItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK));

	label=ResourceManager.getMessage(ResourceManager.PASTE_KEY);
	pasteItem=menu.add(new PasteAction());
	pasteItem.setEnabled(false);
	pasteItem.setText(label);
	pasteItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_MASK));

	label=ResourceManager.getMessage(ResourceManager.SELECT_ALL_KEY);
	selectAllItem=menu.add(new SelectAllAction()); //new JMenuItem(label);
	selectAllItem.setText(label);
	selectAllItem.setEnabled(false);

	label=ResourceManager.getMessage(ResourceManager.SEARCH_KEY);
	searchItem=menu.add(new SearchAction());
	searchItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.ALT_MASK));
	searchItem.setText(label);

	label=ResourceManager.getMessage(ResourceManager.REPLACE_KEY);
	replaceItem=menu.add(new SearchAndReplaceAction());
	replaceItem.setEnabled(false);
	replaceItem.setText(label);
	replaceItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, InputEvent.SHIFT_MASK | InputEvent.ALT_MASK));

	label=ResourceManager.getMessage(ResourceManager.GOTO_LINE_KEY);
	gotoLineItem=menu.add(new GotoLineAction());
	gotoLineItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, InputEvent.ALT_MASK));
	gotoLineItem.setText(label);

	menu.add(gotoLineItem);

	menu.add(new JSeparator());

	if (WORD_WRAP)
	{
	
	    wordWrapItem=new JCheckBoxMenuItem(ResourceManager.getMessage(ResourceManager.WORD_WRAP_KEY));
	    wordWrapItem.addActionListener(new WordWrapAction());
	    wordWrapItem.setSelected(false);
	    wordWrapItem.setEnabled(false);
	    menu.add(wordWrapItem);
	}

	highlightItem=new JCheckBoxMenuItem(ResourceManager.getMessage(ResourceManager.SYNTAX_HIGHLIGHT_KEY));
	highlightItem.setSelected(false);
	highlightItem.setEnabled(false);
	highlightItem.addActionListener(new SyntaxHighlightAction());
	menu.add(highlightItem);

	StateMonitor.registerProperty(this, 
				      "searchInProgress",
				      SearchAction.SEARCH_IN_PROGRESS_PROPERTY,
				      null,
				      null);

	return menu;
    }

    private JMenu createViewMenu()
    {
	JMenu menu=new JMenu(ResourceManager.getMessage(ResourceManager.VIEW_KEY));
	String label=ResourceManager.getMessage(ResourceManager.REFRESH_KEY);
	refreshItem=menu.add(new RefreshAction(explorer));
	refreshItem.setText(label);

	menu.add(new JSeparator());

	label=ResourceManager.getMessage(ResourceManager.PROPERTIES_KEY);
	propertiesItem=menu.add(new PropertiesAction(explorer));
	propertiesItem.setText(label);

	menu.add(new JSeparator());
	
	label=ResourceManager.getMessage(ResourceManager.PREFERENCES_KEY);
	preferencesItem=menu.add(new PreferencesAction());
	preferencesItem.setText(label);
	return menu;
    }

    private JMenu createHelpMenu()
    {
	if (aboutAction==null)
	{
	    aboutAction=new AboutAction();
	}
	JMenu jm= new JMenu(ResourceManager.getMessage(ResourceManager.HELP_KEY));
	String label=ResourceManager.getMessage(ResourceManager.ABOUT_KEY);
	JMenuItem tmp=jm.add(aboutAction);
	tmp.setText(label);
	return jm;
    }

    /**
     * a hook which permits applications to change the text on the "About" splash screen
     */
    public static void setSplashScreenText(String text)
    {
	if (aboutAction==null) aboutAction= new AboutAction();
	aboutAction.setSplashScreenText(text);
    }

    class UndoListener implements ChangeListener
    {
	private DAVEditor editor;

	UndoListener()
	{}

	UndoListener(DAVEditor editor)
	{
	    setEditor(editor);
	}

	void setEditor(DAVEditor editor)
	{
	    disconnect();
	    this.editor=editor;
	    if (editor!=null)
	    {
		DAVEditorUndoManager undoer=editor.getUndoManager();
		if (undoer!=null) 
		    undoer.addChangeListener(this);
	    }
	}

	DAVEditor getEditor()
	{
	    return this.editor;
	}

	void disconnect()
	{
	    if (editor!=null)
		editor.getUndoManager().removeChangeListener(this);
	}
	
	public void stateChanged(ChangeEvent cheese)
	{
	    resetUndoItems();
	}
    }

}

/* $Log: ExplorerMenuBar.java,v $
/* Revision 1.37  2001/02/06 23:15:53  smulloni
/* added syntax highlight and selectAll actions to the menu
/*
/* Revision 1.36  2001/02/06 22:13:40  smulloni
/* first more-or-less working version of syntax highlighting, with customization.
/*
/* Revision 1.35  2001/01/05 08:01:12  smulloni
/* changes to the connection gui for the Explorer; added depth configurability to
/* propfind in the jpython test script; added an experimental "allprop" system
/* property which affects the propfind query type
/*
/* Revision 1.34  2000/12/21 18:53:13  smulloni
/* cosmetic improvements.
/*
/* Revision 1.33  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.32  2000/12/19 20:17:54  smulloni
/* added an "About..." menu item.
/*
/* Revision 1.31  2000/12/19 19:22:22  smulloni
/* some tweaks: ExitAction now cleans up the app's buffers.  The app now does
/* not resize every time a buffer is added or removed.
/*
/* Revision 1.30  2000/12/15 23:22:07  smulloni
/* added word wrap to text editor.
/*
/* Revision 1.29  2000/12/14 23:09:17  smulloni
/* fixes to search and replace; partial fix to adding custom properties bug.
/*
/* Revision 1.28  2000/12/14 06:36:26  smulloni
/* changes to search and replace in text editor.
/*
/* Revision 1.27  2000/12/08 05:50:30  smulloni
/* fixed MessageCatalogEditor.  The spi features are now a special build option,
/* and editors are loaded through reflection.
/*
/* Revision 1.26  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.25  2000/12/03 20:40:05  smulloni
/* reconciling the minos.skunk.org cvs repository with sourceforge.
/*
/* Revision 1.25  2000/12/01 22:54:13  smullyan
/* added undo and basic editing functionality to text editor; defined some
/* keybindings in an inflexible way that will have to be changed soon.
/*
/* Revision 1.24  2000/12/01 16:25:52  smullyan
/* improvements to look and feel; fixed NPE in DAVFile; new actions for text
/* editor
/*
/* Revision 1.23  2000/11/29 23:16:04  smullyan
/* adding first rough cut of search capability to the text editor.  View
/* is being updated to allow components to be docked into the status bar.
/*
/* Revision 1.22  2000/11/28 00:01:38  smullyan
/* added a status bar/minibuffer, with a location field showing the current line and
/* column number (for the SimpleTextEditor and kin only).
/*
/* Revision 1.21  2000/11/23 04:37:32  smullyan
/* editor and explorer now synch their files using the StateMonitor, which has
/* been improved significantly
/*
/* Revision 1.20  2000/11/22 00:11:27  smullyan
/* editor now locks and unlocks, more or less appropriately.
/*
/* Revision 1.19  2000/11/20 23:30:20  smullyan
/* more editor integration work.
/*
/* Revision 1.18  2000/11/18 04:36:04  smullyan
/* work on StateMonitor and related functionality.
/*
/* Revision 1.17  2000/11/17 20:25:05  smullyan
/* new SaveAction; a StateMonitor being added to handle application state.
/*
/* Revision 1.16  2000/11/16 20:45:18  smullyan
/* the start of editor integration.
/*
/* Revision 1.15  2000/11/10 22:40:06  smullyan
/* added icon to table for resource type; fixes to copy and move; disabling of
/* menu items that are inappropriate.
/*
/* Revision 1.14  2000/11/09 23:34:57  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
