/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.FontMetrics;
import javax.swing.BorderFactory;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.FontUIResource;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;

/**
 *  very largely based on org.armedbear.j.DefaultLookAndFeel, copyright 2000 Peter Graves,
 *  which, like this software, is released under the GPL.  The admirably sleek and eye-friendly
 *  l&f of his "j" editor is well worth emulating.
 */
public class LookAndFeel extends DefaultMetalTheme
{
    private final ColorUIResource primary1 = new ColorUIResource( 0, 0, 0 ); 
    private FontUIResource plainFont, boldFont;

    public static void setLookAndFeel( )
    {
        // This is the default.
        String lookAndFeelClassName = "javax.swing.plaf.metal.MetalLookAndFeel";
	MetalLookAndFeel.setCurrentTheme(new LookAndFeel());
	UIManager.put("Tree.collapsedIcon", ResourceManager.getImageIcon(ResourceManager.COLLAPSED_ICON));
	UIManager.put("Tree.expandedIcon", ResourceManager.getImageIcon(ResourceManager.EXPANDED_ICON));
        try
        {
            UIManager.setLookAndFeel(lookAndFeelClassName);
        }
        catch (Exception e)
        {
        }
    }

    public LookAndFeel()
    {
        String fontName = "Dialog";
        int    fontSize = 11;
        Font font = new Font(fontName, Font.PLAIN, fontSize);
        plainFont = new FontUIResource(font);
	font=new Font(fontName, Font.BOLD, fontSize);
	boldFont=new FontUIResource(font);
    }

    public void addCustomEntriesToTable( UIDefaults table )
    {
        //table.put( "Button.border", BorderFactory.createRaisedBevelBorder( ) );
        table.put( "TextField.border", BorderFactory.createLoweredBevelBorder( ) );
        table.put( "SplitPaneUI", "javax.swing.plaf.basic.BasicSplitPaneUI" );
        table.put( "ScrollBarUI", "javax.swing.plaf.basic.BasicScrollBarUI" );
        table.put( "TreeUI", "javax.swing.plaf.basic.BasicTreeUI" );
        table.put( "SplitPane.dividerSize", new Integer( 3 ) );
        table.put( "ScrollBar.background", new Color( 0xe0e0e0 ) );
        table.put( "ScrollBar.foreground", new Color( 0xc0c0c0 ) );
        table.put( "ScrollBar.track", new Color( 0xe0e0e0 ) );
        table.put( "ScrollBar.trackHighlight", Color.black );
        table.put( "ScrollBar.thumb", new Color( 0xc0c0c0 ) );
        table.put( "ScrollBar.thumbHighlight", Color.white );
        table.put( "ScrollBar.thumbDarkShadow", Color.black );
        table.put( "ScrollBar.thumbLightShadow", new Color( 0x808080 ) );
        table.put( "Button.textIconGap", new Integer( 2 ) );
    }

    protected ColorUIResource getPrimary1( )
    {
        return primary1;
    }

    public FontUIResource getControlTextFont( )
    {
        return plainFont;
    }

    public FontUIResource getSystemTextFont( )
    {
        return plainFont;
    }

    public FontUIResource getUserTextFont( )
    {
        return plainFont;
    }
    
    public FontUIResource getMenuTextFont( )
    {
        return plainFont;
    }

    public FontUIResource getWindowTitleFont( )
    {
        return plainFont;
    }

    public FontUIResource getSubTextFont( )
    {
        return plainFont;
    }
}
