/*
 *  Copyright (c) 2001, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import org.skunk.config.Configurator;
import org.skunk.dav.client.DAVConnection;
import org.skunk.dav.client.DAVConnectionPool;
import org.skunk.dav.client.DAVConstants;
import org.skunk.swing.text.EntryDocument;
import org.skunk.swing.text.PositiveIntegerEntryFilter;
import org.skunk.trace.Debug;

/**
 * a customizer for network properties.  Was previously called GeneralCustomizer.
 * now also includes server customization: host, port, etc.
 * @author $Author: smulloni $
 * @version $Revision: 1.5 $
 */
public class NetworkCustomizer extends PreferencesCustomizer
{
    private JCheckBox rememberPassBox;
    private JPanel customizer;
    private Object socketTimeoutVal;
    private Object reversionVal;
    private JButton delButton, addButton;
    private JTextField timeoutField;
    private JTextField hostField, portField, pathField, 
	nameField, passField, ownerField;
    private JList serverList;
    private DefaultListModel serverListModel;
    private JComboBox protoBox;
    
    private boolean timeoutFieldIsSensitive=true;
    private boolean serverFieldsAreSensitive=true;
    

    public NetworkCustomizer(String title, Configurator configurator)
    {
	super(title, configurator);
	initConfig();
	initComponents();
    }

    private void initConfig()
    {
	for (Enumeration enum=DAVConnectionPool.getDAVConnections();enum.hasMoreElements();)
	{
	    Object target=enum.nextElement();
	    if (Debug.DEBUG) Debug.trace(this, Debug.DP5, "next DAVConnection: "+ target);
	    addConfigTarget(target);
	}
	socketTimeoutVal=getConfigurator()
	    .getStore()
	    .getConfigValue(DAVConnection.class, 
			    DAVConnection.SOCKET_TIMEOUT_PROPERTY);
	if (socketTimeoutVal==null)
	    socketTimeoutVal=new Integer(DAVConnection.DEFAULT_SOCKET_TIMEOUT);	
	reversionVal=socketTimeoutVal;

	addConfigTarget(ServerData.container);
    }

    private void initComponents()
    {
	customizer=new JPanel(new GridBagLayout());
	GridBagConstraints gbc=new GridBagConstraints();
	gbc.anchor=GridBagConstraints.NORTHWEST;
	gbc.gridx=0;
	gbc.gridy=0;
	gbc.gridwidth=1;
	gbc.gridheight=1;
	gbc.insets=new Insets(2, 2, 2, 2);
	gbc.fill=GridBagConstraints.HORIZONTAL;
	gbc.ipadx=1;
	gbc.ipady=1;
	gbc.weightx=1.0;
	gbc.weighty=0.20;
	gbc.anchor=GridBagConstraints.WEST;
	customizer.add(initTimeout(), gbc);
	gbc.gridy++;
	gbc.weightx=0.0;
	gbc.weighty=0.80;
	customizer.add(initServers(), gbc);
	//some empty space
	gbc.weightx=0.0;
	gbc.gridy++;
	gbc.weighty=1.0;
	customizer.add(new JLabel(), gbc);
    }

    public void ok()
    {
	Debug.trace(this, Debug.DP4, "in ok()");
	if (updateServerConfigValues())
	{
	    super.ok();
	    ExplorerMenuBar.refreshMenus();
	}
    }

    public void tryOn()
    {
	Debug.trace(this, Debug.DP4, "in tryOn()");
	if (updateServerConfigValues())
	{
	    super.tryOn();
	    ExplorerMenuBar.refreshMenus();
	}
	    
    }

    private void cleanUp()
    {
	//use this flag to suppress processing of document events
	timeoutFieldIsSensitive=false;
	timeoutField.setText(reversionVal.toString());
	timeoutFieldIsSensitive=true;
	refreshServerForm();
	refreshServerList();
	ExplorerMenuBar.refreshMenus();
    }

    public void revert()
    {
	super.revert();
	cleanUp();
    }

    public void revertToDefault()
    {
	super.revertToDefault();
	timeoutFieldIsSensitive=false;	
	socketTimeoutVal=new Integer(DAVConnection.DEFAULT_SOCKET_TIMEOUT);
	timeoutField.setText(socketTimeoutVal.toString());
	timeoutFieldIsSensitive=true;
	//refreshServerForm();
	//refreshServerList();
	//ExplorerMenuBar.refreshMenus();
    }

    public void cancel()
    {
	super.cancel();
	cleanUp();
    }

    private JPanel initTimeout()
    {
	JPanel daPanel=new JPanel(new GridBagLayout());
	
	String timeoutTitle=ResourceManager.getMessage(ResourceManager.NETWORK_CUSTOMIZER_TIMEOUT_TITLE);
	JLabel timeoutLabel=new JLabel(timeoutTitle);
	timeoutField=new JTextField(7); 
	timeoutField.setDocument(new EntryDocument(new PositiveIntegerEntryFilter(0, 99999)));
	timeoutField.setText(socketTimeoutVal.toString());
	timeoutField.getDocument().addDocumentListener(new TimeoutDocumentListener());

	GridBagConstraints gbc=new GridBagConstraints();
	gbc.gridx=0;
	gbc.gridy=0;
	gbc.gridwidth=1;
	gbc.gridheight=1;
	gbc.insets=new Insets(2, 2, 2, 2);
	gbc.fill=GridBagConstraints.NONE;
	gbc.ipadx=1;
	gbc.ipady=1;
	gbc.weightx=0.0;
	gbc.weighty=0.0;
	gbc.anchor=GridBagConstraints.NORTHWEST;
	
	daPanel.add(timeoutLabel, gbc);
	gbc.gridx++;
	gbc.weightx=1.0;
	daPanel.add(timeoutField, gbc);


	String borderTitle=ResourceManager.getMessage(ResourceManager.TIMEOUT_BORDER_TITLE);
	daPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
							   borderTitle, 
							   TitledBorder.LEFT,
							   TitledBorder.TOP));
	return daPanel;
    }

    public JPanel initServers()
    {
	JPanel daPanel=new JPanel(new GridBagLayout());

	addButton=new JButton(ResourceManager.getMessage(ResourceManager.ADD_SERVER_LABEL));
	addButton.setActionCommand("add");
	delButton=new JButton(ResourceManager.getMessage(ResourceManager.DEL_SERVER_LABEL));
	delButton.setActionCommand("delete");
	delButton.setEnabled(false);

	protoBox=new JComboBox(getAvailableProtocols());
	protoBox.setActionCommand("protocol");
	serverListModel=new DefaultListModel();
	refreshServerList();
	serverList=new JList(serverListModel);
	serverList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

	ServerFormListener formListener=new ServerFormListener();
	serverList.addListSelectionListener(formListener);
	addButton.addActionListener(formListener);
	delButton.addActionListener(formListener);
	protoBox.addActionListener(formListener);
	hostField=new JTextField(22);
	hostField.getDocument().addDocumentListener(formListener);
	portField=new JTextField(5);
	portField.setDocument(new EntryDocument(new PositiveIntegerEntryFilter(0, ((int)Math.pow(2, 16))-1)));
	portField.getDocument().addDocumentListener(formListener);
	pathField=new JTextField(22);
	pathField.getDocument().addDocumentListener(formListener);
        nameField=new JTextField(22);
	nameField.getDocument().addDocumentListener(formListener);
	passField=new JPasswordField(22);
	passField.getDocument().addDocumentListener(formListener);
	rememberPassBox=new JCheckBox(ResourceManager.getMessage(ResourceManager.REMEMBER_PASSWORD_LABEL));
	rememberPassBox.addItemListener(formListener);
	ownerField=new JTextField(22);
	ownerField.getDocument().addDocumentListener(formListener);

	GridBagConstraints gbc=new GridBagConstraints();
	gbc.gridx=0;
	gbc.gridy=0;
	gbc.fill=GridBagConstraints.NONE;
	gbc.insets=new Insets(2, 2, 2, 2);
	gbc.ipadx=1;
	gbc.ipady=1;
	gbc.anchor=GridBagConstraints.WEST;
	gbc.gridwidth=1;
	gbc.gridheight=GridBagConstraints.REMAINDER;
	
	Box serverBox=Box.createVerticalBox();
	serverBox.add(Box.createVerticalGlue());
	serverBox.add(new JScrollPane(serverList));
	serverBox.add(Box.createVerticalGlue());
	serverBox.add(addButton);
	serverBox.add(Box.createVerticalGlue());
	serverBox.add(delButton);
	serverBox.add(Box.createVerticalGlue());

	daPanel.add(serverBox, gbc);

	gbc.gridheight=1;
	gbc.gridx=1;
	
	JLabel tmpLabel;
	gbc.anchor=GridBagConstraints.EAST;
	tmpLabel=new JLabel(ResourceManager.getMessage(ResourceManager.PROTOCOL_PROMPT),
			    SwingConstants.RIGHT);
	daPanel.add(tmpLabel, gbc);

	gbc.gridx++;
	gbc.anchor=GridBagConstraints.WEST;
	daPanel.add(protoBox, gbc);

	gbc.gridx=1;
	gbc.gridy++;
	gbc.anchor=GridBagConstraints.EAST;
	tmpLabel=new JLabel(ResourceManager.getMessage(ResourceManager.HOST_PROMPT), 
			    SwingConstants.RIGHT);
	daPanel.add(tmpLabel, gbc);
	
	gbc.gridx++;
	gbc.anchor=GridBagConstraints.WEST;
	daPanel.add(hostField, gbc);

	gbc.gridx=1;
	gbc.gridy++;
	gbc.anchor=GridBagConstraints.EAST;
	tmpLabel=new JLabel(ResourceManager.getMessage(ResourceManager.PORT_PROMPT), 
			    SwingConstants.RIGHT);
	daPanel.add(tmpLabel, gbc);

	gbc.gridx++;
	gbc.anchor=GridBagConstraints.WEST;
	daPanel.add(portField, gbc);
	
	gbc.gridx=1;
	gbc.gridy++;
	gbc.anchor=GridBagConstraints.EAST;
	tmpLabel=new JLabel(ResourceManager.getMessage(ResourceManager.PATH_PROMPT), 
			    SwingConstants.RIGHT);
	daPanel.add(tmpLabel, gbc);

	gbc.gridx++;
	gbc.anchor=GridBagConstraints.WEST;
	daPanel.add(pathField, gbc);

	gbc.gridx=1;
	gbc.gridy++;
	gbc.anchor=GridBagConstraints.EAST;
	tmpLabel=new JLabel(ResourceManager.getMessage(ResourceManager.USERNAME_PROMPT),
			    SwingConstants.RIGHT);
	daPanel.add(tmpLabel, gbc);

	gbc.gridx++;
	gbc.anchor=GridBagConstraints.WEST;
	daPanel.add(nameField, gbc);

	gbc.gridx=1;
	gbc.gridy++;
	gbc.anchor=GridBagConstraints.CENTER;
	gbc.gridwidth=2;
	daPanel.add(rememberPassBox, gbc);
		
	gbc.gridwidth=1;
	gbc.gridx=1;
	gbc.gridy++;
	gbc.anchor=GridBagConstraints.EAST;
	tmpLabel=new JLabel(ResourceManager.getMessage(ResourceManager.PASSWORD_PROMPT),
			    SwingConstants.RIGHT);
	daPanel.add(tmpLabel, gbc);

	gbc.gridx++;
	gbc.anchor=GridBagConstraints.WEST;
	daPanel.add(passField, gbc);

	gbc.gridx=1;
	gbc.gridy++;
	gbc.anchor=GridBagConstraints.EAST;
	tmpLabel=new JLabel(ResourceManager.getMessage(ResourceManager.OWNER_PROMPT),
			    SwingConstants.RIGHT);
	daPanel.add(tmpLabel, gbc);
      
	gbc.gridx++;
	gbc.anchor=GridBagConstraints.WEST;
	daPanel.add(ownerField, gbc);
	String borderTitle=ResourceManager.getMessage(ResourceManager.SERVER_BORDER_TITLE);
	daPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
							   borderTitle, 
							   TitledBorder.LEFT,
							   TitledBorder.TOP));
	if (serverListModel.size()>0)
	    serverList.setSelectedIndex(0);

	return daPanel;
    }

    private void refreshServerList()
    {
	serverListModel.clear();
	for (Enumeration enum=ServerData.getServers().elements();enum.hasMoreElements();)
	{
	    serverListModel.addElement(enum.nextElement());
	}	
    }

    private void populateForm(ServerData sd)
    {
	//if the server data has https for the protocol, and
	//SSL is not enabled, we've got a problem.
	if (sd==null)
	{
	    
	    protoBox.setSelectedItem(DAVConstants.HTTP);
	    hostField.setText("");
	    portField.setText("80");
	    pathField.setText("/");
	    nameField.setText("");
	    rememberPassBox.setSelected(false);
	    passField.setText("");
	    ownerField.setText("");
	}
	else
	{
	    boolean usesSSL=sd.usesSSL();
	    if (usesSSL 
		&& !DAVConnectionPool.canSupportSSL())
	    {
		//issue warning
		String message=ResourceManager.getMessage(ResourceManager.HTTPS_NOT_SUPPORTED_MESSAGE);
		String title=ResourceManager.getMessage(ResourceManager.HTTPS_NOT_SUPPORTED_DIALOG_TITLE);
		JOptionPane.showMessageDialog(this.customizer, 
					      message, 
					      title, 
					      JOptionPane.WARNING_MESSAGE);
	    }
	    protoBox.setSelectedItem(usesSSL ? DAVConstants.HTTPS : DAVConstants.HTTP);
	    hostField.setText(sd.getHost());
	    portField.setText(Integer.toString(sd.getPort()));
	    pathField.setText(sd.getInitialPath());
	    nameField.setText(sd.getUsername());
	    rememberPassBox.setSelected(sd.getRememberPassword());
	    passField.setText(sd.getPassword());
	    ownerField.setText(sd.getOwner());
	}
    }

    private class ServerFormListener 
	implements ActionListener, DocumentListener, ListSelectionListener, ItemListener
    {
	public void itemStateChanged(ItemEvent ie)
	{
	    int stateChange=ie.getStateChange();
	    if (stateChange==ItemEvent.DESELECTED)
	    {
		boolean sensitivity=serverFieldsAreSensitive;
		serverFieldsAreSensitive=false;
		passField.setText("");
		serverFieldsAreSensitive=sensitivity;
		passField.setEnabled(false);
		if (serverFieldsAreSensitive)
		    setDirty(true);
	    }
	    else if (stateChange==ItemEvent.SELECTED)
	    {
		passField.setEnabled(true);
		if (serverFieldsAreSensitive)
		    setDirty(true);
	    }
	   
	}

	public void valueChanged(ListSelectionEvent lassie)
	{
	    Object val=serverList.getSelectedValue();
	    serverFieldsAreSensitive=false;
	    if (val!=null)
	    {
		populateForm((ServerData)val);
		delButton.setEnabled(true);
	    }
	    else
	    {
		populateForm(null);
		delButton.setEnabled(false);
	    }
	    serverFieldsAreSensitive=true;
	}

	public void actionPerformed(ActionEvent ae)
	{
	    // for add and delete buttons
	    String command=ae.getActionCommand();
	    if ("add".equals(command))
	    {
		ServerData sd=new ServerData(false, "", 80, "/", "", "", false, false);
		serverListModel.addElement(sd);
		serverList.setSelectedValue(sd, true);
		setDirty(true);
	    }
	    else if ("delete".equals(command))
	    {
		Object o=serverList.getSelectedValue();
		int index=serverList.getSelectedIndex();
		if (o!=null)
		{
		    serverListModel.removeElement(o);
		    updateServerData();
		    int size=serverListModel.size();
		    if (size>index)
			serverList.setSelectedIndex(index);
		    else if (size==index)
			serverList.setSelectedIndex(size-1);
		    else if (index==0)
			populateForm(null);
		    setDirty(true);
		}
	    }
	    else if ("protocol".equals(command))
	    {
		    formChanged();
	    }
	}

	private void formChanged()
	{
	    if (serverFieldsAreSensitive)
	    {
		updateServerData();
	    }
	}

	public void changedUpdate(DocumentEvent donkey) {}

	public void insertUpdate(DocumentEvent donkey)
	{
	    formChanged();
	}

	public void removeUpdate(DocumentEvent donkey)
	{
	    formChanged();
	}
    }

    private boolean portIsValid()
    {
	try 
	{
	    Integer.parseInt(portField.getText().trim());
	    return true;
	}
	catch (NumberFormatException nafta)
	{
	    return false;
	}
    }

    private Object[] getAvailableProtocols()
    {
	//if (DAVConnectionPool.canSupportSSL())
	return new Object[] { DAVConstants.HTTP, DAVConstants.HTTPS } ;
	    //else return new Object[] { DAVConstants.HTTP };
    }

    private Vector getServers()
    {
	Vector v=new Vector();
	for (Enumeration enum=serverListModel.elements();enum.hasMoreElements();)
	{
	    v.add(enum.nextElement());
	}
	return v;
    }

    private boolean updateServerConfigValues()
    {
	if (portIsValid())
	{
	    Vector servers=getServers();
	    Debug.trace(this, Debug.DP2, "servers: {0}", servers);
	    setConfigValue(ServerDataContainer.class, 
			   ServerDataContainer.SERVER_DATA_VECTOR_PROPERTY, 
			   servers);	    
	}
	else
	{
	    String message=ResourceManager.getMessage(ResourceManager.PORT_INVALID_MESSAGE);
	    String title=ResourceManager.getMessage(ResourceManager.PORT_INVALID_TITLE);
	    JOptionPane.showMessageDialog(this.customizer, 
					  message, 
					  title, 
					  JOptionPane.ERROR_MESSAGE);
	    return false;
	}
	return true;
    }

    private void refreshServerForm()
    {
	serverFieldsAreSensitive=false;
	Object selected=serverList.getSelectedValue();
	Vector servers=ServerData.getServers();
	serverListModel.clear();
	for (Enumeration enum=servers.elements();enum.hasMoreElements();)
	{
	    Object o=enum.nextElement();
	    if (selected!=null)
	    {
		if (((ServerData)selected).getHost().equals(((ServerData)o).getHost()))
		    selected=o;
	    }
	    serverListModel.addElement(o);
	}
	if (selected!=null)
	{
	    serverList.setSelectedValue(selected, true);
	}
	serverFieldsAreSensitive=true;
    }

    private void updateServerData()
    {
	//modify the active ServerData according to the contents of the form
	Object o=serverList.getSelectedValue();
	if (o!=null)
	{
	    ServerData sd=(ServerData)o;
	    sd.setHost(hostField.getText().trim());
	    sd.setInitialPath(pathField.getText().trim());
	    sd.setUsername(nameField.getText().trim());
	    sd.setRememberPassword(rememberPassBox.isSelected());
	    sd.setPassword(passField.getText());
	    sd.setUsesSSL(protoBox.getSelectedItem().equals(DAVConstants.HTTPS));
	    sd.setOwner(ownerField.getText());
	    String portText=portField.getText().trim();
	    if (portText.length()>0)
		sd.setPort(Integer.parseInt(portText));
	    setDirty(true);
	}
    }

    public JComponent getCustomizer()
    {
	return customizer;
    }

    private class TimeoutDocumentListener implements DocumentListener
    {
	public void changedUpdate(DocumentEvent donkey)
	{
	    //do nothing
	}

	public void insertUpdate(DocumentEvent donkey)
	{
	    if (timeoutFieldIsSensitive)  _process(donkey.getDocument());
	}

	public void removeUpdate(DocumentEvent donkey)
	{
	    if (timeoutFieldIsSensitive) _process(donkey.getDocument());
	}

	private void _process(Document doc)
	{
	    try
	    {
		Integer aye=new Integer(doc.getText(0, doc.getLength()));
		if (aye.equals(socketTimeoutVal))
		    return;
	        socketTimeoutVal=aye;
		setConfigValue(DAVConnection.class, DAVConnection.SOCKET_TIMEOUT_PROPERTY, aye);
		setDirty(true);
	    }
	    catch (BadLocationException ballocks)
	    {
		if (Debug.DEBUG) Debug.trace(this, Debug.DP2, ballocks);
	    }
	    catch (NumberFormatException e)
	    {
		if (Debug.DEBUG) Debug.trace(this, Debug.DP2, e);
	    }
	}
    }
    
}

/* $Log: NetworkCustomizer.java,v $
/* Revision 1.5  2001/07/29 22:37:36  smulloni
/* fix to NetworkCustomizer; workaround to bad design of ServerData
/*
/* Revision 1.4  2001/07/26 18:35:33  smulloni
/* patched HTTPClient so jikes doesn't issue warnings about catching Exception;
/* replaced PropertyChangeEvents with ChangeEvents in PreferencesCustomizer and
/* related classes; incremented app version to 1.0.2.3.
/*
/* Revision 1.3  2001/06/13 01:09:03  smulloni
/* Some improvements to the network customizer (which is still buggy).
/* Also improvements for using the doAllprop switch, and for the layout of
/* the PropertiesDialog.
/*
/* Revision 1.2  2001/06/09 03:09:09  smulloni
/* added server customization to the NetworkCustomizer.  Buggy.
/*
/* Revision 1.1  2001/02/06 22:15:04  smulloni
/* add two example files and moved GeneralCustomizer to NetworkCustomizer
/* */
