/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Iterator;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.text.JTextComponent;
import org.skunk.dav.client.DAVConnectionPool;
import org.skunk.dav.client.DAVConstants;
import org.skunk.swing.text.EntryDocument;
import org.skunk.swing.text.PositiveIntegerEntryFilter;

public class NewConnectionDialog extends JDialog implements ActionListener
{
    private HashMap textfields=new HashMap();
    private JButton okButton;
    private static HashMap defaultData=new HashMap();
    private static NewConnectionDialog dialog;
    private ServerData sd=null;
    private static final String PROTOCOL="protocol";
    private static final String HOST="host";
    private static final String PORT="port";
    private static final String PATH="path";
    private static final String USERNAME="username";
    private static final String PASSWORD="password";
    private static final String OK="OK";
    private static final String CANCEL="CANCEL";
    private JComboBox protocolChooser;

    static 
    {
	defaultData.put(PROTOCOL, DAVConstants.HTTP);
	defaultData.put(HOST, "");
	defaultData.put(PORT, "80");
	defaultData.put(PATH, "/");
	defaultData.put(USERNAME, "");
	defaultData.put(PASSWORD, "");
    }
	

    private  NewConnectionDialog(Frame owner, String title)
    {
	super(owner, title, true);
	init();
    }

    private void init()
    {
	Container contentPane=getContentPane();
	contentPane.setLayout(new BorderLayout());
	contentPane.add(getForm(), BorderLayout.CENTER);
	contentPane.add(getButtons(), BorderLayout.SOUTH);
	getRootPane().setDefaultButton(okButton);
	pack();
    }

    private JPanel getForm()
    {
	JPanel formPanel=new JPanel();
	formPanel.setLayout(new GridBagLayout());
	GridBagConstraints gbc=new GridBagConstraints();
	gbc.gridx=0;
	gbc.gridy=0;
	gbc.gridwidth=1;
	gbc.gridheight=1;
	gbc.fill=gbc.NONE;
	gbc.insets=new Insets(2, 2, 2, 2);
	gbc.ipadx=1;
	gbc.ipady=1;
	JLabel tmpLabel;

	gbc.anchor=gbc.EAST;
	tmpLabel=new JLabel(ResourceManager.getMessage(ResourceManager.PROTOCOL_PROMPT),
			    SwingConstants.RIGHT);
	formPanel.add(tmpLabel, gbc);

	gbc.gridx++;
	gbc.anchor=gbc.WEST;
	protocolChooser=new JComboBox(getAvailableProtocols());
	formPanel.add(protocolChooser, gbc);
	gbc.gridx=0;
	gbc.gridy++;

	JTextField tmpText;

	gbc.anchor=gbc.EAST;	
	tmpLabel=new JLabel(ResourceManager.getMessage(ResourceManager.HOST_PROMPT), 
			    SwingConstants.RIGHT);
	formPanel.add(tmpLabel, gbc);

	gbc.gridx++;
	gbc.anchor=gbc.WEST;
	tmpText=new JTextField(22);
	textfields.put(HOST, tmpText);
	formPanel.add(tmpText, gbc);

	gbc.gridx=0;
	gbc.gridy++;
	gbc.anchor=gbc.EAST;
	tmpLabel=new JLabel(ResourceManager.getMessage(ResourceManager.PORT_PROMPT), 
			    SwingConstants.RIGHT);
	formPanel.add(tmpLabel, gbc);

	gbc.gridx++;
	gbc.anchor=gbc.WEST;
	tmpText=new JTextField(5);
	tmpText.setDocument(new EntryDocument(new PositiveIntegerEntryFilter(0, ((int)Math.pow(2, 16))-1)));
	tmpText.setText(defaultData.get(PORT).toString());
	textfields.put(PORT, tmpText);
	formPanel.add(tmpText, gbc);

	gbc.gridx=0;
	gbc.gridy++;
	gbc.anchor=gbc.EAST;
	tmpLabel=new JLabel(ResourceManager.getMessage(ResourceManager.PATH_PROMPT), 
			    SwingConstants.RIGHT);
	formPanel.add(tmpLabel, gbc);
	
	gbc.gridx++;
	gbc.anchor=gbc.WEST;
	tmpText=new JTextField(22);
	tmpText.setText(defaultData.get(PATH).toString());
	textfields.put(PATH, tmpText);
	formPanel.add(tmpText, gbc);

	gbc.gridx=0;
	gbc.gridy++;
	gbc.anchor=gbc.EAST;
	tmpLabel=new JLabel(ResourceManager.getMessage(ResourceManager.USERNAME_PROMPT), 
			    SwingConstants.RIGHT);
	formPanel.add(tmpLabel, gbc);
	
	gbc.gridx++;
	gbc.anchor=gbc.WEST;
	tmpText=new JTextField(22);
	textfields.put(USERNAME, tmpText);
	formPanel.add(tmpText, gbc);

	gbc.gridx=0;
	gbc.gridy++;
	gbc.anchor=gbc.EAST;
	tmpLabel=new JLabel(ResourceManager.getMessage(ResourceManager.PASSWORD_PROMPT), 
			    SwingConstants.RIGHT);
	formPanel.add(tmpLabel, gbc);

	gbc.gridx++;
	gbc.anchor=gbc.WEST;
	tmpText=new JPasswordField(22);
	textfields.put(PASSWORD, tmpText);
	formPanel.add(tmpText, gbc);
	
	formPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
						ResourceManager.getMessage(ResourceManager.ADD_CONNECTION_PROMPT),
						TitledBorder.LEFT, 
						TitledBorder.TOP));
	return formPanel;
    }

    private Component getButtons()
    {
	Box buttonBox=Box.createHorizontalBox();
	okButton=new JButton(ResourceManager.getMessage(ResourceManager.OK));
	okButton.setActionCommand(OK);
	okButton.addActionListener(this);
	buttonBox.add(Box.createHorizontalGlue());
	buttonBox.add(okButton);
	buttonBox.add(Box.createHorizontalGlue());
	JButton tmpButton=new JButton(ResourceManager.getMessage(ResourceManager.CANCEL));
	tmpButton.setActionCommand(CANCEL);
	tmpButton.addActionListener(this);
	buttonBox.add(tmpButton);
	buttonBox.add(Box.createHorizontalGlue());
	return buttonBox;
    }

    public void actionPerformed(ActionEvent ae)
    {
	if (ae.getActionCommand().equals(CANCEL))
	{
	    sd=null;
	}
	else 
	{
	    boolean useSSL=protocolChooser.getSelectedItem().toString().equals(DAVConstants.HTTPS);
	    String host=getValue(HOST);
	    try
	    {
		int port= Integer.parseInt(getValue(PORT));
		String path=getValue(PATH);
		String username=getValue(USERNAME);
		String password=getValue(PASSWORD);
		
		sd=new ServerData(useSSL, host, port, path, username, password);
	    }
	    catch (NumberFormatException nafta)
	    {
		sd=null;
	    }

	}
	protocolChooser.setSelectedIndex(0);
	resetTextFields();
	hide();	
    }

    private Object[] getAvailableProtocols()
    {
	if (DAVConnectionPool.canSupportSSL())
	    return new Object[] { DAVConstants.HTTP, DAVConstants.HTTPS } ;
	else return new Object[] { DAVConstants.HTTP };
    }

    private String getValue(String key)
    {
	return ((JTextField)textfields.get(key)).getText().trim();
    }

    private void resetTextFields()
    {
	for (Iterator it=textfields.keySet().iterator();it.hasNext();)
	{
	    String key=it.next().toString();
	    JTextComponent txtfld=(JTextComponent)textfields.get(key);
	    if (key.equals(HOST))
		txtfld.setText(defaultData.get(HOST).toString());
	    else if (key.equals(PORT)) 
		txtfld.setText(defaultData.get(PORT).toString());
	    else if (key.equals(PATH))
		txtfld.setText(defaultData.get(PATH).toString());
	    else txtfld.setText("");
	}
    }

    public static ServerData getNewServerData(Frame owner)
    {
	if (dialog==null) 
	    dialog=new NewConnectionDialog(owner, 
					   ResourceManager.getMessage(ResourceManager.ADD_CONNECTION_TITLE));
	dialog.setLocationRelativeTo(owner);
	dialog.show();
	if (dialog.sd!=null)
	    return (ServerData) dialog.sd.clone();
	else return null;
    }

}

/* $Log: NewConnectionDialog.java,v $
/* Revision 1.10  2001/09/01 14:42:59  smulloni
/* fix for port entry upper limit being too low on add connection dialog
/* (thanks to Will Radcliffe for pointing this out).
/*
/* Revision 1.9  2001/01/05 08:01:12  smulloni
/* changes to the connection gui for the Explorer; added depth configurability to
/* propfind in the jpython test script; added an experimental "allprop" system
/* property which affects the propfind query type
/*
/* Revision 1.8  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.7  2000/12/18 23:22:47  smulloni
/* added optional SSL capability to the gui application.  Separate make targets
/* exist for building it will SSL support.
/*
/* Revision 1.6  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.5  2000/11/09 23:34:58  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
