/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
//import java.beans.PropertyChangeEvent;
//import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.skunk.trace.Debug;

public class PreferencesPanel extends JPanel
{
    private Explorer ex;
    private JTabbedPane preferencesPane;
    private JButton tryButton, revertButton, okButton, cancelButton, defaultButton;
    private JDialog prefDialog;
    private ArrayList customizers;
    private ChangeListener dirtListener;

    public PreferencesPanel()
    {
	super();
	this.ex=ex;
	customizers=new ArrayList();
	dirtListener=new DirtListener();
	initComponents();
    }

    private void initComponents()
    {
	setLayout(new BorderLayout());
	preferencesPane=new JTabbedPane(SwingConstants.TOP);
	add(preferencesPane, BorderLayout.CENTER);
	Box buttonBox =Box.createHorizontalBox();
	buttonBox.add(Box.createHorizontalGlue());
	
	tryButton=new JButton(ResourceManager.getMessage(ResourceManager.TRY));
	tryButton.addActionListener(new TryAction());
	tryButton.setEnabled(false);
	buttonBox.add(tryButton);
	buttonBox.add(Box.createHorizontalGlue());

	revertButton=new JButton(ResourceManager.getMessage(ResourceManager.REVERT));
	revertButton.addActionListener(new RevertAction());
	revertButton.setEnabled(false);
	buttonBox.add(revertButton);
	buttonBox.add(Box.createHorizontalGlue());
	
	defaultButton=new JButton(ResourceManager.getMessage(ResourceManager.DEFAULT));
	defaultButton.addActionListener(new DefaultAction());
	buttonBox.add(defaultButton);
	buttonBox.add(Box.createHorizontalGlue());

	okButton=new JButton(ResourceManager.getMessage(ResourceManager.OK));
	okButton.addActionListener(new OKAction());
	buttonBox.add(okButton);
	buttonBox.add(Box.createHorizontalGlue());

	cancelButton=new JButton(ResourceManager.getMessage(ResourceManager.CANCEL));
	cancelButton.addActionListener(new CancelAction());
	buttonBox.add(cancelButton);
	buttonBox.add(Box.createHorizontalGlue());	

	add(buttonBox, BorderLayout.SOUTH);
    }

    public void showDialog()
    {
	if (prefDialog==null)
	{
	    prefDialog=new JDialog();
	    prefDialog.setTitle(ResourceManager.getMessage(ResourceManager.PREFERENCES_KEY));
	    prefDialog.setContentPane(this);
	    prefDialog.pack();
	}
	prefDialog.setLocationRelativeTo(ex);
	prefDialog.setVisible(true);
    }

    public PreferencesCustomizer getCurrentCustomizer()
    {
	return (PreferencesCustomizer) customizers.get(preferencesPane.getSelectedIndex());
    }

    public void setCurrentCustomizer(PreferencesCustomizer currentOne)
    {
	if (customizers.contains(currentOne))
	    preferencesPane.setSelectedIndex(customizers.indexOf(currentOne));
    }

    public void setCurrentCustomizer(String customizerName)
    {
	for (Iterator it=customizers.iterator();it.hasNext();)
	{
	    PreferencesCustomizer pc=(PreferencesCustomizer)it.next();
	    if (customizerName.equals(pc.getTitle()))
	    {
		setCurrentCustomizer(pc);
		return;
	    }
	}
    }

    public void addPreferencesCustomizer(PreferencesCustomizer pc)
    {
	if (!customizers.contains(pc))
	{
	    pc.addDirtListener(dirtListener);
	    customizers.add(pc);
	    preferencesPane.addTab(pc.getTitle(), pc.getCustomizer());
	}
    }

    class OKAction implements ActionListener
    {
	public void actionPerformed(ActionEvent e)
	{
	    getCurrentCustomizer().ok();
	    prefDialog.setVisible(false);
	    tryButton.setEnabled(false);
	    revertButton.setEnabled(false);
	    prefDialog.dispose();
	}
    }

    class CancelAction implements ActionListener
    {
	public void actionPerformed(ActionEvent e)
	{
	    getCurrentCustomizer().cancel();
	    prefDialog.setVisible(false);
	    tryButton.setEnabled(false);
	    revertButton.setEnabled(false);
	    prefDialog.dispose();
	}
    }

    class TryAction implements ActionListener
    {
	public void actionPerformed(ActionEvent e)
	{
	    getCurrentCustomizer().tryOn();
	    revertButton.setEnabled(true);
	    tryButton.setEnabled(false);
	}
    }

    class RevertAction implements ActionListener
    {
	public void actionPerformed(ActionEvent e)
	{
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			getCurrentCustomizer().revert();
			revertButton.setEnabled(false);
			tryButton.setEnabled(false);
		    }
		});
	}
    }

    class DefaultAction implements ActionListener
    {
	public void actionPerformed(ActionEvent e)
	{
	    getCurrentCustomizer().revertToDefault();
	}
    }

    class DirtListener implements ChangeListener
    {
	public void stateChanged(ChangeEvent cheese)
	{
	    Debug.trace(this, Debug.DP3, "in stateChanged({0})", cheese);
	    Object source=cheese.getSource();
	    if (!source.equals(getCurrentCustomizer()))
	    {
		Debug.trace(this, Debug.DP3, "change event from weird place: {0}", source);
		return;
	    }
	    tryButton.setEnabled(getCurrentCustomizer().isDirty());
	}
    }
}

/* $Log: PreferencesPanel.java,v $
/* Revision 1.10  2001/07/26 18:35:33  smulloni
/* patched HTTPClient so jikes doesn't issue warnings about catching Exception;
/* replaced PropertyChangeEvents with ChangeEvents in PreferencesCustomizer and
/* related classes; incremented app version to 1.0.2.3.
/*
/* Revision 1.9  2001/02/06 00:11:18  smulloni
/* struggle, perhaps futile, with the TextEditorCustomizer and other implicated
/* classes
/*
/* Revision 1.8  2001/01/05 08:01:12  smulloni
/* changes to the connection gui for the Explorer; added depth configurability to
/* propfind in the jpython test script; added an experimental "allprop" system
/* property which affects the propfind query type
/*
/* Revision 1.7  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.6  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.5  2000/11/09 23:34:58  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
