/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import org.skunk.swing.text.EntryDocument;
import org.skunk.swing.text.EntryFilter;

public class StatusInputPanel extends JPanel
{
    private JLabel labelComponent;
    private JTextField textFieldComponent;
    private String labelString;
    private int textFieldLength;
    private ArrayList listeners;

    public StatusInputPanel(String labelString, int textFieldLength)
    {
	super();
	this.labelString=labelString;
	this.textFieldLength=textFieldLength;
	this.listeners=new ArrayList();
	initComponents();
    }

    public StatusInputPanel(String labelString, int textFieldLength, EntryFilter entryFilter)
    {
	this(labelString, textFieldLength);
	setEntryFilter(entryFilter);
    }

    private void initComponents()
    {
	labelComponent=new JLabel(labelString);
	textFieldComponent=new JTextField(textFieldLength);
	textFieldComponent.setMaximumSize(textFieldComponent.getPreferredSize());
	textFieldComponent.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent ae)
		{
		    fireActionEvent();
		}
	    });
	setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
	this.add(labelComponent);
	this.add(Box.createHorizontalStrut(2));
	this.add(textFieldComponent);
	this.add(Box.createHorizontalGlue());		  
    }

    public void focus()
    {
	textFieldComponent.requestFocus();
    }

    public void setEntryFilter(EntryFilter entryFilter)
    {
	textFieldComponent.setDocument(new EntryDocument(entryFilter));
    }

    public String getInputText()
    {
	return textFieldComponent.getText();
    }

    public void setInputText(String inputText)
    {
	textFieldComponent.setText(inputText);
    }

    public void addActionListener(ActionListener listener)
    {
	listeners.add(listener);
    }

    public void removeActionListener(ActionListener listener)
    {
	listeners.remove(listener);
    }

    private void fireActionEvent()
    {
	ActionEvent ae=new ActionEvent(this, ActionEvent.ACTION_PERFORMED, null);
	//this is to prevent concurrent modifications, should an action listener call removeActionListener
	Object[] listenersArray=listeners.toArray();
	for (int i=0;i<listenersArray.length;i++)
	{
	    ((ActionListener)listenersArray[i]).actionPerformed(ae);
	}
    }
}
