/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import org.skunk.assertF.Assertion;
import org.skunk.config.Configurator;
import org.skunk.swing.DualListPanel;
import org.skunk.trace.Debug;

public class TableViewCustomizer extends PreferencesCustomizer
{
    private JPanel customizer;
    private DAVTableHeader[] sourceHeaders, configHeaders;
    private DualListPanel duellist;
    private ListDataListener listDataListener;

    private static final DAVTableHeader[] allHeaders=
    {
	DAVTableHeader.LOCK,
	DAVTableHeader.FILE_NAME,
	DAVTableHeader.CREATION_DATE,
	DAVTableHeader.DISPLAY_NAME,
	DAVTableHeader.CONTENT_LANGUAGE,
	DAVTableHeader.CONTENT_LENGTH,
	DAVTableHeader.CONTENT_TYPE,
	DAVTableHeader.ETAG,
	DAVTableHeader.LAST_MODIFIED,
	DAVTableHeader.EXECUTABLE,
	DAVTableHeader.LOCKTOKEN,
	DAVTableHeader.LOCKTYPE,
	DAVTableHeader.LOCKSCOPE,
	DAVTableHeader.OWNER,
	DAVTableHeader.RESOURCETYPE
    };

    public TableViewCustomizer(String title, Configurator configurator)
    {
	super(title, configurator);
	initConfig();
	initComponents();
    }

    private void initConfig()
    {
	for (Iterator it=Explorer.explorers();it.hasNext();)
	    addConfigTarget(it.next());
	Object val=getConfigurator().getStore().getConfigValue(Explorer.class, 
								Explorer.TABLE_HEADERS_PROPERTY);
	if (val==null)
	{
	    Explorer ex=null; 
	    for (Iterator it=Explorer.explorers();it.hasNext();)
	    {
		ex=(Explorer) it.next(); //any one will do
		break;
	    }		
	    Assertion.assertF((ex!=null), "explorer not null");
	    val=ex.getTableHeaders();
	}
	if (val instanceof DAVTableHeader[])
	    configHeaders=(DAVTableHeader[]) val;
	else configHeaders=new DAVTableHeader[0];
	Debug.trace(this, Debug.DP3, "we got back from config an instance of {0}", val.getClass());
	//create two arrays of headers, source and sink
	sourceHeaders=new DAVTableHeader[allHeaders.length-configHeaders.length];
	int j=0;
	for (int i=0;i<allHeaders.length;i++)
	{
	    if (!inArray(allHeaders[i], configHeaders))
		sourceHeaders[j++]=allHeaders[i];
	}
    }

    private void initComponents()
    {
	customizer=new JPanel(new BorderLayout());	
	String sourceLabel=ResourceManager.getMessage(ResourceManager.TABLE_VIEW_SOURCE_LIST_TITLE);
	String sinkLabel=ResourceManager.getMessage(ResourceManager.TABLE_VIEW_SINK_LIST_TITLE);
	duellist=new DualListPanel(sourceHeaders, configHeaders, sourceLabel, sinkLabel);
	duellist.setListPreferredSize(new Dimension(160, 340));
	listDataListener=new HeaderListener();
	duellist.addListDataListener(listDataListener);
	String borderTitle=ResourceManager.getMessage(ResourceManager.TABLE_VIEW_CUSTOMIZER_BORDER_TITLE);	
	customizer.add(duellist, BorderLayout.NORTH);
	customizer.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
							      borderTitle,
							      TitledBorder.LEFT,
							      TitledBorder.TOP));
    }

    private static boolean inArray(Object obj, Object[] oArray)
    {
	for (int i=0;i<oArray.length;i++)
	    if (obj.toString().equals(oArray[i].toString()))
		return true;
	return false;
    }

    public void revert()
    {
	super.revert();
	resetDualListPanel();
    }	

    private void resetDualListPanel(Object[] values)
    {
	duellist.removeListDataListener(listDataListener);
	if (values !=null && values instanceof Object[])
	    duellist.reset(values);
	else duellist.reset();
	duellist.addListDataListener(listDataListener);
	listDataListener.intervalAdded(null);
    }

    private void resetDualListPanel()
    {
	resetDualListPanel(null);
    }

    public void revertToDefault()
    {
	super.revertToDefault();
	Object val=getConfigurator().getDefaultStore().getConfigValue(Explorer.class, 
								      Explorer.TABLE_HEADERS_PROPERTY);
	if (val instanceof Object[]) 
	{
	    Debug.trace(this, Debug.DP3, "resetting to {0}", new Object[] {val});
	    resetDualListPanel((Object[]) val);
	}
	else
	{
	    Debug.trace(this, Debug.DP3, "default value is unexpected: {0}", new Object[] {val});
	}
    }

    public JComponent getCustomizer()
    {
	return customizer;
    }

    class HeaderListener implements ListDataListener
    {
	public void intervalAdded(ListDataEvent e)
	{
	    Debug.trace(this, Debug.DP3, "in intervalAdded()");
	    _process(e);
	}

	public void intervalRemoved(ListDataEvent e)
	{
	    Debug.trace(this, Debug.DP3, "in intervalRemoved()");
	    _process(e);
	}

	public void contentsChanged(ListDataEvent e)
	{
	    Debug.trace(this, Debug.DP3, "in contentsChanged()");
	    _process(e);
	}

	private void _process(ListDataEvent e)
	{
	    Object[] chosenItems=duellist.getChosenItems();
	    DAVTableHeader[] tmpHeaders=new DAVTableHeader[chosenItems.length];
	    for (int i=0;i<chosenItems.length;i++)
		tmpHeaders[i]=(DAVTableHeader) chosenItems[i];
	    setConfigValue(Explorer.class, Explorer.TABLE_HEADERS_PROPERTY, tmpHeaders);
	    if (e!=null) setDirty(true);
	}
    }
}

/* $Log: TableViewCustomizer.java,v $
/* Revision 1.10  2001/02/06 22:13:40  smulloni
/* first more-or-less working version of syntax highlighting, with customization.
/*
/* Revision 1.9  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.8  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.7  2000/11/10 22:40:08  smullyan
/* added icon to table for resource type; fixes to copy and move; disabling of
/* menu items that are inappropriate.
/*
/* Revision 1.6  2000/11/09 23:35:00  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
