/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.action;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.util.Vector;
import javax.swing.AbstractAction;
import org.skunk.dav.client.gui.ConnectionDialog;
import org.skunk.dav.client.gui.Explorer;
import org.skunk.dav.client.gui.ExplorerMenuBar;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.dav.client.gui.ServerData;
import org.skunk.dav.client.gui.StateMonitor;
import org.skunk.dav.client.gui.StateProperties;

public class ConnectionAction extends AbstractAction
{
    public static final String 
	CONNECTION_MESSAGE=ResourceManager.getMessage(ResourceManager.CONNECTION_PROMPT, 50);
    public static final String 
	CONNECTION_TITLE=ResourceManager.getMessage(ResourceManager.CONNECT_KEY);
    public static final String 
	NEW_CONNECTION_OPTION=ResourceManager.getMessage(ResourceManager.ADD_NEW_CONNECTION_LABEL);
    public static final String 
	REMOVE_CONNECTION_OPTION=ResourceManager.getMessage(ResourceManager.REMOVE_CONNECTION_LABEL);
    public static final String CHOOSE_COMMAND="choose";
    public static final String CONNECT_COMMAND="connect";
    private Explorer ex=null;
    private ServerData sd=null;

    public ConnectionAction()
    {
	super();
    }

    public ConnectionAction(Explorer ex)
    {
	super();
	this.ex=ex;
    }

    public ConnectionAction(Explorer ex, ServerData sd)
    {
	super();
	this.ex=ex;
	this.sd=sd;
    }    
    
    protected static Object[] getServerArray()
    {
	Vector v=(Vector)ServerData.getServers().clone();
	v.addElement(NEW_CONNECTION_OPTION);
	v.addElement(REMOVE_CONNECTION_OPTION);
	return v.toArray();
    }
        
    public void actionPerformed(ActionEvent ae)
    {
	String actionCommand=(ae==null) 
	    ? CONNECT_COMMAND 
	    : ae.getActionCommand();

	if (CHOOSE_COMMAND.equals(actionCommand))
	{
	    Explorer localExplorer=(ex==null)
		? Explorer.guessActiveExplorer()
		: ex;	    
	    ConnectionDialog conDialog=new ConnectionDialog(localExplorer);
	    if (conDialog.show())
	    {
		ServerData localServerData=conDialog.getServerData();
		if (localServerData!=null)
		    localExplorer.addConnectionNode(localServerData, 
						    new ConnectionRunner(localExplorer));
	    }
	}
	else if (CONNECT_COMMAND.equals(actionCommand))
	{
	    if (ex!=null && sd!=null)
	    {
		ex.addConnectionNode(sd, new ConnectionRunner(ex));
	    }	    
	}
  }

    private static class ConnectionRunner implements Runnable
    {
	private Explorer ex;

	ConnectionRunner(Explorer ex)
	{
	    this.ex=ex;
	}

	public void run()
	{
	    StateMonitor.setProperty(StateProperties.CONNECTED,
				     new Boolean(true),
				     this.ex);
	}
    }
}

/* $Log: ConnectionAction.java,v $
/* Revision 1.13  2001/06/05 01:09:53  smulloni
/* fixed bug wtih failed connection attempts, which used to set the
/* connection flag as if the connection had succeeded.
/*
/* Revision 1.12  2001/05/30 03:49:35  smulloni
/* Some bug fixes and improvements to authentication handling.  Users are now
/* prompted for a password the first time they connect to a given server
/* in a session; if the password is rejected, they are asked again.
/* Users also now have the option of whether a given password will be remembered
/* or not.
/*
/* Revision 1.11  2001/01/05 08:01:12  smulloni
/* changes to the connection gui for the Explorer; added depth configurability to
/* propfind in the jpython test script; added an experimental "allprop" system
/* property which affects the propfind query type
/*
/* Revision 1.10  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.9  2000/12/04 23:51:16  smulloni
/* added ImageViewer; fixed word in SimpleTextEditor
/*
/* Revision 1.8  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.7  2000/11/18 04:36:05  smullyan
/* work on StateMonitor and related functionality.
/*
/* Revision 1.6  2000/11/10 22:40:09  smullyan
/* added icon to table for resource type; fixes to copy and move; disabling of
/* menu items that are inappropriate.
/*
/* Revision 1.5  2000/11/09 23:35:01  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
