/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.action;

import java.awt.event.ActionEvent;
import java.io.File;
import java.net.URLDecoder;
import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import org.skunk.assertF.Assertion;
import org.skunk.dav.client.DAVConstants;
import org.skunk.dav.client.DAVFile;
import org.skunk.dav.client.gui.DAVFileChooser;
import org.skunk.dav.client.gui.DAVTreeNode;
import org.skunk.dav.client.gui.Explorer;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.dav.client.gui.ServerData;
import org.skunk.trace.Debug;


public class CopyAction extends AbstractAction
{
    private Explorer ex;
    private DAVTreeNode node;
    private DAVFile file;
    private String destinationURL;

    public CopyAction(Explorer ex, DAVTreeNode node, 
		      DAVFile file, String destinationURL)
    {
	this.ex=ex;
	this.node=node;
	this.file=file;
	this.destinationURL=destinationURL;
    }

    public CopyAction(Explorer ex, DAVFile file)
    {
	this(ex, null, file, null);
    }

    public CopyAction(Explorer ex)
    {
	this(ex, null, null, null);
    }

    public void actionPerformed(ActionEvent ae)
    {
	DAVTreeNode localNode;
	DAVFile localFile=(file!=null)
	    ? file 
	    : ex.getSelectedTableFile();
	if (localFile==null)
	{
	    DAVFileChooser.ChoiceStruct choice=ex.chooseFile(null, 
							     getDialogTitle(),
							     false,
							     null);
	    if (choice==null)
	    {
		Debug.trace(this, Debug.DP3, "user aborted copy");
		return;
	    }
	    DAVTreeNode leafNode=choice.getSelectedLeaf();  
	    if (leafNode==null)
	    {
		Debug.trace(this, Debug.DP3, "user aborted copy");
		return;
	    }

	    localFile=leafNode.getDAVFile();
	    if (localFile==null)
	    {
		Debug.trace(this, Debug.DP3, "selected node has null file: {0}",
			    leafNode);
		return;
	    }
	    //user may have navigated to any node; need to find which one
	    //corresponds to the DAVFile
	    localNode=ex.getNodeForFile(localFile);
	}
	else
	{
	    localNode=(node!=null) 
		? node 
		: ex.getSelectedNode();
	    Assertion.assertF((localNode!=null), "node is not null");
	}	    
	String localDestinationURL=(destinationURL!=null) 
	    ? destinationURL 
	    : obtainDestination(localFile.getUnicodeFileName(), localNode); //sachiho modified
	    //: obtainDestination(URLDecoder.decode(localFile.getUnicodeFileName()), localNode);
	if (localDestinationURL!=null)
	    execute(ex, localNode, localFile, localDestinationURL);	    
    }

    protected void execute(Explorer ex, DAVTreeNode node, DAVFile file, String destinationURL)
    {
	ex.copy(node, file, destinationURL);
    }

    protected String getDialogTitle()
    {
	return ResourceManager.getMessage(ResourceManager.COPY_DIALOG_TITLE);
    }

    protected Explorer getExplorer()
    {
	return ex;
    }

    protected String obtainDestination(String filename, DAVTreeNode node)
    {
	String title=getDialogTitle();
	DAVFileChooser.ChoiceStruct choice=ex.chooseFile(node, title, true, filename); 
	if (choice==null)
	    return null;
	DAVTreeNode parentNode=choice.getParentNode();
	String newName=choice.getFilename();
	if (parentNode==null || newName==null)
	    return null;
	String destination=new StringBuffer()
	    .append(parentNode.getDAVFile().getFullNameWithProtocol())
	    .append(newName)
	    .toString();

	Debug.trace(this, Debug.DP3, "returning from obtainDestination(): {0}",
		    destination);
	return destination;
    }
}

/* $Log: CopyAction.java,v $
/* Revision 1.11  2001/07/18 03:23:07  smulloni
/* added URL decoding to filename display in the table model and copy/move/rename dialogs;
/* not in the PropertiesDialog table, however.
/*
/* Revision 1.10  2001/01/03 20:11:31  smulloni
/* the DAVFileChooser now replaces JFileChooser for remote file access.
/* DAVMethod now has a protocol property.
/*
/* Revision 1.9  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.8  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.7  2000/11/10 22:40:09  smullyan
/* added icon to table for resource type; fixes to copy and move; disabling of
/* menu items that are inappropriate.
/*
/* Revision 1.6  2000/11/09 23:35:01  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
