/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.action;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import org.skunk.assertF.Assertion;
import org.skunk.dav.client.gui.Explorer;
import org.skunk.dav.client.gui.ExplorerMenuBar;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.dav.client.gui.ServerData;
import org.skunk.dav.client.gui.StateMonitor;
import org.skunk.dav.client.gui.StateProperties;
import org.skunk.trace.Debug;

public class DisconnectionAction extends AbstractAction
{
    Explorer ex;
    ServerData sd;

    public DisconnectionAction(Explorer ex, ServerData sd)
    {
	this.ex=ex;
	this.sd=sd;
    }

    public DisconnectionAction(Explorer ex)
    {
	this(ex, null);
    }
    
    public void actionPerformed(ActionEvent ae)
    {	
	if (sd!=null)
	    disconnect(sd);
	else if (ex.getConnectedServerCount()==1)
	{
	    Debug.trace(this, Debug.DP3, 
			"only one connected server, will disconnect");
	    ServerData[] allServers=ex.getConnectedServers();
	    Assertion.assertF((allServers.length==1), 
			     "length of array of connected servers is 1");
	    disconnect(allServers[0]);
	}
	else	    
	{
	    Debug.trace(this, Debug.DP3, "finding selected connection");
	    ServerData selectedServer=ex.getSelectedConnection();
	    if (selectedServer!=null) 
		disconnect(selectedServer);
	    else 
	    {
		//it would be preferable to show a choice dialog here rather than an error message.
		String message=ResourceManager.getMessage(ResourceManager.DISCONNECTION_ERROR_MESSAGE);
		String title=ResourceManager.getMessage(ResourceManager.DISCONNECTION_ERROR_TITLE);
		JOptionPane.showMessageDialog(ex, message, title, JOptionPane.ERROR_MESSAGE);
	    }
	}	
    }

    private void disconnect(ServerData sd)
    {
	ex.removeConnectionNode(sd);
	if (ex.getConnectedServerCount()==0)
	{
	    //ExplorerMenuBar.getMenuBar(ex).setConnected(false);
	    StateMonitor.setProperty(StateProperties.CONNECTED, 
				     new Boolean(false), 
				     ex);
	}
    }
}

/* $Log: DisconnectionAction.java,v $
/* Revision 1.8  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.7  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.6  2000/11/18 04:36:05  smullyan
/* work on StateMonitor and related functionality.
/*
/* Revision 1.5  2000/11/10 22:40:09  smullyan
/* added icon to table for resource type; fixes to copy and move; disabling of
/* menu items that are inappropriate.
/*
/* Revision 1.4  2000/11/09 23:35:02  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
