/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.action;

import java.awt.event.ActionEvent;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
//import javax.swing.JLabel;
//import javax.swing.JOptionPane;
import org.skunk.dav.client.DAVFile;
import org.skunk.dav.client.gui.DAVFileChooser;
import org.skunk.dav.client.gui.DAVTreeNode;
import org.skunk.dav.client.gui.Explorer;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.trace.Debug;

public class DownloadAction extends AbstractAction
{
    private Explorer ex;
    private DAVFile file;

    public DownloadAction(Explorer ex)
    {
	this(ex, null);
    }

    public DownloadAction(Explorer ex, DAVFile file)
    {
	this.ex=ex;
	this.file=file;
    }

    public void actionPerformed(ActionEvent ae)
    {
	DAVFile localDAVFile=(file!=null) ? file : ex.getSelectedTableFile();
	if (localDAVFile==null || localDAVFile.isCollection())
	{
	    String title=ResourceManager.getMessage(ResourceManager.DOWNLOAD_CHOOSER_DIALOG_TITLE);
	    DAVFileChooser.ChoiceStruct choice=ex.chooseFile(null, title, false, null);
	    if (choice==null)
	    {
		Debug.trace(this, Debug.DP3, "user aborted download");
		return;
	    }
	    DAVTreeNode selectedNode=choice.getSelectedLeaf();
	    if (selectedNode!=null)
	    {
		localDAVFile=selectedNode.getDAVFile();
	    }
	    else
	    {
		Debug.trace(this, Debug.DP3, "no file selected");
		return;
	    }

	}
	//get the local target
	File f=null;
	JFileChooser chooser=new JFileChooser();
	chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

	// sachiho added; use new method getUnicodeFileName()
//	chooser.setSelectedFile(new File(chooser.getCurrentDirectory(), 
//					 localDAVFile.getFileName()));
	chooser.setSelectedFile(new File(chooser.getCurrentDirectory(), 
					 localDAVFile.getUnicodeFileName()));
	if (chooser.showSaveDialog(ex)==JFileChooser.APPROVE_OPTION 
	    && (f=chooser.getSelectedFile())!=null)
	{
	    byte[] fileBytes=ex.get(localDAVFile);
	    try
	    {
		BufferedOutputStream boss=new BufferedOutputStream(new FileOutputStream(f));
		for (int i=0;i<fileBytes.length;i+=2048)
		    boss.write(fileBytes, i, Math.min(2048, fileBytes.length-i));
		boss.flush();
		boss.close();
		// Elias Sinderson pointedly suggested that this dialog was unnecessary.
// 		String message=ResourceManager.getMessage(ResourceManager.DOWNLOAD_SUCCESS_MESSAGE, 
// 							  new Object[] {localDAVFile.getFullName(), 
// 									f.getAbsolutePath()});
// 		String title=ResourceManager.getMessage(ResourceManager.DOWNLOAD_SUCCESS_DIALOG_TITLE);
// 		JOptionPane.showMessageDialog(ex, new JLabel(message),title, JOptionPane.INFORMATION_MESSAGE);
	    }
	    catch (IOException oyVeh)
	    {
		// probably I should have a dialog here. TO BE DONE
		Debug.trace(this, Debug.DP2, oyVeh);
	    }
	}
	

    }
}

/* $Log: DownloadAction.java,v $
/* Revision 1.8  2001/07/25 21:00:50  smulloni
/* removed file download confirmation dialog.
/*
/* Revision 1.7  2001/01/03 20:11:31  smulloni
/* the DAVFileChooser now replaces JFileChooser for remote file access.
/* DAVMethod now has a protocol property.
/*
/* Revision 1.6  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.5  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.4  2000/11/09 23:35:02  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
