/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.action;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import org.skunk.dav.client.gui.AppContext;
import org.skunk.dav.client.gui.Buffer;
import org.skunk.dav.client.gui.Explorer;
import org.skunk.dav.client.gui.ExplorerApp;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.dav.client.gui.View;
import org.skunk.dav.client.gui.editor.DAVEditor;

public class ExitAction extends AbstractAction
{
    public void actionPerformed(ActionEvent ae)
    {
	//if there are open, unsaved buffers, prompt user.
	//if user does not cancel, close and unlock all buffers.
	//I am not performing any saves here.
	ArrayList cleanBuffers=new ArrayList();
	ArrayList dirtyBuffers=new ArrayList();
	int numViews=ExplorerApp.getAppContext().getViewCount();

	for (int i=0;i<numViews;i++)
	{
	    View v=ExplorerApp.getAppContext().getView(i);
	    for (Iterator it=v.getDockedBuffers();it.hasNext();)
	    {
		Buffer b=(Buffer)it.next();
		if (b instanceof Explorer)
		    continue;
		if (b instanceof DAVEditor
		    && ((DAVEditor)b).isDirty())
		{
		    dirtyBuffers.add(b);
		}
		else cleanBuffers.add(b);
	    }
	}
	if (dirtyBuffers.size()>0)
	{
	    String title=ResourceManager.getMessage(ResourceManager.DIRTY_BUFFER_ON_EXIT_TITLE);
	    String message=ResourceManager.getMessage(ResourceManager.DIRTY_BUFFER_ON_EXIT_MESSAGE);
	    int option=JOptionPane.showConfirmDialog(ExplorerApp.getAppContext().getCurrentView().getComponent(),
					  message,
					  title,
					  JOptionPane.OK_CANCEL_OPTION,
					  JOptionPane.WARNING_MESSAGE);
	    if (option==JOptionPane.CANCEL_OPTION)
		return;					  
	}
	//CloseBufferAction does not require an ActionEvent.
	for (Iterator it=dirtyBuffers.iterator();it.hasNext();)
	{
	    new CloseBufferAction((Buffer)it.next(), true).actionPerformed(null);
	}
	for (Iterator it=cleanBuffers.iterator();it.hasNext();)
	{
	    new CloseBufferAction((Buffer)it.next(), true).actionPerformed(null);
	}
	System.exit(0);
    }
}

/* $Log: ExitAction.java,v $
/* Revision 1.6  2001/01/02 20:43:03  smulloni
/* stopped trying to undock the Explorer component in ExitAction, which is a no-no.
/*
/* Revision 1.5  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.4  2000/12/19 19:22:22  smulloni
/* some tweaks: ExitAction now cleans up the app's buffers.  The app now does
/* not resize every time a buffer is added or removed.
/*
/* Revision 1.3  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.2  2000/11/09 23:35:02  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
