/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.action;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import org.skunk.dav.client.DAVFile;
import org.skunk.dav.client.gui.DAVTreeNode;
import org.skunk.dav.client.gui.Explorer;
import org.skunk.trace.Debug;

public class LockAction extends AbstractAction
{
    private Explorer ex;
    private DAVTreeNode node;
    private DAVFile file;

    public LockAction(Explorer ex)
    {
	this(ex, null, null);
    }

    public LockAction(Explorer ex, DAVFile file)
    {
	this(ex, null, file);
    }

    public LockAction(Explorer ex, DAVTreeNode node, DAVFile file)
    {
	this.ex=ex;
	this.node=node;
	this.file=file;
    }

    public void actionPerformed(ActionEvent ae)
    {
	DAVTreeNode localNode=(node==null) ? ex.getSelectedNode() : node;

	if (localNode==null)
	{
	    Debug.trace(this, Debug.DP3, "null node in LockAction");
	    return;
	}
	DAVFile localFile=(file==null) ? ex.getSelectedTableFile() : file;
	if (localFile==null)
	{
	    Debug.trace(this, Debug.DP3, "null file in LockAction");
	    return;
	}
	if (localFile.isCollection())
	{
	    Debug.trace(this, Debug.DP3, 
			"should now return if collection-locking is not permitted");
	}

	Debug.trace(this, Debug.DP3, 
		    "should now lock file");
	ex.lock(localNode, localFile);
    }
}

/* $Log: LockAction.java,v $
/* Revision 1.5  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.4  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.3  2000/11/09 23:35:03  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
