/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.action;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import org.skunk.dav.client.DAVFile;
import org.skunk.dav.client.gui.Explorer;
import org.skunk.dav.client.gui.DAVTreeNode;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.trace.Debug;


/**
 *
 */
   public class NewCollectionAction extends AbstractAction
{
    private Explorer ex;
    private DAVTreeNode node;
    private String collName;

    public NewCollectionAction(Explorer explorer)
    {
	this(explorer, null, null);
    }

    public NewCollectionAction(Explorer explorer, DAVTreeNode node)
    {
	this(explorer, node, null);
    }

    public NewCollectionAction(Explorer explorer, DAVTreeNode node, String collName)
    {
	this.ex=explorer;
	this.node=node;
	this.collName=collName;
    }

    public void actionPerformed(ActionEvent ae)
    {
	DAVTreeNode localNode=null;
	String localCollName=null;
	if (node==null)
	{
	    //obtain current path from explorer, and collection name from user through dialog
	    localNode=ex.getSelectedNode();
	    if (localNode==null)
	    {
		//the action should actually be disabled in this case.
		Debug.trace(this, Debug.DP3, 
			    "error -- new collection action requested outside of DAV connection");
		return;
	    }
	}
	else localNode=node;
	if (collName==null)
	{	
	    String message=ResourceManager.getMessage(ResourceManager.NEW_COLLECTION_PROMPT);
	    String title= ResourceManager.getMessage(ResourceManager.NEW_COLLECTION_DIALOG_TITLE);
	    localCollName=JOptionPane.showInputDialog(ex, message, title, JOptionPane.QUESTION_MESSAGE);
	}
	else localCollName=collName;

	if (localCollName==null || localCollName.trim().length()==0)
	{
	    Debug.trace(this, Debug.DP3, "user has cancelled or submitted whitespace");
	    return;
	}
	ex.createCollection(localNode, localCollName);
    }
}

/* $Log: NewCollectionAction.java,v $
/* Revision 1.7  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.6  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.5  2000/11/09 23:35:03  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
