/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.action;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import org.skunk.dav.client.gui.DAVTreeNode;
import org.skunk.dav.client.gui.Explorer;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.trace.Debug;

public class NewFileAction extends AbstractAction
{
    private Explorer ex;

    public NewFileAction(Explorer ex)
    {
	this.ex=ex;
    }
    public void actionPerformed(ActionEvent ae)
    {
	//obtain current path from explorer, and file name from user through dialog
	DAVTreeNode node=ex.getSelectedNode();
	if (node!=null)
	{	
	    String message=ResourceManager.getMessage(ResourceManager.NEW_FILE_PROMPT);
	    String title= ResourceManager.getMessage(ResourceManager.NEW_FILE_DIALOG_TITLE);
	    String fileName=JOptionPane.showInputDialog(ex, message, title, JOptionPane.QUESTION_MESSAGE);

	if (fileName==null || fileName.trim().length()==0)
	{
	    Debug.trace(this, Debug.DP3, "user has cancelled or submitted whitespace");
	    return;
	}

		// sachiho added ; convert UTF16 to UTF8 and url-encode (but space will become %20)
		String ufileName="";
		try {
			ufileName = java.net.URLEncoder.encode(fileName , "UTF-8");
			//System.out.println(ufileName);
		}
		catch(Exception e) {
			System.err.println(e);
		}

	    ex.createFile(node, ufileName.replaceAll("\\+","%20") ); 
//	    ex.createFile(node, fileName); 
	}
	else
	{
	    //the action should actually be disabled in this case.
	    Debug.trace(this, Debug.DP3, 
			"error -- new file action requested outside of DAV connection");
	}

    }
}

/* $Log: NewFileAction.java,v $
/* Revision 1.7  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.6  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.5  2000/11/09 23:35:03  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
