/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.action;

import java.awt.Frame;
import java.awt.Window;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.SwingUtilities;
import org.skunk.dav.client.DAVFile;
import org.skunk.dav.client.gui.Explorer;
import org.skunk.dav.client.gui.PropertiesDialog;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.trace.Debug;

/**
 * PropertiesAction.java
 *
 *
 * Created: Tue Oct 24 11:37:00 2000
 *
 * @author Jacob Smullyan
 * @version $Version$
 */

public class PropertiesAction extends AbstractAction
{
    private DAVFile file;
    private Explorer ex;

    public PropertiesAction(Explorer ex, DAVFile file)
    {
	this.ex=ex;
	this.file=file;
    }

    public PropertiesAction(Explorer ex)
    {
	this(ex, null); 
    }

    public void actionPerformed(ActionEvent e)
    {
	DAVFile localFile=(file!=null) ? file : ex.getCurrentFile();
	if (localFile==null)
	{
	    Debug.trace(this, Debug.DP3, "no file specified for PropertiesAction");
	    return;
	}
	Window w=SwingUtilities.windowForComponent(ex);	
	PropertiesDialog dialog=new PropertiesDialog((w instanceof Frame) 
						     ? (Frame) w 
						     : null, 
						     ex,
						     localFile);
	dialog.setLocationRelativeTo(ex);
	dialog.show();
    }
}

/* $Log: PropertiesAction.java,v $
/* Revision 1.7  2001/01/05 08:01:12  smulloni
/* changes to the connection gui for the Explorer; added depth configurability to
/* propfind in the jpython test script; added an experimental "allprop" system
/* property which affects the propfind query type
/*
/* Revision 1.6  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.5  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.4  2000/11/13 23:28:31  smullyan
/* first pass at a gui for proppatch, added to the propertiese dialog.  Highly
/* problematic still, needs substantial work.
/*
/* Revision 1.3  2000/11/09 23:35:04  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
