/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.action;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.io.IOException;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import org.skunk.dav.client.gui.Explorer;
import org.skunk.dav.client.gui.ExplorerMenuBar;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.dav.client.gui.ServerData;
import org.skunk.trace.Debug;

public class RemoveConnectionAction extends AbstractAction
{
    private Explorer ex;
    
    public static final String REMOVE_CONNECTION_TITLE=ResourceManager.getMessage(ResourceManager.REMOVE_CONNECTION_TITLE);
    public static final String REMOVE_CONNECTION_PROMPT=ResourceManager.getMessage(ResourceManager.REMOVE_CONNECTION_PROMPT)
	+ "                            ";
    public static final String REMOVE_CONNECTION_LABEL=ResourceManager.getMessage(ResourceManager.REMOVE_CONNECTION_LABEL);

    public RemoveConnectionAction(Explorer ex)
    {
	this.ex=ex;
    }
    
    public void actionPerformed(ActionEvent ae)
    {
	Object[] serverArray=ServerData.getServers().toArray();
	Object srcObj=ae.getSource();
	Component parent=(srcObj instanceof Component) ? SwingUtilities.getRoot((Component)srcObj) : null;
	Object value=JOptionPane.showInputDialog(parent,
						 REMOVE_CONNECTION_PROMPT,
						 REMOVE_CONNECTION_TITLE,
						 JOptionPane.PLAIN_MESSAGE,
						 null,
						 serverArray,
						 serverArray[0]
						 );
	if (value!=null && ServerData.getServers().contains(value))
	{
	    try
	    {
		ServerData sd=(ServerData) value;
		if (ex.isConnected(sd))
		    ex.removeConnectionNode(sd);
		ServerData.removeServer(sd);
		ExplorerMenuBar.refreshMenus();
	    }
	    catch (IOException oyVeh)
	    {
		Debug.trace(this, Debug.DP2, oyVeh);
	    }
	}      
    }
}

/* $Log: RemoveConnectionAction.java,v $
/* Revision 1.5  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.4  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.3  2000/11/09 23:35:05  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
