/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.action;

import java.awt.event.ActionEvent;
import javax.swing.JOptionPane;
import org.skunk.dav.client.DAVConstants;
import org.skunk.dav.client.DAVFile;
import org.skunk.dav.client.gui.DAVTreeNode;
import org.skunk.dav.client.gui.Explorer;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.trace.Debug;

public class RenameAction extends MoveAction
{
    public RenameAction(Explorer ex, DAVTreeNode node, 
		      DAVFile file, String destinationURL)
    {
	super(ex, node, file, destinationURL);
    }

    public RenameAction(Explorer ex, DAVFile file)
    {
	super(ex, null, file, null);
    }

    public RenameAction(Explorer ex)
    {
	super(ex, null, null, null);
    }

    protected String getDialogMessage()
    {
	return ResourceManager.getMessage(ResourceManager.RENAME_DIALOG_MESSAGE);
    }

    protected String getDialogTitle()
    {
	return ResourceManager.getMessage(ResourceManager.RENAME_DIALOG_TITLE);
    }

    protected String obtainDestination(String filename, DAVTreeNode node)
    {
	String title=getDialogTitle();
	String message=getDialogMessage();
        Object newName=JOptionPane.showInputDialog(getExplorer(), 
						   message, 
						   title, 
						   JOptionPane.QUESTION_MESSAGE,
						   null,
						   null,
						   filename);
	if (newName==null) 
	    return null;
	newName=newName.toString().trim();
	if (newName.equals(filename) 
	    || newName.equals("")) 
	    return null;

	// sachiho added ;  convert UTF16 to UTF8 and url-encode (but space will become %20)
	String fileName=newName.toString();
	String ufileName="";
	try {
		ufileName = java.net.URLEncoder.encode(fileName , "UTF-8");
	}
	catch(Exception e) {
		System.err.println(e);
	}
	newName=ufileName.replaceAll("\\+","%20");

	String protocol=node.getDAVFile().getProtocol();
	if (protocol==null || (protocol!=DAVConstants.HTTP 
			       && protocol!=DAVConstants.HTTPS))
	    protocol=DAVConstants.HTTP;
	String s=new StringBuffer(protocol)
	    .append("://")
	    .append(node.getDAVFile().getFullName())
	    .append(newName)
	    .toString();
	Debug.trace(this, Debug.DP3, "destination is {0}", s);
	return s;	
    }
}

/* 
$Log: RenameAction.java,v $
Revision 1.6  2001/05/29 19:38:23  smulloni
support for https; sanity checks added for renaming to "" or to the existing
filename.

Revision 1.5  2001/05/29 19:21:09  smulloni
Rename dialogs now default to the original file name.

Revision 1.4  2000/12/19 22:36:05  smulloni
adjustments to preamble.

Revision 1.3  2000/12/03 23:53:26  smulloni
added license and copyright preamble to java files.

Revision 1.2  2000/11/08 23:29:12  smullyan
numerous changes: put actions now check for locktokens; right-click upload;
fixes to lock renderer.
 
*/
