/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.action;

import java.awt.event.ActionEvent;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import org.skunk.dav.client.DAVFile;
import org.skunk.dav.client.gui.DAVFileChooser;
import org.skunk.dav.client.gui.Explorer;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.trace.Debug;

public class UploadAction extends AbstractAction
{
    private Explorer ex;
    private DAVFile file;

    public UploadAction(Explorer ex)
    {
	this(ex, null);
    }

    public UploadAction(Explorer ex, DAVFile file)
    {
	this.ex=ex;
	this.file=file;
    }

    public void actionPerformed(ActionEvent ae)
    {
	//obtain file to upload from a JFileChooser
	JFileChooser chooser=new JFileChooser();
	chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
	File f=null;
	byte[] fileBody=null;
	if (chooser.showOpenDialog(ex)==JFileChooser.APPROVE_OPTION 
	    && (f=chooser.getSelectedFile())!=null)
	{
	    try
	    {
		FileInputStream fizz=new FileInputStream(f);
		int initialSize=(int) Math.min((long) f.length(), Integer.MAX_VALUE);
		ByteArrayOutputStream baos=new ByteArrayOutputStream(initialSize);
		byte[] buffer = new byte[2048];
		int bytesRead;
		while ((bytesRead=fizz.read(buffer))!=-1)
		{
		    baos.write(buffer, 0, bytesRead);
		}
		fileBody=baos.toByteArray();
	    }
	    catch (IOException oyVeh)
	    {
		Debug.trace(this, Debug.DP2, oyVeh);
		return;
	    }
	}	
	else return;
	String fileName=null;
	if (file==null)
	{
	    String title= ResourceManager.getMessage(ResourceManager.UPLOAD_CHOOSER_DIALOG_TITLE);
	    DAVFileChooser.ChoiceStruct choice=ex.chooseFile(null, title, true, f.getName());
	    if (choice==null)
	    {
		Debug.trace(this, Debug.DP3, "user cancelled upload");
		return;
	    }
	    fileName=choice.getFilePath();
	    if (fileName==null)
	    {
		Debug.trace(this, Debug.DP3, "user cancelled upload");
		return;	
	    }
	}
	else
	{
	    fileName=file.getFullName();
	}
	ex.createFile(fileName, fileBody);
    }
}

/* $Log: UploadAction.java,v $
/* Revision 1.9  2001/01/03 20:11:31  smulloni
/* the DAVFileChooser now replaces JFileChooser for remote file access.
/* DAVMethod now has a protocol property.
/*
/* Revision 1.8  2000/12/19 22:36:05  smulloni
/* adjustments to preamble.
/*
/* Revision 1.7  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.6  2000/11/09 23:35:06  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
