/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.editor;

import java.util.ArrayList;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoableEdit;
import javax.swing.undo.UndoableEditSupport;
import javax.swing.undo.UndoManager;

/**
 *  the conventional extension of UndoManager to support 
 *  monitoring its state with a change listener. 
 */
public class DAVEditorUndoManager extends UndoManager
{
    private ArrayList listeners=new ArrayList();
    private ChangeEvent changeEvent=new ChangeEvent(this);
    private boolean inUpdate=false;
    private HarkeningUndoSupport support=new HarkeningUndoSupport(this);

    public synchronized void setLimit(int limit)
    {
	super.setLimit(limit);
	fireChangeEvent();
    }

    public void undoableEditHappened(UndoableEditEvent ugh)
    {
	if (!inUpdate)
	    super.undoableEditHappened(ugh);
	else 
	    support.undoableEditHappened(ugh);
    }

    public synchronized void discardAllEdits()
    {
	super.discardAllEdits();
	fireChangeEvent();
    }

    public synchronized void undo()
	throws CannotUndoException
    {
	super.undo();
	fireChangeEvent();
    }

    public synchronized void redo()
	throws CannotRedoException
    {
	super.redo();
	fireChangeEvent();
    }

    public void beginUpdate()
    {
	inUpdate=true;
	support.beginUpdate();
    }

    public void endUpdate()
    {
	inUpdate=false;
	support.endUpdate();
    }

    public synchronized boolean addEdit(UndoableEdit edit)
    {
	boolean added=super.addEdit(edit);
	fireChangeEvent();
	return added;
    }

    public void addChangeListener(ChangeListener chigger)
    {
	listeners.add(chigger);
    }

    public void removeChangeListener(ChangeListener chigger)
    {
	listeners.remove(chigger);
    }

    protected void fireChangeEvent()
    {
	//by obtaining an array, I avoid any concurrent modification exceptions
	//that might be thrown if a listener removes itself when called.
	Object[] listenerArray=listeners.toArray();
	for (int i=0;i<listenerArray.length;i++)
	    ((ChangeListener)listenerArray[i]).stateChanged(changeEvent);
    }

    /**
     * a filter which receives UndoableEditEvents and then farms them out to the undo manager, 
     * after optionally coalescing them into CompoundEdits
     */
    class HarkeningUndoSupport extends UndoableEditSupport 
	implements UndoableEditListener
    {
	HarkeningUndoSupport(UndoableEditListener l)
	{
	    this.addUndoableEditListener(l);
	}

	public void undoableEditHappened(UndoableEditEvent ugh)
	{
	    postEdit(ugh.getEdit());
	}
    }
}
