package org.skunk.dav.client.gui.editor;

import java.awt.BorderLayout;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import org.skunk.dav.client.DAVFile;
import org.skunk.dav.client.gui.ResourceManager;

public class ImageViewer extends AbstractDAVEditor
{
    private ImagePanel ipanel;
    private JScrollPane scroller;
    
    public ImageViewer(DAVFile file)
    {
	super(file);
	ipanel=new ImagePanel();
	scroller=new JScrollPane(ipanel);
    }

    public DAVEditorUndoManager getUndoManager()
    {
	return null;
    }

    public JComponent getComponent()
    {
	return scroller;
    }

    public boolean isWriteable()
    {
	return false;
    }

    public void setWriteable(boolean writeable)
    {
	// do nothing
    }

    public void load()
    {
	ImageIcon ii=new ImageIcon(getResourceBody());
	ii.setDescription(getName());
	ipanel.setImageIcon(ii);
    }
	
}

class ImagePanel extends JPanel
{
    private JLabel imageLabel;
    private JLabel descriptionLabel;

    public ImagePanel()
    {
	this(null);
    }

    public ImagePanel(ImageIcon ii)
    {
	super();
	imageLabel=new JLabel();
	imageLabel.setIcon(ii);
	descriptionLabel=new JLabel(getLabel(ii));
	descriptionLabel.setHorizontalAlignment(JLabel.CENTER);
	doImageLayout();
    }

    private String getLabel(ImageIcon ii)
    {
	if (ii!=null)
	    return ResourceManager.getMessage(ResourceManager.IMAGE_DESCRIPTION, 
					      new Object[] { ii.getDescription(),
								 new Integer(ii.getIconWidth()), 
								 new Integer(ii.getIconHeight()) });
	else return "";	
    }

    public void setImageIcon(ImageIcon ii)
    {
	imageLabel.setIcon(ii);
	String label=getLabel(ii);
	imageLabel.setToolTipText(label);
	descriptionLabel.setText(label);
	revalidate();
    }

    private void doImageLayout()
    {
	Box daBox=Box.createVerticalBox();	
 	daBox.add(Box.createVerticalGlue());
 	daBox.add(imageLabel);
 	daBox.add(Box.createVerticalStrut(5));
	daBox.add(descriptionLabel);
 	daBox.add(Box.createVerticalGlue());
	setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
	add(Box.createHorizontalGlue());
	add(daBox);
	add(Box.createHorizontalGlue());
    } 
}

	

    
