/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.editor;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import javax.swing.AbstractAction;
import javax.swing.KeyStroke;
import javax.swing.JTextPane;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.JTextComponent;
import javax.swing.text.Keymap;

public class KeyMapper
{
    public static final JTextComponent.KeyBinding[] defaultBindings=new JTextComponent.KeyBinding[]
    {
	//move to start of line
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.CTRL_MASK), 
				      DefaultEditorKit.beginLineAction),
	//move to end of line
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_E, InputEvent.CTRL_MASK),
				      DefaultEditorKit.endLineAction),
	//move forward one character
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_MASK),
				      DefaultEditorKit.forwardAction),
	//move backward one character
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_B, InputEvent.CTRL_MASK),
				      DefaultEditorKit.backwardAction),
	//move down one row
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK),
				      DefaultEditorKit.downAction),
	//move up one row
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_MASK),
				      DefaultEditorKit.upAction),
	//page down
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK),
				      DefaultEditorKit.pageDownAction),
	//page up
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.ALT_MASK | InputEvent.SHIFT_MASK),
				      DefaultEditorKit.pageUpAction),
	//delete forward one char
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_MASK),
				      DefaultEditorKit.deleteNextCharAction),
	//move back by paragraph
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_BRACELEFT, InputEvent.ALT_MASK),
				      DefaultEditorKit.beginParagraphAction),
	//move forward by paragraph
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_BRACERIGHT, InputEvent.ALT_MASK),
				      DefaultEditorKit.endParagraphAction),
	//move back by word
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_B, InputEvent.ALT_MASK),
				      DefaultEditorKit.beginWordAction),
	//move forward by word
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.ALT_MASK),
				      DefaultEditorKit.nextWordAction)	
    };

    public static void loadKeymap(JTextComponent c)
    {
	Keymap k=c.getKeymap();
	JTextComponent.loadKeymap(k, defaultBindings, c.getActions());	    
    }
}
