/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.editor;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import org.skunk.dav.client.gui.ExplorerApp;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.dav.client.gui.StateMonitor;
import org.skunk.dav.client.gui.View;
import org.skunk.dav.client.gui.editor.action.SearchAndReplaceAction;
import org.skunk.trace.Debug;

/**
 * a panel for performing search and replace operations
 */
public class SearchAndReplacePanel extends AbstractSearchPanel
{
    private JButton findNextButton, replaceAllButton, replaceButton;

    public SearchAndReplacePanel()
    {
	super();
    }

    protected void initComponents()
    {
	JLabel findLabel=new JLabel(ResourceManager.getMessage(ResourceManager.REPLACE_PROMPT));
	JLabel replaceLabel=new JLabel(ResourceManager.getMessage(ResourceManager.REPLACE_WITH_PROMPT));
	createEntryField();
	createReverseCheckBox();
	createSearchModeButtonGroup();
	createReplaceField();
	createFindNextButton();
	createReplaceButton();
	createReplaceAllButton();

	this.setLayout(new GridBagLayout());
	GridBagConstraints gbc=new GridBagConstraints();
	gbc.gridx=0;
	gbc.gridy=0;
	gbc.anchor=GridBagConstraints.NORTHWEST;
	gbc.fill=GridBagConstraints.BOTH;
	gbc.insets=new Insets(2, 2, 2, 2);
	gbc.gridwidth=1;
	gbc.gridheight=1;

	this.add(findLabel, gbc);

	gbc.gridx++;	
	this.add(entryField, gbc);

	gbc.gridx++;
	this.add(findNextButton, gbc);

	gbc.gridx=0;
	gbc.gridy++;
	this.add(replaceLabel, gbc);

	gbc.gridx++;
	this.add(replaceField, gbc);
	
	gbc.gridx++;
	this.add(replaceButton, gbc);
	
	gbc.gridy++;
	this.add(replaceAllButton, gbc);

	gbc.gridx=0;
	gbc.gridy++;
	Box optionBox=Box.createHorizontalBox();
	optionBox.add(reverseButton);
	optionBox.add(literalButton);
	optionBox.add(caseButton);
	optionBox.add(regexpButton);
	gbc.gridwidth=3;
	gbc.anchor=GridBagConstraints.CENTER;
	this.add(optionBox, gbc);
    }

    public void addNotify()
    {
	super.addNotify();
	getRootPane().setDefaultButton(findNextButton);
    }

    protected void createFindNextButton()
    {
	findNextButton=new JButton(ResourceManager.getMessage(ResourceManager.FIND_NEXT_OPTION));
	findNextButton.addActionListener(new ActionListener() 
	    {
		public void actionPerformed(ActionEvent ae)
		{
		    fireActionEvent(isReverse()? SEARCH_FROM_CARET : SEARCH_FROM_SELECTION);
		}
	    });
    }

    private StateMonitor.Mapper mapper;
    private StateMonitor.DomainMatcher matcher;

    private StateMonitor.Mapper getButtonValueMapper()
    {
	if (mapper==null) 
	{
	    mapper=new StateMonitor.Mapper()
		{
		    public Object getTargetValue(Object applicationValue)
		    {
			return new Boolean(applicationValue!=null);					 
		    }
		    
		    public boolean canMapValue(Object applicationValue)
		    {
			return (applicationValue==null
				|| applicationValue instanceof SimpleTextEditor);
		    }
		};
	}
	return mapper;
    }

    private StateMonitor.DomainMatcher getButtonDomainMatcher()
    {
	if (matcher==null) 
	{
	    matcher = new StateMonitor.DomainMatcher() 
		{
		    public boolean domainMatches(Object domainKey)
		    {
			View v=ExplorerApp.getAppContext().getCurrentView();
			return (v!=null 
				&& domainKey!=null 
				&& domainKey.equals(v.getFocussedBuffer()));
		    }
		};
	}
	return matcher;
    }

    protected void createReplaceButton()
    {
	replaceButton=new JButton(ResourceManager.getMessage(ResourceManager.REPLACE_OPTION));
	replaceButton.setEnabled(false);
	StateMonitor.registerProperty(replaceButton, 
				      "enabled",
				      SearchAndReplaceAction.MATCH_PROPERTY,
				      getButtonValueMapper(),
				      getButtonDomainMatcher());

	replaceButton.addActionListener(new ActionListener() 
	    {
		public void actionPerformed(ActionEvent ae)
		{
		    SearchAndReplacePanel.this.replaceButton.setEnabled(false);
		    fireActionEvent(REPLACE_NEXT);
		}
	    });
    }

    protected void createReplaceAllButton()
    {
	replaceAllButton=new JButton(ResourceManager.getMessage(ResourceManager.REPLACE_ALL_OPTION));
	replaceAllButton.setEnabled(false);
	StateMonitor.registerProperty(replaceAllButton, 
				      "enabled",
				      SearchAndReplaceAction.MATCH_PROPERTY,
				      getButtonValueMapper(),
				      getButtonDomainMatcher());
	replaceAllButton.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent ae)
		{
		    SearchAndReplacePanel.this.replaceAllButton.setEnabled(false);
		    fireActionEvent(REPLACE_ALL);
		}
	    });
    }
}
