/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.editor.action;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.JComponent;
import org.skunk.dav.client.gui.Buffer;
import org.skunk.dav.client.gui.ExplorerApp;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.dav.client.gui.StatusInputPanel;
import org.skunk.dav.client.gui.View;
import org.skunk.dav.client.gui.editor.SimpleTextEditor;
import org.skunk.swing.text.PositiveIntegerEntryFilter;
import org.skunk.trace.Debug;

public class GotoLineAction extends AbstractAction
{
    public void actionPerformed(ActionEvent ae)
    {
	Object source=ae.getSource();
	View currentView=ExplorerApp.getAppContext().getCurrentView();

	if (source instanceof StatusInputPanel)
	{
	    StatusInputPanel panel=(StatusInputPanel) source;
	    Buffer buffy=currentView.getFocussedBuffer();
	    if (buffy!=null && buffy instanceof SimpleTextEditor)
	    {
		SimpleTextEditor editor=(SimpleTextEditor)buffy;
		try
		{
		    String s=panel.getInputText();
		    if (s!=null)
		    {
			int i=Integer.parseInt(s);
			editor.gotoLine(i-1);
		    }
		}
		catch (NumberFormatException nafta)
		{
		    Debug.trace(this, Debug.DP3, nafta);
		}
	    }
	    panel.removeActionListener(this);
	    currentView.undockStatus(panel);	  
	    panel=null;
	    buffy.getComponent().requestFocus();
	}
	else 
	{
	    String label=ResourceManager.getMessage(ResourceManager.GOTO_LINE_PROMPT);
	    int length=5;
	    PositiveIntegerEntryFilter filter=new PositiveIntegerEntryFilter(1, Integer.MAX_VALUE);
	    StatusInputPanel sip=new StatusInputPanel(label, length, filter);
	    sip.addActionListener(new GotoLineAction());
	    currentView.dockStatus(sip);
	    sip.focus();
	}
    }
}
