/*
 *  Copyright (c) 2000, Jacob Smullyan
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.gui.editor.action;

import java.awt.Component;
import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.WindowConstants;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.dav.client.gui.StateMonitor;
import org.skunk.dav.client.gui.View;
import org.skunk.dav.client.gui.editor.ISearchPanel;
import org.skunk.dav.client.gui.editor.SearchAndReplacePanel;
import org.skunk.dav.client.gui.editor.SimpleTextEditor;
import org.skunk.trace.Debug;

public class SearchAndReplaceAction extends SearchAction
{
    private JDialog dialog;
    private ISearchPanel panel;
    private int foundCount=0;
    public static final String MATCH_PROPERTY="search_and_replace_match";

    protected ISearchPanel createSearchPanel()
    {
	ISearchPanel isp=new SearchAndReplacePanel();
	isp.addActionListener(this);
	return isp;
    }

    protected void showSearchPanel(View currentView, ISearchPanel panel)
    {
	this.panel=panel;
	this.foundCount=0;
	Component c=currentView.getComponent();
	Frame appFrame=(c instanceof Frame) 
	    ? (Frame) c
	    : null;
	dialog=new JDialog(appFrame, 
			   ResourceManager.getMessage(ResourceManager.REPLACE_TITLE), 
			   true);
	dialog.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	dialog.addWindowListener(new WindowAdapter()
	    {
		public void windowClosing(WindowEvent woo)
		{
		    SearchAndReplaceAction.this.panel.fireActionEvent(ISearchPanel.END_SEARCH);
		    Debug.trace(this, Debug.DP3, "have fired end search event");
		}
	    });

	dialog.setContentPane(panel.getComponent());
	dialog.pack();
	dialog.setLocationRelativeTo(c);
	dialog.setVisible(true);
    }

    protected void hideSearchPanel(View currentView, ISearchPanel panel)
    {
	if (dialog!=null)
	{
	    dialog.setVisible(false);
	    dialog.dispose();
	}
    }

    protected void handleMatch(SimpleTextEditor editor, int[] found)
    {
	foundCount++;
	super.handleMatch(editor, found);
	Debug.trace(this, Debug.DP3, "in handleMatch() -- about to set match property");
	StateMonitor.setProperty(MATCH_PROPERTY, editor, editor);
    }

    protected void handleNoMatch(SimpleTextEditor editor)
    {
	if (foundCount==0)
	    super.handleNoMatch(editor);
	else
	{
	    String title=ResourceManager.getMessage(ResourceManager.END_REPLACE_TITLE);
	    Object[] interpolations=new Object[] { new Integer(foundCount) };
	    String message=ResourceManager.getMessage(ResourceManager.END_REPLACE_MESSAGE, interpolations);
	    foundCount=0;	    
	    JOptionPane.showMessageDialog(editor.getComponent(), message, title, JOptionPane.INFORMATION_MESSAGE);
	}
	StateMonitor.setProperty(MATCH_PROPERTY, null, editor);
    }
}
/* $Log: SearchAndReplaceAction.java,v $
/* Revision 1.3  2000/12/15 23:22:08  smulloni
/* added word wrap to text editor.
/*
/* Revision 1.2  2000/12/14 23:09:17  smulloni
/* fixes to search and replace; partial fix to adding custom properties bug.
/*
/* Revision 1.1  2000/12/14 06:36:26  smulloni
/* changes to search and replace in text editor.
/* */
