/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/


package org.skunk.dav.client.method;

import org.skunk.dav.client.AbstractDAVMethod;
import org.skunk.dav.client.DAVConstants;
import org.skunk.dav.client.DAVMethodName;
import java.util.Map;
import java.util.StringTokenizer;

public class OptionsMethod extends AbstractDAVMethod
{
    private String[] options;
      
    public OptionsMethod(String url)
    {
	super(url);
    }
    
    public DAVMethodName getRequestMethodName()
    {
	return DAVMethodName.OPTIONS;
    }
    
    public void processResponseHeaders()
    {
	//if status is ok, get relevant options
	int s=getStatus();
	if (s>=200 && s<300)
	{
	    Map headers=getResponseHeaders();
	    if (headers.containsKey(DAVConstants.ALLOW_HEADER))
	    {
		Object allowed=headers.get(DAVConstants.ALLOW_HEADER);
		if (allowed!=null)
		    options=extractOptions(allowed.toString());
	    }
	}
	if (options==null) options=new String[] {};
    }

    public String[] getOptions()
    {
	return options;
    }
    
    private String[] extractOptions(String optionStr)
    {
	StringTokenizer st=new StringTokenizer(optionStr, ", ");
	String[] ops=new String[st.countTokens()];
	for (int i=0;i<ops.length;i++)
	    ops[i]=st.nextToken();
	return ops;
    }
}

/* $Log: OptionsMethod.java,v $
/* Revision 1.7  2001/07/15 18:16:37  smulloni
/* fixed some copyright notices.
/*
/* Revision 1.6  2000/12/03 23:53:26  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.5  2000/11/09 23:35:08  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
