/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

 package org.skunk.dav.client.method;

import org.skunk.dav.client.AbstractWriteDAVMethod;
import org.skunk.dav.client.DAVConstants;
import org.skunk.dav.client.DAVMethodName;
import org.skunk.dav.client.DAVProperty;
import org.skunk.dav.client.IfHeader;
import org.skunk.minixml.MalformedXMLException;
import org.skunk.minixml.XMLDocument;
import org.skunk.minixml.XMLElement;
import org.skunk.minixml.XMLViewer;
import org.skunk.trace.Debug;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;


public class PropPatchMethod extends AbstractWriteDAVMethod
{
    private ArrayList patchProps=new ArrayList();
    private ArrayList unpatchProps=new ArrayList();
    private String lockToken;

    public PropPatchMethod(String url)
    {
        super(url);
    }

    public final DAVMethodName getRequestMethodName()
    {
        return DAVMethodName.PROPPATCH;
    }

    public void processRequestBody()
    {
	String NScode="skunk";
	XMLElement propertyUpdateElem=new XMLElement(DAVConstants.PROPERTYUPDATE_ELEM, 
						     NScode, 
						     DAVConstants.DAV_NAMESPACE);
	if (patchProps.size()>0)
	{
	    XMLElement setElem=new XMLElement(DAVConstants.SET_ELEM, NScode);
	    for (Iterator it=patchProps.iterator();it.hasNext();)
		setElem.addChild(new XMLElement(DAVConstants.PROP_ELEM, NScode)
				 .addChild((DAVProperty)it.next()));
	    propertyUpdateElem.addChild(setElem);
	}
	if (unpatchProps.size()>0)
	{
	    XMLElement removeElem=new XMLElement(DAVConstants.REMOVE_ELEM, NScode);
	    for (Iterator it=unpatchProps.iterator();it.hasNext();)
		removeElem.addChild(new XMLElement(DAVConstants.PROP_ELEM, NScode)
				    .addChild((DAVProperty)it.next()));
	    propertyUpdateElem.addChild(removeElem);
	}
	XMLDocument doc=new XMLDocument();
	try
	{
	    doc.addElement(DAVConstants.XML_BOILERPLATE);
	    doc.addElement(propertyUpdateElem);
	}
	catch (MalformedXMLException malex)
	{
	    Debug.trace(this, Debug.DP2, malex);
	}
	if (Debug.isDebug(this, Debug.DP8))
	{
	    XMLViewer.viewDocumentDialog(doc);
	}
	setRequestBody((doc.toString()+DAVConstants.CRLF).getBytes());		 
    }
    
    public void putProperty(DAVProperty property, Object value)
    {
	property.addChild(value);
	patchProps.add(property);
    }

    public void removeProperty(DAVProperty property)
    {
	unpatchProps.add(property);
    }

    /**
     * convenience method for the case when there is only one
     * lock token; for more flexibility, create an IfHeader.
     * Note that using this will create a new IfHeader at request header
     * processing time, destroying any IfHeader that has been manually 
     * created!!  This method exists solely for backwards compatibility; 
     * it will probably be deprecated in the future.
     */
    public void setLockToken(String lockToken)
    {
	this.lockToken=lockToken;
    }

    /**
     * accesses the locktoken property set with the setLockToken() method.
     */
    public String getLockToken()
    {
	return lockToken;
    }

    public void processRequestHeaders()
    {
	if (this.lockToken!=null)
	{
	    // blow away any if headers that are lying around
	    IfHeader iffy=new IfHeader(true);
	    iffy.addStateToken(getRequestURL(), 
			       lockToken);
	    this.setIfHeader(iffy);
	}
	super.processRequestHeaders();
    }
}


/* $Log: PropPatchMethod.java,v $
/* Revision 1.10  2001/07/17 04:41:32  smulloni
/* integrated IfHeader more closely into method package; added lock test.
/*
/* Revision 1.9  2001/07/15 18:16:37  smulloni
/* fixed some copyright notices.
/*
/* Revision 1.8  2001/01/02 17:43:38  smulloni
/* changed debug level for XMLViewer popups to Debug.DP8.
/*
/* Revision 1.7  2000/12/03 23:53:27  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.6  2000/11/14 23:18:32  smullyan
/* fix for responsedescription property in multistatus
/*
/* Revision 1.5  2000/11/09 23:35:09  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
