/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.dav.client.method;

import org.skunk.dav.client.AbstractWriteDAVMethod;
import org.skunk.dav.client.DAVConstants;
import org.skunk.dav.client.DAVMethodName;
import org.skunk.dav.client.IfHeader;

public class PutMethod extends AbstractWriteDAVMethod
{
    private byte[] putData;
    private String lockToken;

    public PutMethod(String url, byte[] putData)
    {
        super(url);
	this.putData=putData;
    }

//     public void processRequestHeaders()
//     {
// 	if (lockToken!=null)
// 	{
// 	    StringBuffer ifHeader=new StringBuffer("<");
// 	    ifHeader.append(getRequestURL());
// 	    ifHeader.append("> (<");
// 	    ifHeader.append(lockToken);
// 	    ifHeader.append(">)");	    
// 	    getRequestHeaders().put(DAVConstants.IF_HEADER, ifHeader.toString());
// 	}
//     }

    public final DAVMethodName getRequestMethodName()
    {
        return DAVMethodName.PUT;
    }

    public void processRequestBody()
    {
	setRequestBody(putData);
    }

//     public void setLockToken(String lockToken)
//     {
// 	this.lockToken=lockToken;
//     }

//     public String getLockToken()
//     {
// 	return lockToken;
//     }

    /**
     * convenience method for the case when there is only one
     * lock token; for more flexibility, create an IfHeader.
     * Note that using this will create a new IfHeader at request header
     * processing time, destroying any IfHeader that has been manually 
     * created!!  This method exists solely for backwards compatibility; 
     * it will probably be deprecated in the future.
     */
    public void setLockToken(String lockToken)
    {
	this.lockToken=lockToken;
    }

    /**
     * accesses the locktoken property set with the setLockToken() method.
     */
    public String getLockToken()
    {
	return lockToken;
    }

    public void processRequestHeaders()
    {
	if (this.lockToken!=null)
	{
	    // blow away any if headers that are lying around
	    IfHeader iffy=new IfHeader(true);
	    iffy.addStateToken(getRequestURL(), 
			       lockToken);
	    this.setIfHeader(iffy);
	}
	super.processRequestHeaders();
    }
}


/* $Log: PutMethod.java,v $
/* Revision 1.7  2001/07/17 04:41:32  smulloni
/* integrated IfHeader more closely into method package; added lock test.
/*
/* Revision 1.6  2001/07/15 18:16:37  smulloni
/* fixed some copyright notices.
/*
/* Revision 1.5  2000/12/03 23:53:27  smulloni
/* added license and copyright preamble to java files.
/*
/* Revision 1.4  2000/11/09 23:35:09  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
