#! /usr/bin/env/jpython --

# test script for Java dav client classes

from org.skunk.dav.client import DAVConnection, Depth, DAVProperty, DAVException
from org.skunk.dav.client.method import OptionsMethod, PropFindMethod, MkcolMethod, LockMethod, UnlockMethod, CopyMethod, MoveMethod, GetMethod, PutMethod, DeleteMethod, PropPatchMethod
from org.skunk.dav.client.gui import DAVAuthenticatorImpl
import string
import sys
import traceback
from java.lang import String
from javax.swing import JOptionPane

class DAVTest:

	def __init__(self, host=None, port=None):
		self.testmode=0
		if host and port:
			self.doOpen(host, port)
		else:
			self.myDAV=None
			self.connectedHost=None
			self.connectedPort=None

	def __del__(self):
		if self.myDAV:
			self.doClose()
	
	def createDAV(self, host, port, protocol):
		if protocol=="http":
			self.myDAV=DAVConnection(host, port)
		else:
			self.myDAV=DAVConnection(host, port, 1)
		self.myDAV.setAuthenticator(DAVAuthenticatorImpl())
		self.connectedHost=host
		self.connectedPort=port
		
	def getUsername(self):
		return self.myDAV.getUsername()

	def execute(self, method):
		try:
			print "in execute"
			self.myDAV.execute(method)
		except:
			print "error in execute"
			traceback.print_exc()
			traceback.print_stack()

	def doOptions(self, resource=None, bPrint=1):
		if not resource:
			resource=raw_input("resource: ")
		optionMethod=OptionsMethod(resource)
		self.execute(optionMethod)
		if bPrint:
			print "the server permits the following options on resource %s: " % resource
			for i in optionMethod.options:
				print "\t %s" % i
			
	def doGet(self, resource=None):
		if not resource:
			resource=raw_input("resource: ")
		getMethod=GetMethod(resource)
		self.execute(getMethod)
		print "status: %d" % getMethod.getStatus()
		print String(getMethod.getResponseBody())

	def doDiscover(self, resource=None, bPrint=1):
		if not resource:
			resource=raw_input("resource: ")
		method=PropFindMethod(resource, Depth.ONE, [DAVProperty.LOCKDISCOVERY])
		self.execute(method)
		file=method.getDAVFile()
		tok= file.getExclusiveLockToken()
		if bPrint:
			if tok:
				print "locktoken for %s: %s" % (resource, tok)		
			else:
				print "no locktoken for %s" % resource
			print file.toVerboseString()
		return tok			

	def doPut(self, resource=None, *putText):
		if not resource:
			resource=raw_input("resource: ")
		if not putText:
			print "enter text to put (line with single '.' to end): "
			putText=''
			while 1:
				inp=raw_input()
				if inp=='.':
					break
				putText="%s\n%s" % (putText, inp)
		else:
			putText=string.join(putText, ' ')
		# check to see if file has a lockToken
		lockToken=self.doDiscover(resource, 0)
		putMethod=PutMethod(resource, putText)
		if lockToken:
			putMethod.setLockToken(lockToken)
		self.execute(putMethod)
		print "status: %d" % putMethod.getStatus()
								
	def doMkcol(self, directoryName=None):
		if not directoryName:
			directoryName=raw_input('enter directory name to create: ')
		mkcolMethod=MkcolMethod(directoryName)
		self.execute(mkcolMethod)
		print "status: %d" % mkcolMethod.getStatus()

	def doLs(self, resource=None):
		if not resource:
			resource=raw_input("resource: ")
		method=PropFindMethod(resource, Depth.ONE, 1)
		self.execute(method)
		print "status: %d" % method.getStatus()
		file=method.getDAVFile()
		print file.toVerboseString()
		if (file.isCollection()):
			iterator=file.children()
			while (iterator.hasNext()):
				print iterator.next()
		
		
	
	def doPropfind(self, url=None):
		if not url:
			url=raw_input("resource: ")
		while 1:
			type=raw_input("propfind type (allprop, propname, prop) [allprop]: ")
			if not type:
				type="allprop";
				break
			elif type in ["allprop", "propname", "prop"]:
				break
			else: print "your response was not grokked"
		if type=="allprop":
			propfindMethod=PropFindMethod(url, Depth.ONE, 1)
		elif type=="propname":
			propfindMethod=PropFindMethod(url, Depth.ONE, 0)
		elif type=="prop":
			propStrings=string.split(raw_input("enter list of DAV properties, separated by spaces: "))
			properties=[]
			for s in propStrings:
				if s=='executable':
					properties.append(DAVProperty.EXECUTABLE)
				else:
					properties.append(DAVProperty.getCanonicalProperty(s))
				propfindMethod=PropFindMethod(url, Depth.ONE, properties)
		
		self.execute(propfindMethod)
		status=propfindMethod.getStatus()
		print "status: %d" % status
		if status < 400:
			file=propfindMethod.getDAVFile()
			print file.toVerboseString()
		else:
			stuffToRead=propfindMethod.getResponseBody()
			JOptionPane.showMessageDialog(None, String(stuffToRead))

	def doProppatch(self, resource=None, b_set=None, property=None, namespace=None, value=None):
		if not resource:
			resource=raw_input('resource: ')
		if b_set==None:
			while 1:
				answer=raw_input('set or remove [s/r]:')
				if answer=='s':
					b_set=1
					break
				elif answer=='r':
					b_set=0
					break
				else:
					print "please enter 's' or 'r'."
		if not property:
			property=raw_input('property: ')
		if namespace==None:
			namespace=raw_input('namespace: ')
		if b_set and not value:
			value=raw_input('value: ')
		
		propPatchMethod=PropPatchMethod(resource)
		if namespace:
			davprop=DAVProperty(property, "sk2", namespace)
		else:
			davprop=DAVProperty(property)
		if b_set:
			propPatchMethod.putProperty(davprop, value)
		else:
			propPatchMethod.removeProperty(davprop)
		self.execute(propPatchMethod)
		status=propPatchMethod.getStatus()
		print "status: %d" % status
		if (status>=400):
			stuffToRead=propPatchMethod.getResponseBody()
			JOptionPane.showMessageDialog(None, String(stuffToRead))
								
	def doLock(self, resource=None):
		if not resource:
			resource=raw_input('enter resource to lock: ')
		lockMethod=LockMethod(resource, self.getUsername())
		self.execute(lockMethod)
		status =lockMethod.getStatus()
		print "status: %d" % status
		if status < 400:
			file=lockMethod.getDAVFile()
			print file.toVerboseString()
		else:
			stuffToRead=lockMethod.getResponseBody()
			JOptionPane.showMessageDialog(None, String(stuffToRead))
			

	def doUnlock(self, resource=None):
		if resource==None:
			resource=raw_input('enter resource to unlock: ')
		# first need to get lockToken for resource
		propfindMethod=PropFindMethod(resource, Depth.ONE, [DAVProperty.LOCKDISCOVERY])
		self.execute(propfindMethod)
		file=propfindMethod.getDAVFile()
		# note that below I am only testing the exclusive lock case
		lockToken=file.getExclusiveLockToken()
		if not lockToken:
			print "no lockToken found!"
		else:
			unlockMethod=UnlockMethod(resource, lockToken)
			self.execute(unlockMethod)
			print "status: %d" % unlockMethod.getStatus()

	def doCopy(self, resource=None, destination=None):
		if not resource:
			resource=raw_input('enter resource to copy: ')
		if not destination:
			destination=raw_input('enter destination: ')
		srcTok=self.doDiscover(resource, 0)
		dstTok=self.doDiscover(destination, 0)
		copyMethod=CopyMethod(resource, self.getAbsoluteURL(destination))
		if srcTok:
			copyMethod.setSourceLockToken(srcTok)
		if dstTok:
			copyMethod.setDestLockToken(dstTok)
		self.execute(copyMethod)
		status=copyMethod.getStatus()
		print "status: %d" % status

	def doMove(self, resource=None, destination=None):
		if not resource:
			resource=raw_input('enter resource to move: ')
		if not destination:
			destination=raw_input('enter destination: ')
		srcTok=self.doDiscover(resource, 0)
		dstTok=self.doDiscover(destination, 0)
		moveMethod=MoveMethod(resource, self.getAbsoluteURL(destination))
		if srcTok:
			moveMethod.setSourceLockToken(srcTok)
		if dstTok:
			moveMethod.setDestLockToken(dstTok)
		self.execute(moveMethod)
		status=moveMethod.getStatus()
		print "status: %d" % status
		if (status>=400):
			stuffToRead=moveMethod.getResponseBody()
			JOptionPane.showMessageDialog(None, String(stuffToRead))

	def doOpen(self, host=None, port=None, protocol=None):
		if self.myDAV:
			print "close current connection first"
			return
		defaultProtocol="http"
		defaultHost='support.skunk.org'
		if not host:
			host=raw_input("host [%s]: " % defaultHost)
			if not host: host=defaultHost
		if not port:
			if host==defaultHost:
				defaultPort='8888'
			else:
				defaultPort='80'
			while 1:
				try:	
					portStr=raw_input('port [%s]: ' % defaultPort)
					if not portStr:
						portStr=defaultPort
					port=int(portStr)
					break
				except ValueError:
					print "please enter a number!"
		if not protocol:
			while 1:
				protocol=raw_input('protocol [%s]: ' % defaultProtocol)
				if not protocol:
					protocol=defaultProtocol
				if protocol=="http" or protocol=="https":
					break
				else:
					print "must be http or https"
					
		try:
			self.createDAV(host, int(port), protocol)
			self.doOptions('/', 0)
			print "connected to %s on port %d" % (host, port)
		except:
			traceback.print_exc()
			print "could not create connection to %s on port %d" % (host, port)

	def doDelete(self, resource=None):
		if not resource:
			resource=raw_input("resource: ")
		deleteMethod=DeleteMethod(resource)
		self.execute(deleteMethod)
		print "status: %d" % deleteMethod.getStatus()

	def doClose(self):
		self.myDAV.closeConnection()
		self.myDAV=None
		self.connectedHost=None
		self.connectedPort=None

	def getAbsoluteURL(self, resource):
		return "http://"+str(self.connectedHost)+":"+str(self.connectedPort)+str(resource)

	def doUsage(self):
		print """supported commands are: open, close, discover, get, put, delete,
lock, unlock, copy, move, options, mkcol, propfind and proppatch"""
	
	def mainloop(self):
		while 1:
			commands=string.split(string.lower(raw_input("davtest> ")))
			if (not commands) or len(commands)==0:
				continue
			command=commands[0]
			if command=='testmode':
				if self.testmode:
					self.testmode=0
					print "testmode off"
					continue
				else:
					self.testmode=1
					print "testmode on"
					continue
			if command in ['quit', 'exit', 'bye', 'q']:
				raise SystemExit
			elif command == 'help' or command == '?':
				self.doUsage()
				continue
			for i in range(1, len(commands)):
				commands[i]="'"+commands[i]+"'"
			if self.testmode:
				exec 'self.do'+string.capitalize(command)+'('+string.join(commands[1:], ', ')+')'
			else:
				try:
					exec 'self.do'+string.capitalize(command)+'('+string.join(commands[1:], ', ')+')'
				except (AttributeError, NameError, SyntaxError):
					print "unrecognized command: %s" % command
					self.doUsage()

##############################

def parseHost(s):
	colonIndex=string.find(s, ':')
	if colonIndex==-1:
		return s, None
	else:
		host=s[:colonIndex]
		portStr=s[colonIndex+1:]
		try:
			port=int(portStr)
			return host, port
		except ValueError:
			return host, None

if __name__=='__main__':
	test=DAVTest()
	if len(sys.argv)>1:
		(host, port)=parseHost(sys.argv[1])
		if not port and sys.argv[2]:
			try:
				port=int(sys.argv[2])
			except ValueError:
				pass
		if host and port:		
			test.doOpen(host, port)
	test.mainloop()
