/*
 *  Copyright (c) 2001, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

/**
 * @author Jacob Smullyan
 * @version $Revision: 1.5 $
 */
package org.skunk.minixml;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.ListIterator;
import java.util.StringTokenizer;

/**
 *  an insultingly simple wrapper for a bunch of document elements.
 */
public class XMLDocument
{
    private ArrayList contents=new ArrayList();
    private boolean lenient;
    
    public XMLDocument() 
    {
	this(false);
    }

    public XMLDocument(boolean lenient)
    {
	this.lenient=lenient;
    }

    public boolean isLenient()
    {
	return this.lenient;
    }
    
    public XMLDocument addElement(Object elem) throws MalformedXMLException
    {
	if (elem instanceof XMLElement)
	{
	    if (hasRoot() && (!lenient))
		throw new MalformedXMLException
		    ("adding more than one root element in strict parse mode");
	}
	contents.add(elem);	
	return this;
    }

    public boolean hasRoot()
    {
	int elemCnt=0;
	for (ListIterator lit=contents.listIterator();lit.hasNext();)
	{
	    if (lit.next() instanceof XMLElement)
		elemCnt++;
	    if (elemCnt==2) break;
	}
	return (elemCnt==1);
    }

    public ListIterator contents()
    {
	return contents.listIterator();
    }

    /**
     *  returns the first XMLElement in the contents collection
     */
    public XMLElement getRootElement()
    {
	for (ListIterator lit=contents.listIterator();lit.hasNext();)
	{
	    Object o=lit.next();
	    if (o instanceof XMLElement)
		return (XMLElement) o;
	}
	return null;
    }

    /**
     *  convenience method for accessing an element with
     *  a url-like path, e.g., html/body/h1.  Always returns
     *  the first matching element.
     */
    public XMLElement getElement(String urlLikePath)
    {
	StringTokenizer st=new StringTokenizer(urlLikePath, "/");
	XMLElement elem=getRootElement();
	if (st.countTokens()<1)
	    return null;
	if (!st.nextToken().equals(elem.getElementName()))
	    return null;
	while (st.hasMoreTokens())
	{
	    elem=elem.getChild(st.nextToken());
	    if (elem==null) break;
	}
	return elem;
    }
    
    /**
     * returns a textual representation of the document, 
     * itself xml
     */
    public String toString()
    {
	StringBuffer sb=new StringBuffer();
	for (ListIterator li=contents.listIterator();li.hasNext();)
	    sb.append(li.next());
	return sb.toString();
    }

    /**
     * convenience method
     */
    public final byte[] getBytes()
    {
	return toString().getBytes();
    }
}

/* $Log: XMLDocument.java,v $
/* Revision 1.5  2001/07/17 03:00:04  smulloni
/* added XMLCData class to represent CDATA, and fixed CDATA parsing, which was
/* wrong in any case; added license preambles where I had forgotten to put them.
/*
/* Revision 1.4  2000/11/09 23:35:11  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
