package org.skunk.net;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import java.security.KeyStore;
import javax.net.*;
import javax.net.ssl.*;
import java.security.Security;
import javax.security.cert.X509Certificate;
import com.sun.net.ssl.*;

public class LoggingProxyServer implements Runnable
{
    private int port;
    private String remoteHost;
    private int  remotePort;
    private String logFileName;
    private Thread myThread;
    private boolean stopThread=false;
    private ServerSocket myServerSocket;
    private boolean secure;
  
    public LoggingProxyServer(int port, String remoteHost, int remotePort, String logFileName, boolean secure)
    {
	this.port=port;
	this.remoteHost=remoteHost;
	this.remotePort=remotePort;
	this.logFileName=logFileName;
	this.secure=secure;
	myThread=new Thread(this);
	myThread.setPriority(Thread.NORM_PRIORITY);
	myThread.start();
    }

    public void quit()
    {
	stopThread=true;
    }

    public void finalize()
    {
	quit();
    }

    private static ServerSocketFactory getSSLServerSocketFactory()
    {
	Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
	SSLServerSocketFactory ssf = null;
	try 
	{
	    // set up key manager to do server authentication
	    SSLContext ctx;
	    KeyManagerFactory kmf;
	    KeyStore ks;
	    char[] passphrase = "passphrase".toCharArray();
	    
	    ctx = SSLContext.getInstance("TLS");
	    kmf = KeyManagerFactory.getInstance("SunX509");
	    ks = KeyStore.getInstance("JKS");
	    
	    ks.load(LoggingProxyServer.class.getResourceAsStream("testkeys"), passphrase);
	    kmf.init(ks, passphrase);
	    ctx.init(kmf.getKeyManagers(), null, null);
	    
	    ssf = ctx.getServerSocketFactory();
	    return ssf;
	} 
	catch (Exception e) 
	{
	    e.printStackTrace();
	}
	return null;
    }

    /** 
     * 
     */
    private void addAnonymousCipherSuites(SSLServerSocket sss)
    {
	String[] supported=sss.getSupportedCipherSuites();
	ArrayList enabled=new ArrayList();
	String[] oldEnabled=sss.getEnabledCipherSuites();
	for (int i=0;i>oldEnabled.length;i++)
	    enabled.add(oldEnabled[i]);
	for (int i=0;i<supported.length;i++)
	{
	    String candidate=supported[i];
	    if (candidate.indexOf("_anon_")>0 
		&& !enabled.contains(candidate))
	    {
		enabled.add(candidate);
	    }
	}
	String[] enabledArray=new String[enabled.size()];
	for (int i=0;i<enabledArray.length;i++)
	    enabledArray[i]=enabled.get(i).toString();
	sss.setEnabledCipherSuites(enabledArray);
    }

    protected ServerSocket getServerSocket(int port) throws IOException
    {
	if (secure)
	{
	    ServerSocket ss=getSSLServerSocketFactory().createServerSocket(port);
	    addAnonymousCipherSuites((SSLServerSocket)ss);
	    return ss;
	    
	}
	else return new ServerSocket(port);
    }

    public void run()
    {
	try
	{
	    myServerSocket=getServerSocket(port); //new ServerSocket(port);
	    while (true)
	    {
		//gracefully kill thread if flag is set
		if (stopThread) 
		{
		    try
		    {
			myServerSocket.close();
		    }
		    finally
		    {
			return;
		    }
		}
		try
		{
		    //get a request on listening socket
		    Socket clientToProxySocket=myServerSocket.accept();
		    final InputStream in=clientToProxySocket.getInputStream();
		    final OutputStream out=clientToProxySocket.getOutputStream();
		    Socket proxyToRemoteSocket=new Socket(remoteHost, remotePort);  
		    //is this a reasonable timeout or not?
		    //proxyToRemoteSocket.setSoTimeout(150000);
		    final InputStream remoteIn=proxyToRemoteSocket.getInputStream();
		    final OutputStream remoteOut=proxyToRemoteSocket.getOutputStream();
		    //copy stream unchanged to log and remote host 
		    /*
		      note that this logs to the same file in two threads.  They can thus end up mixing their output.
		      for our test purposes this is okay -- the only interpolated messages seem to be some TCP codes
		    */
		    Thread requestThread=new Thread()
		    {
			public void run()
			{
			    try
			    {
				copyStream(in, new LoggingOutputStream(logFileName, remoteOut));
			    }
			    catch (IOException ohGoodness)
			    {
				ohGoodness.printStackTrace(System.err);
			    }
			}
		    };
		    
		    requestThread.start();
		    
		    Thread responseThread=new Thread()
		    {
			public void run()
			{
			    try
			    {			
				copyStream(remoteIn, new LoggingOutputStream(logFileName, out)); 
			    }
			    catch (IOException notAgain)
			    {
				notAgain.printStackTrace(System.err);
			    }
			}
		    };
		    
		    responseThread.start();	
		}
	        catch (IOException oyster)
		{
		    System.err.println("error in main thread loop:");
		    oyster.printStackTrace();
		    stopThread=true;
		    continue;
		}
	    }
	}
	catch (IOException oyVeh)
	{
	    oyVeh.printStackTrace();
	}

    }

    private void copyStream(InputStream in, OutputStream out) throws IOException
    {

	byte[] buffer=new byte[1024];
	while (true)
	{
	    int bytesRead=in.read(buffer);
	    if (bytesRead==-1) break;
	    out.write(buffer, 0, bytesRead);
	}
	out.flush();
    }

	

    public static void main(String[] args)
    {	
	if (args.length != 4 && args.length!=5)
	{
	    usage();
	}
	try
	{
	    int port=Integer.parseInt(args[0]);
	    String remoteHost=args[1];
	    int remotePort=Integer.parseInt(args[2]);
	    String logFile=args[3];
	    boolean secure=args.length==5 && args[4].equals("-s");
	    LoggingProxyServer lps=new LoggingProxyServer(port, remoteHost, remotePort, logFile, secure);
	    System.out.println("proxy server for "+remoteHost+ ":"+remotePort
			       +" running on port "+port+" logging to "+logFile);
	    if (secure) 
		System.out.println("server socket is SSL");
	}
	catch (Exception e)
	{
	    e.printStackTrace(System.err);
	}
    }

    public static void usage()
    {
	System.out.println("Usage: java org.skunk.net.LoggingProxyServer port remoteHost remotePort logFile [-s]");
	System.out.println("\tuse of the -s option will cause a secure server socket to be created.");
	System.exit(1);
    }
}

class LoggingOutputStream extends FilterOutputStream
{
    private OutputStream logStream; 

    public LoggingOutputStream(String fileName, OutputStream out) throws IOException
    {
	this(new FileOutputStream(fileName, true), out);
    }

    public LoggingOutputStream(OutputStream logStream, OutputStream out)
    {
	super(out);
	this.logStream=logStream;
    }
	
    
    public void  write(int b) throws IOException
    {
	out.write(b);
	logStream.write(b);
    }

    public void write(byte[] b) throws IOException
    {
	out.write(b);
	logStream.write(b);
    }

    public void write(byte[] b, int off, int len) throws IOException
    {
	out.write(b, off, len);
	logStream.write(b, off, len);
    }

    public void flush() throws IOException
    {
	out.flush();
	logStream.flush();
    }

    public void close() throws IOException
    {
	out.close();
	logStream.close();
    }
}
