/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/


package org.skunk.spi;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.border.Border;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import org.skunk.assertF.Assertion;
import org.skunk.dav.client.DAVFile;
import org.skunk.dav.client.gui.ExplorerApp;
import org.skunk.dav.client.gui.ResourceManager;
import org.skunk.dav.client.gui.editor.AbstractDAVEditor;
import org.skunk.dav.client.gui.editor.CannotLoadException;
import org.skunk.dav.client.gui.editor.DAVEditorUndoManager;
import org.skunk.trace.Debug;

/**
 *  facility for editing Skunk message catalogs.
 *  need to support cut, copy and paste;
 *  undo, if the layout does not make that confusing.
 */
public class MessageCatalogEditor extends AbstractDAVEditor
{
    private JComponent edComp;
    private JComponent messageDisplay;
    private JSplitPane splitter;
    private Map messageMap=null;
    private MessageCatalogTableModel tableModel;
    private Map.Entry selectedMessage;
    private volatile int selectedTableRow=-99;
    private boolean complex;
    private JLabel currentLabel;
    private JTable currentTable;
    private HashMap messageDisplayMap=new HashMap();
    private static final int UNSELECTED=-99;
    public static final String ISO_LATIN_1="8859_1";

    public MessageCatalogEditor(DAVFile file)
    {
	super(file);
    }

    public DAVEditorUndoManager getUndoManager()
    {
	return null;
    }

    private void initModel()
    {
	tableModel =new MessageCatalogTableModel();
    }

    private void initComponents()
    {
	edComp=new JPanel(new BorderLayout());
	splitter=new JSplitPane(JSplitPane.VERTICAL_SPLIT);
	splitter.setTopComponent(createMessageTable());
	splitter.setBottomComponent(createMessageDisplay());
	splitter.setDividerLocation(0.5);
	edComp.add(splitter, BorderLayout.CENTER);
	edComp.add(createButtonPanel(), BorderLayout.SOUTH);
    }

    private JComponent createMessageTable()
    {
	TableSelectionListener tsl=new TableSelectionListener();
	JTable table=new JTable(tableModel);
	table.getTableHeader().setReorderingAllowed(false);
	table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	currentTable=table;
	table.addMouseListener(tsl);
	table.addMouseMotionListener(tsl);
	table.getSelectionModel().addListSelectionListener(tsl);
	if (table.getRowCount()>0) 
	    table.setRowSelectionInterval(0, 0);
	JScrollPane scroller=new JScrollPane(table);
	try
	{
	    Dimension d=ExplorerApp.getAppContext().getCurrentView().getFocussedBuffer().getComponent().getSize();
	    Dimension newDim=new Dimension(d.width, d.height/2);
	    scroller.setPreferredSize(newDim);
	}
	catch (NullPointerException e) {}
	return scroller;
    }

    private void selectFirstRow()
    {
	selectedTableRow=UNSELECTED;
	if (currentTable.getRowCount()>0) 
	    currentTable.setRowSelectionInterval(0, 0);
    }

    private void selectRow(Object key)
    {
	selectedTableRow=UNSELECTED;
	if (key==null) return;
	int i=0;
	for (Iterator it=messageMap.keySet().iterator();it.hasNext();i++)
	{
	    if (key.equals(it.next()) && currentTable.getRowCount()>i)
	    {
		currentTable.setRowSelectionInterval(i, i);
		break;
	    }
	}
    }

    private void displayMessage()
    {
	Component c=splitter.getBottomComponent();
	if (c!=null)
	    splitter.remove(c);
	messageDisplay=null;
	messageDisplay=getMessageDisplay(selectedMessage);	
	splitter.setBottomComponent(messageDisplay);
	messageDisplay.revalidate();
    }

    private void showTab(int tab)
    {
	if (isComplex() 
	    && messageDisplay instanceof MessageDisplay)
	    ((MessageDisplay) messageDisplay).showTab(tab);
    }

    private JComponent createMessageDisplay()
    {
	if (selectedMessage==null)
	{
	    messageDisplay=new JPanel();
	}
	else
	{
	    messageDisplay=getMessageDisplay(selectedMessage);
	}

	JScrollPane scroller=new JScrollPane(messageDisplay);
	scroller.setPreferredSize(messageDisplay.getPreferredSize());
	return scroller;
    }

    private JComponent getMessageDisplay(Map.Entry message)
    {
	if (message==null)
	    return new JPanel();
	Object key=message.getKey();
	if (messageDisplayMap.containsKey(key))
	{
	    MessageDisplay display = (MessageDisplay) messageDisplayMap.get(key);
	    return display;
	}
	else
	{
	    MessageDisplay display=new MessageDisplay(message);
	    messageDisplayMap.put(message.getKey(), display);
	    return display;    
	}
    }
	    

    private class MessageDisplay extends JPanel
    {
	private Map.Entry message;
	private Dimension dim=new Dimension(400, 150);
	private JTabbedPane tabPane;
	private ArrayList complexTextAreas=new ArrayList();
	private JTextArea simpleTextArea;

	MessageDisplay(Map.Entry message)
	{
	    super();
	    this.message=message;
	    initMessageDisplayComponents();
	}

	Map.Entry getMessage()
	{
	    return this.message;
	}

	void showTab(int tab)
	{
	    if (tabPane!=null 
		&& tab >=0
		&& tab<tabPane.getTabCount())
	    {
		tabPane.setSelectedIndex(tab);
		JTextArea jta=(JTextArea)complexTextAreas.get(tab);
		jta.requestFocus();
		jta.getCaret().setVisible(true);
	    }
	}

	void focus()
	{
	    JTextArea jta;
	    if (isComplex())
	    {
		jta=(JTextArea)complexTextAreas.get(tabPane.getSelectedIndex());
	    }
	    else
	    {
		jta=simpleTextArea;
	    }
	    jta.requestFocus();
	    jta.getCaret().setVisible(true);
	}

	void initMessageDisplayComponents()
	{
	    if (this.message==null)
	    {
		return;
	    }
	    int columnCount=MessageCatalogEditor.this.tableModel.getColumnCount();

	    setLayout(new GridBagLayout());
	    GridBagConstraints gbc=new GridBagConstraints();
	    gbc.gridx=0; 
	    gbc.gridy=0;
	    gbc.gridwidth=1;
	    gbc.gridheight=1;
	    gbc.fill=GridBagConstraints.HORIZONTAL;
	    gbc.anchor=GridBagConstraints.NORTHWEST;
	    gbc.insets=new Insets(2, 2, 2, 2);
	    JLabel keyLabel=new JLabel();
	    keyLabel.setFont(new Font("Dialog", Font.BOLD, 16));
	    keyLabel.setText(message.getKey().toString());	  
	    add(keyLabel, gbc);
	    MessageCatalogEditor.this.currentLabel=keyLabel;
	    gbc.gridy++;
	    if (!MessageCatalogEditor.this.isComplex())
	    {
		simpleTextArea=new JTextArea(message.getValue().toString(), 10, 60);
		simpleTextArea.getDocument().addDocumentListener(new MessageDocumentListener(message));
		add(new JScrollPane(simpleTextArea), gbc);	 
	    }
	    else
	    {
		tabPane=new JTabbedPane();
		tabPane.setTabPlacement(SwingConstants.RIGHT);
		Object value=message.getValue();
		Assertion.assertF((value instanceof Map), "selected Message value is a Map");
		Map currentValues=(Map) value;
		for (Iterator it=currentValues.keySet().iterator();it.hasNext();)
		{
		    Object langKey=it.next();
		    String messageStr=currentValues.get(langKey).toString();
		    JTextArea textarea=new JTextArea(messageStr, 10, 60);
		    complexTextAreas.add(textarea);
		    textarea.getDocument().addDocumentListener(new MessageDocumentListener(message, 
											   langKey));
		    tabPane.addTab(langKey.toString(), new JScrollPane(textarea));
		}
		add(tabPane, gbc);
	    }
	}

	public Dimension getPreferredSize()
	{
	    return dim;
	}

	public Dimension getMinimumSize()
	{
	    return dim;
	}

	public Dimension getMaximumSize()
	{
	    return dim;
	}
    }

    public static final TreeMap newSimpleCatalog()
    {
	return new TreeMap();	
    }

    public static final TreeMap newComplexCatalog()
    {
	TreeMap subTm=new TreeMap();
	subTm.put("eng", "");
	subTm.put("por", "");
	subTm.put("esp", "");
	TreeMap tm=new TreeMap();
	tm.put("", subTm);
	return tm;
    }

    private JComponent createButtonPanel()
    {
	JPanel boxPanel=new JPanel();
	boxPanel.setLayout(new BoxLayout(boxPanel, BoxLayout.X_AXIS));
	JButton addButton=new JButton(ResourceManager.getMessage(ResourceManager.ADD_MESSAGE));
	addButton.addActionListener(new AddMessageAction());
	JButton removeButton=new JButton(ResourceManager.getMessage(ResourceManager.REMOVE_MESSAGE));
	removeButton.addActionListener(new RemoveMessageAction());
	JButton renameButton=new JButton(ResourceManager.getMessage(ResourceManager.RENAME_MESSAGE));
	renameButton.addActionListener(new RenameMessageAction());
	boxPanel.add(Box.createHorizontalGlue());
	boxPanel.add(addButton);
	boxPanel.add(Box.createHorizontalStrut(5));
	boxPanel.add(removeButton);
	boxPanel.add(Box.createHorizontalStrut(5));
	boxPanel.add(renameButton);
	boxPanel.add(Box.createHorizontalGlue());
	Border outerBorder=BorderFactory.createLoweredBevelBorder();
	Border innerBorder=BorderFactory.createEmptyBorder(10, 10, 10, 10);
	boxPanel.setBorder(BorderFactory.createCompoundBorder(outerBorder, innerBorder));
	return boxPanel;
    }

    public void load() throws CannotLoadException
    {
	byte[] body=getResourceBody();
	if (body==null || body.length==0)
	{
	    if (getResourceName().endsWith(".cat"))
	    {
		messageMap=newComplexCatalog();
		complex=true;
	    }
	    else 
	    {
		messageMap=newSimpleCatalog();
		complex=false;
	    }
	}	    
	else try
	{
	    messageMap = invert(DictParser.fromString(new String(getResourceBody())));
	
	  // or use org.skunk.spi MessageHasher, which is a jythonc'ed class:
	  //  MessageHasher hasher=new MessageHasher();
	  //  messageMap=invert(hasher.stringToHash(new String(getResourceBody())));
	
	}
	catch (Assertion badAss)
	{
	    JOptionPane.showMessageDialog(ExplorerApp.getAppContext().getCurrentView().getFocussedBuffer().getComponent(),
					  ResourceManager.getMessage(ResourceManager.CANNOT_OPEN), 
					  ResourceManager.getMessage(ResourceManager.ERROR_DIALOG_TITLE), 
					  JOptionPane.ERROR_MESSAGE);
	    throw new CannotLoadException("cannot load file "+getResourceName());
	}
	initModel();
	initComponents();
    }

    public void save()
    {
	String s=DictParser.toString(invert(messageMap));
	
	  //or, using MessageHasher: 
	//MessageHasher hasher=new MessageHasher();
	//String s=invert(hasher.hashToString(messageMap));
	try
	{
	    setResourceBody(s.getBytes(ISO_LATIN_1));
	}
	catch (UnsupportedEncodingException uee)
	{
	    Debug.trace(this, Debug.DP2, uee);
	    setResourceBody(s.getBytes());
	}
	super.save();
    }

    /**
     * takes a two-dimensional tabular structure like:
     *
     *     { 
     *       a => { x => fa, y => ga, z => ha }, 
     *       b => { x => fb, y => gb, z => hb }
     *     }
     *
     * and inverts its dimensions, returning: 
     *       
     *     {
     *       x => { a => fa, b => fb },
     *       y => { a => ga, b => gb },
     *       z => { a => ha, b => hb }
     *     }
     *
     * one-dimensional structures are unaffected by inversion.
     */
    private TreeMap invert(Map m)
    {
	TreeMap retMap=new TreeMap();
	try
	{
	    for (Iterator it=m.keySet().iterator();it.hasNext();)
	    {
		Object key=it.next();
		Object value=m.get(key);
		if (!(value instanceof Map))
		{
		    complex=false;
		    return (m instanceof TreeMap)
			? (TreeMap) m
			: new TreeMap(m);
		}		
		Map tmp=(Map)value;
		for (Iterator subIt=tmp.keySet().iterator(); subIt.hasNext();)
		{
		    Object subKey=subIt.next();
		    if (retMap.containsKey(subKey))
		    {
			Object retVal=retMap.get(subKey);
			Assertion.assertF((retVal instanceof Map), 
					 "second level value for key {0} and subKey {1} is a Map",
					 new Object[] {key, subKey} );
			Map subMap=(Map)retVal;
			subMap.put(key, tmp.get(subKey));
		    }
		    else
		    {
			Map subMap=new TreeMap();
			subMap.put(key, tmp.get(subKey));
			retMap.put(subKey, subMap);
		    }
		}
	    }
	    complex=true;
	    return retMap;
	}
	catch (Assertion badAss)
	{
	    Debug.trace(this, 
			Debug.DP3, 
			"map not a consistent 2D tabular structure, cannot invert");
	    Debug.trace(this, Debug.DP3, badAss);
	    throw badAss;
	}
    }

    public JComponent getComponent()
    {
	return edComp;
    }

    private Map getMessageMap()
    {
	return this.messageMap;
    }

    public boolean isComplex()
    {
	return this.complex;
    }

    public Object getBlankComplexValue()
    {
	TreeMap tm=new TreeMap();
	for (int i=1;i<tableModel.getColumnCount();i++)
	{
	    tm.put(tableModel.getColumnName(i), "");
	}
	return tm; 
    }

    private class AddMessageAction extends AbstractAction
    {
	public void actionPerformed(ActionEvent ae)
	{
	    String newKey=JOptionPane.showInputDialog(getComponent(), 
						      ResourceManager.getMessage(ResourceManager.NEW_MESSAGE_PROMPT));
	    if (newKey!=null)
	    {
		Object blankVal=(isComplex())
				 ? getBlankComplexValue()
				 : "";
		getMessageMap().put(newKey.trim(), blankVal);
		tableModel.fireTableStructureChanged();
		MessageCatalogEditor.this.setDirty(true);
		MessageCatalogEditor.this.selectRow(newKey);
	    }
	}
    }

    private class RemoveMessageAction extends AbstractAction
    {
	public void actionPerformed(ActionEvent ae)
	{
	    getMessageMap().remove(selectedMessage.getKey());
	    tableModel.fireTableStructureChanged();
	    MessageCatalogEditor.this.setDirty(true);
	    MessageCatalogEditor.this.selectFirstRow();
	}
    }

    private class RenameMessageAction extends AbstractAction
    {
	public void actionPerformed(ActionEvent ae)
	{
	    String newKey=JOptionPane.showInputDialog(getComponent(), 
						      ResourceManager.getMessage(ResourceManager.RENAME_MESSAGE_PROMPT));
	    if (newKey!=null)
	    {
		newKey=newKey.trim();
		Map.Entry entry=MessageCatalogEditor.this.selectedMessage;
		Object oldKey=entry.getKey();
		Object value=entry.getValue();
		getMessageMap().remove(oldKey);
		getMessageMap().put(newKey, value);
		MessageCatalogEditor.this.messageDisplayMap.remove(oldKey);
		tableModel.fireTableStructureChanged();
		displayMessage();
		MessageCatalogEditor.this.setDirty(true);
		MessageCatalogEditor.this.selectRow(newKey);
	    }
	}
    }	    
    /**
     * the table admits only row selection, but clicking on a column corresponding to a tab in the message
     * display will cause the corresponding tab to be displayed.
     */
    private class TableSelectionListener extends MouseAdapter implements ListSelectionListener, MouseMotionListener
    {
	public void valueChanged(ListSelectionEvent lassie)
	{
	    if ((!isComplex()) 
		|| MessageCatalogEditor.this.selectedTableRow==UNSELECTED)
	    {
		Debug.trace(this, Debug.DP3, "in valueChanged()");
		ListSelectionModel listModel=(ListSelectionModel)lassie.getSource();
		int row=listModel.getMinSelectionIndex();
		changeSelection(row, -1);
	    }
	}

	private void changeSelection(int row, int column)
	{
	    Debug.trace(this, Debug.DP3, "in changeSelection: row "+row +", column: "+column);
	    if (row!=MessageCatalogEditor.this.selectedTableRow)
	    {
		MessageCatalogEditor.this.selectedTableRow=row;
		Map.Entry entry=getEntry(row);
		if (entry!=null) 
		{
		    MessageCatalogEditor.this.selectedMessage=entry;
		    MessageCatalogEditor.this.displayMessage();
		}	 	
	    }
	    if (isComplex()) //list selection events won't have any effect on columns
	    {
		showTab((column>0) 
			? column-1
			: 0);
	    }
	    else if (MessageCatalogEditor.this.messageDisplay instanceof MessageDisplay)
	    {
		((MessageDisplay)MessageCatalogEditor.this.messageDisplay).focus();
	    }
	}

	public void mouseMoved(MouseEvent moo)
	{
	    //do nothing
	}

	public void mouseClicked(MouseEvent moo)
	{
	    if (isComplex())
	    {
		Debug.trace(this, Debug.DP3, "in mouseClicked()");
		JTable table=(JTable) moo.getSource();
		Point p=new Point(moo.getX(), moo.getY());
		int row=table.rowAtPoint(p);
		int column=table.columnAtPoint(p);
		changeSelection(row, column);
	    }
	}

	public void mouseDragged(MouseEvent moo)
	{
	    mouseClicked(moo);
	}

	private Map.Entry getEntry(int row)
	{
	    Iterator it=getMessageMap().entrySet().iterator();
	    for (int i=0;it.hasNext() && i<=row;i++)
	    {
		Object o=it.next();
		if (i==row && o instanceof Map.Entry) 
		    return (Map.Entry) o;
	    }
	    return null;
	}
    }

    private class MessageCatalogTableModel extends AbstractTableModel
    {
	private int columnCount=-1;
	public int getRowCount()
	{
	    if (getMessageMap()!=null)
		return getMessageMap().size();
	    else return 0;
	}

	public String getColumnName(int column)
	{
	    if (column==0)
		return ResourceManager.getMessage(ResourceManager.KEY);
	    if (getColumnCount()==2 && column==1)
	    {
		return ResourceManager.getMessage(ResourceManager.MESSAGE);
	    }
	    
	    Map mm=getMessageMap();
	    if (mm==null) return null;
	    Iterator it=mm.keySet().iterator();
	    Object randomVal=mm.get(it.next());
	    Assertion.assertF((randomVal instanceof Map), "complex catalog has map for top-level value");
	    Map mm2=(Map)randomVal;
	    it=mm2.keySet().iterator();
	    for (int i=1;i<=column && it.hasNext();i++)
	    {
		Object key=it.next();
		if (i==column)
		    return key.toString();
	    }
	    return null;
	}

	public int getColumnCount()
	{
	    if (columnCount<=0)
	    {
		Map mm=getMessageMap();
		if (mm==null) 
		    columnCount=0;
		else
		{
		    Iterator it=mm.keySet().iterator();
		    if (it.hasNext())
		    {
			Object val=mm.get(it.next());
			if (val instanceof Map)
			{
			    Map subMap=(Map) val;
			    int size=subMap.size();
			    if (size>0)
			    {
				columnCount= 1 + size; //one column for the key, one for each key in the sub-maps
			    }
			}
		    }
		    if (columnCount<0) 
			columnCount=2; //key, value
		}
	    }
	    return columnCount;	    
	}

	public Object getValueAt(int row, int column)
	{
	    Map mm=getMessageMap();
	    if (mm==null) return null;
	    Iterator it=mm.keySet().iterator();
	    for (int i=0;i<=row && it.hasNext();i++)
	    {
		Object key=it.next();
		if (i!=row)
		    continue;
	
		if (column==0)
		{
		    return key;
		}
		Object value=mm.get(key);
		if (value instanceof Map)
		{
		    Map subMap=(Map)value;
		    Iterator subIt=subMap.keySet().iterator();
		    for (int j=1;j<=column && subIt.hasNext();j++)
		    {
			Object subKey=subIt.next();
			if (j==column)
			{
			    return subMap.get(subKey);
			}
		    }
		}
		else if (column==1) 
		    return value;
	    }
	    return null;
	}

    }

    private class MessageDocumentListener implements DocumentListener
    {
	private Map.Entry selectedMessage;
	private Object langKey=null;

	MessageDocumentListener(Map.Entry selectedMessage)
	{
	    this.selectedMessage=selectedMessage;
	}

	MessageDocumentListener(Map.Entry selectedMessage, Object langKey)
	{
	    this.selectedMessage=selectedMessage;
	    this.langKey=langKey;
	}

	public void changedUpdate(DocumentEvent dee)
	{
	    _update(dee);
	}

	public void insertUpdate(DocumentEvent dee)
	{
	    _update(dee);
	}

	public void removeUpdate(DocumentEvent dee)
	{
	    _update(dee);
	}

	private void _update(DocumentEvent dee)
	{
	    Document doc=dee.getDocument();
	    try
	    {
		String text=doc.getText(0, doc.getLength());
		if (langKey==null)
		{
		    selectedMessage.setValue(text);
		}
		else
		{
		    Object value=selectedMessage.getValue();
		    Assertion.assertF((value instanceof Map), "selected message value is a Map");
		    Map m=(Map)value;
		    Assertion.assertF(m.containsKey(langKey), "Map contains key"+langKey);
		    m.put(langKey, text);
		}
		MessageCatalogEditor.this.setDirty(true);
		tableModel.fireTableRowsUpdated(MessageCatalogEditor.this.selectedTableRow, 
						MessageCatalogEditor.this.selectedTableRow);
	    }
	    catch (BadLocationException ballocks)
	    {
		Debug.trace(this, Debug.DP2, ballocks);
	    }
	}
    } 
}

/* $Log: MessageCatalogEditor.java,v $
/* Revision 1.7  2001/07/11 02:06:15  smulloni
/* removed defunct spi files
/*
/* Revision 1.6  2000/12/19 22:36:06  smulloni
/* adjustments to preamble.
/*
/* Revision 1.5  2000/12/13 22:25:33  smulloni
/* now save catalogs explicitly in 8859_1 (ISO Latin-1) encoding
/*
/* Revision 1.4  2000/12/12 19:39:23  smulloni
/* some UI refinements: added mouse listener and mouse motion listener to table,
/* so the tabbed pane of complex catalogs will be set to show the column under
/* mouse point, and the text area in the message display given focus.
/*
/* Revision 1.3  2000/12/11 22:36:13  smulloni
/* improved support for input of UTF.
/*
/* Revision 1.2  2000/12/08 05:50:30  smulloni
/* fixed MessageCatalogEditor.  The spi features are now a special build option,
/* and editors are loaded through reflection.
/* */
