/*
 *  Copyright (c) 2000, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV  is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package org.skunk.swing;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;

/**
 * a button subclass that displays a color and has a built-in 
 * action listener that opens a JColorChooser to select a different one.
 * @author Jacob Smullyan ($Author: smulloni $) <smulloni@krinst.org>
 * @version $Revision: 1.2 $
 */
public class ColorChoiceButton extends JButton
{
    public static final String COLOR_PROPERTY="color";
    private String dialogTitle="Choose color";
    private JColorChooser chooser;

    public ColorChoiceButton(Color defaultColor, Dimension size)
    {
	super();
	setMinimumSize(size);
	setPreferredSize(size);
	setMaximumSize(size);
	setBackground(defaultColor);
	chooser=new JColorChooser();
	addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent ae)
		{
		    final ColorChoiceButton button=(ColorChoiceButton) ae.getSource();
		    chooser.setColor(button.getColor());
		    ActionListener anotherListener=new ActionListener() 
			{
			    public void actionPerformed(ActionEvent ae2) 
			    {
				button.setColor(ColorChoiceButton.this.chooser.getColor());
			    }
			};
		    JDialog dialog=JColorChooser.createDialog(button, 
							      button.getColorChooserDialogTitle(),
							      true,
							      ColorChoiceButton.this.chooser,
							      anotherListener,
							      null);
		    dialog.setLocationRelativeTo(button);
		    dialog.show();
		}
	    });
    }

    public ColorChoiceButton(Color c)
    {
	this(c, new Dimension(63, 28));
    }

    public ColorChoiceButton()
    {
	this(Color.white, new Dimension(63, 28));
    }

    public final Color getColor()
    {
	return getBackground();
    }

    public final void setColor(Color c)
    {
	Color oldColor=getColor();
	setBackground(c);
	fireStateChanged();
	firePropertyChange(COLOR_PROPERTY,
			    oldColor, 
			    c);
    }

    public String getColorChooserDialogTitle()
    {
	return dialogTitle;
    }

    public void setColorChooserDialogTitle(String dialogTitle)
    {
	this.dialogTitle=dialogTitle;
    }

    public static void main(String[] args)
    {
	javax.swing.JFrame frame=new javax.swing.JFrame();
	javax.swing.JPanel panel=new javax.swing.JPanel();
	panel.setLayout(new java.awt.FlowLayout());
	panel.add(new ColorChoiceButton(Color.green));
	panel.setPreferredSize(new Dimension(300, 300));
	frame.setContentPane(panel);
	frame.addWindowListener(new java.awt.event.WindowAdapter()
	    {
		public void windowClosing(java.awt.event.WindowEvent woo)
		{
		    System.exit(0);
		}
	    });
	frame.pack();
	frame.setVisible(true);
    }
}

/* $Log: ColorChoiceButton.java,v $
/* Revision 1.2  2001/02/06 00:11:18  smulloni
/* struggle, perhaps futile, with the TextEditorCustomizer and other implicated
/* classes
/*
/* Revision 1.1  2001/02/02 23:30:33  smulloni
/* adding customization features to the text editor.
/* */
