package org.skunk.swing.text;

import javax.swing.text.*;

/**
 * an EntryFilter that accepts only positive integers within the given range
 */
public class PositiveIntegerEntryFilter implements EntryFilter
{
    private int min, max;

    /**
     * constructs a filter with the given minimum and maximum values.
     * @param min the minimum accepted value
     * @param max the maximum accepted value
     */
    public PositiveIntegerEntryFilter(int min, int max)
    {
	this.min=min;
	this.max=max;
    }

    public boolean accepts(int offs, String str, AttributeSet a, String wholeString)
    {
 	try 
 	{
	    //check for any unacceptable characters that may have been pasted
	    char[] charArray=str.toCharArray();
	    for (int i=0;i<charArray.length;i++)
	    {
		char c=charArray[i];
		if (c<'0' || c>'9')
		    return false;
	    }
	    //this takes care of typed characters
	    StringBuffer sb=new StringBuffer(wholeString);
	    sb.insert(offs, str);
	    String newString=sb.toString();
	    if (newString.startsWith("0") && newString.length()>1)
		return false;
	    int parsed=Integer.parseInt(sb.toString());	    	    
	    if (parsed<min || parsed>max)
		return false;
	}
	catch (NumberFormatException nafta)
	{
	    return false;
	}
	return true;
    }
}

/* $Log: PositiveIntegerEntryFilter.java,v $
/* Revision 1.3  2001/01/04 06:02:49  smulloni
/* added more javadoc documentation.
/*
/* Revision 1.2  2000/11/09 23:35:14  smullyan
/* log added to every Java file, with the help of python.  Lock stealing
/* implemented, and treatment of locks made more robust.
/* */
