/*
 *  Copyright (c) 2001, Jacob Smullyan.
 *
 *  This is part of SkunkDAV, a WebDAV client.  See http://skunkdav.sourceforge.net/ 
 *  for the latest version.
 * 
 *  SkunkDAV is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 * 
 *  SkunkDAV is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with SkunkDAV; see the file COPYING.  If not, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
*/


package org.skunk.swing.text.syntax;

import java.io.IOException;
import java.io.StringReader;
import org.skunk.trace.Debug;

%%
%public
%class STMLFlexScanner
%extends FlexScanner

%unicode
%pack

%char

%type int

%{
  private boolean inScript=false;
  private boolean open=false;
  private boolean inComment=false;
  private StringBuffer scriptBuffer=new StringBuffer();
  private boolean inLanguage=false;
  private String scriptLanguage="javascript";

  public void scan() throws IOException
  {
     while ( !yy_atEOF )	
     {
       yylex();
     }
  }

  private int applyStyle(int type) 
  {
     return applyStyle(type, yychar, yylength());
  }
%}

CommentStart = \<\!--
CommentEnd = --\>
OpenTagStart = \<
CloseTagStart = \<\/
TagEnd = \>
OpenAEDTagStart = \<:
CloseAEDTagStart = \<:\/
AEDTagEnd = :\>
BackTick = \`[^\r\n]*\`
AEDCommentStart=\<:(comment|#):\>
AEDCommentEnd=\<:\/(comment|#):\>
AEDExtremeCommentStart=\<:\*
AEDExtremeCommentEnd=\*:\>

CharacterEntity = \&[:jletterdigit:]+;
StringLiteral=(([\"][^\"]*[\"])|([\'][^\']*[\']))
DocTagStart=("<!")
Doctype=([Dd][Oo][Cc][Tt][Yy][Pp][Ee])
DoctypeText=(([^\>\"\']|{StringLiteral})*)
DoctypeTag=({DocTagStart}{Doctype}{DoctypeText}{TagEnd})

NameOrAttribute=(([^\ \r\n\f\t\<\>\=])*)
Identifier=[:jletter:]([:jletterdigit:]|_)*

ScriptEnd=\<\/[Ss][Cc][Rr][Ii][Pp][Tt]

%state TAG_STATE, TAG_ATTRIBUTE_STATE, TAG_VALUE_STATE, COMMENT_STATE, SCRIPT_STATE, 
%state AED_TAG_STATE, AED_TAG_ATTRIBUTE_STATE, AED_EXTREME_COMMENT_STATE, AED_COMMENT_STATE

%%

<YYINITIAL> {
  {CharacterEntity}     {       
				return applyStyle(SyntaxStyle.TAG3);  
			}
 {AEDExtremeCommentStart} { 
				yybegin(AED_EXTREME_COMMENT_STATE);
				spanStart=yychar;
			}

  {AEDCommentStart}  	{
				yybegin(AED_COMMENT_STATE);
				spanStart=yychar;
			}


  {CloseAEDTagStart}    {
				yybegin(AED_TAG_STATE);
				spanStart=yychar;
				open=false;
				return applyStyle(SyntaxStyle.SEPARATOR1);
			}

  {OpenAEDTagStart}/[^\/\*] {
				yybegin(AED_TAG_STATE);
				spanStart=yychar;
				open=true;
				return applyStyle(SyntaxStyle.SEPARATOR1);
			}

  {AEDTagEnd}           {
				return applyStyle(SyntaxStyle.SEPARATOR1);
			}

  {OpenTagStart}/[^/:]	{ 
			  	yybegin(TAG_STATE);
                          	spanStart=yychar;
				open=true;
                          	return applyStyle(SyntaxStyle.SEPARATOR2);
			}
  {CloseTagStart}	{ 
			  	yybegin(TAG_STATE);
                          	spanStart=yychar;
				open=false;
                          	return applyStyle(SyntaxStyle.SEPARATOR2);
			}
  {TagEnd}		{
				if (inScript) 
                                {
                                   scriptBuffer.setLength(0);
				   spanStart=yychar+yylength();
                                    yybegin(SCRIPT_STATE);
           
                                }
				return applyStyle(SyntaxStyle.SEPARATOR2);
			}
 

  {CommentStart}        {
				yybegin(COMMENT_STATE);
				spanStart=yychar;
				return applyStyle(SyntaxStyle.COMMENT2);
			}

  {DoctypeTag}          {
				return applyStyle(SyntaxStyle.LITERAL2);
			}
}

<AED_TAG_STATE> {

  {AEDTagEnd}           {
				yybegin(YYINITIAL);
				open=false;
				return applyStyle(SyntaxStyle.ERROR);
			}
  "val"			|
  "set"			|
  "call"		|
  "spool"		|
  "args"		|
  "compargs"		|
  "default"		|
  "date"		|
  "del"			|
  "if"			|
  "elif"		|
  "else"		|
  "for"			|
  "loop"		|
  "while"		|
  "break"		|
  "continue"		|
  "try"			|
  "except"		|
  "else"		|
  "finally"		|
  "raise"		|
  "halt"		|
  "include"		|
  "component"		|
  "datacomp"		|
  "return"		|
  "type"		|
  "cache"		|
  "import"		|
  "url"			|
  "curl"		|
  "img"			|
  "form"		|
  "hidden"		|
  "retain"		|
  "redirect"		|
  "sendmail"		|
  "catalog"		|
  "multicatalog"	|
  "msg"			|
  "doc"			|
  "brandcomponent"	|
  "brandinclude"	|
  "branddatacomponent"	|
  "log"                 |
  "debug"               |
  "error"               |
  "warn"                {	
				yybegin(AED_TAG_ATTRIBUTE_STATE);				
				return applyStyle(SyntaxStyle.TAG3);
      			}
}

<AED_TAG_ATTRIBUTE_STATE> {
  {AEDTagEnd}           {
				open=false;
				yypushback(yylength());
				yybegin(YYINITIAL);
			}
  "="                   {       
				return applyStyle(SyntaxStyle.OPERATOR);
			}
  {BackTick}            {
				String s=yytext();
				boolean complete=s.endsWith("`");
				//remove backticks
				s=s.substring(1, (complete) ? s.length()-1 : s.length());
				applyStyle(SyntaxStyle.INCLUDE, yychar, 1);
				
				FlexScanner scanner=Flexicizer.getScanner(FileMode.getMode("python"), 
									  new StringReader(s));
				scanner.setStyleBuffer(this.styleBuffer);
				scanner.setOffset(offset+yychar+1);
				scanner.scan();
				if (complete) return applyStyle(SyntaxStyle.INCLUDE, yychar+s.length()+1, 1);
			}

  {Identifier}		{       return applyStyle(SyntaxStyle.TAG4);  }
}

<TAG_STATE> {
  {NameOrAttribute}	{	
				yybegin(TAG_ATTRIBUTE_STATE);
				inScript=open && yytext().trim().equalsIgnoreCase("script");
				return applyStyle(SyntaxStyle.TAG1);
      			}
  {TagEnd}              {
				yybegin(YYINITIAL);
				open=false;
				return applyStyle(SyntaxStyle.ERROR);
			}
}

<TAG_ATTRIBUTE_STATE> {
  {TagEnd}              {
				open=false;
				yypushback(yylength());
				yybegin(YYINITIAL);
			}
  "="                   {       
				yybegin(TAG_VALUE_STATE);
				return applyStyle(SyntaxStyle.OPERATOR);
			}
  {NameOrAttribute}     { 
			    inLanguage=inScript && yytext().equalsIgnoreCase("language");  
			    return applyStyle(SyntaxStyle.TAG2);  
                        }
}

<TAG_VALUE_STATE> {
  {NameOrAttribute}|{StringLiteral} {
                                if (inLanguage)
                                {
	                          scriptLanguage=yytext().toLowerCase();
				  if (scriptLanguage.startsWith("\"")
                                      || scriptLanguage.startsWith("'"))
                                      scriptLanguage=scriptLanguage.substring(1, Math.max(1, scriptLanguage.length()-1));
                                  
				  if (scriptLanguage.startsWith("javascript") 
                                      || scriptLanguage.equals("jscript")
                                      || scriptLanguage.equals("ecmascript"))
                                      scriptLanguage="javascript";
                                }
                                else scriptLanguage="javascript";
				yybegin(TAG_ATTRIBUTE_STATE);
				return applyStyle(SyntaxStyle.LITERAL1);
			}
  {TagEnd}		{
				yybegin(YYINITIAL);
				open=false;
				return applyStyle(SyntaxStyle.ERROR);
			}
}

<COMMENT_STATE> {
  {CommentEnd}		{
				yybegin(YYINITIAL);
				return applyStyle(SyntaxStyle.COMMENT3,
					spanStart,
					yylength()+yychar-spanStart);
			}

}

<AED_EXTREME_COMMENT_STATE> {
  {AEDExtremeCommentEnd} {
				yybegin(YYINITIAL);
				return applyStyle(SyntaxStyle.COMMENT4,
					spanStart,
					yylength()+yychar-spanStart);
			}

}

<AED_COMMENT_STATE> {
  {AEDCommentEnd} 	{
				yybegin(YYINITIAL);
				return applyStyle(SyntaxStyle.COMMENT1,
					spanStart,
					yylength()+yychar-spanStart);
			}

}

<SCRIPT_STATE> {
  
  {ScriptEnd}         	{
				inScript=false;
				String s=scriptBuffer.toString();
				FlexScanner scanner=Flexicizer.getScanner(FileMode.getMode(scriptLanguage), 
									  new StringReader(s));
                                if (scanner!=null) 
				{ 
				   scanner.setStyleBuffer(this.styleBuffer);
				   scanner.setOffset(offset+spanStart);
                                   scanner.scan();
                                }
				
				yybegin(YYINITIAL);			
				yypushback(yylength());
	
			}
  .			{ 
	                        scriptBuffer.append(yytext());
                        }
  \n                    {      scriptBuffer.append("\n"); }
  
}


/* error fallback */
.|\n                             { return applyStyle(SyntaxStyle.DEFAULT); }

<<EOF>>	                         { return -1;}

